@@ -1,10 +1,11 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from trytond.const import CONTEXT_CACHE_SIZE, MODEL_CACHE_SIZE
+from trytond.const import MODEL_CACHE_SIZE
 
 DatabaseIntegrityError = None
 DatabaseOperationalError = None
 
+
 class DatabaseInterface(object):
     '''
     Define generic interface for database connection
@@ -104,8 +105,7 @@ class CursorInterface(object):
     IN_MAX = 1000
 
     def __init__(self):
-        from trytond.cache import LRUDict
-        self.cache = LRUDict(CONTEXT_CACHE_SIZE)
+        self.cache = {}
 
     def get_cache(self, context=None):
         '''
