@@ -2,7 +2,6 @@
 #this repository contains the full copyright notices and license terms.
 from trytond.backend.database import DatabaseInterface, CursorInterface
 from trytond.config import CONFIG
-from trytond.session import Session
 from psycopg2.pool import ThreadedConnectionPool
 from psycopg2.extensions import cursor as PsycopgCursor
 from psycopg2.extensions import ISOLATION_LEVEL_REPEATABLE_READ
@@ -22,7 +21,6 @@ import os
 if os.name == 'posix':
     import pwd
 from decimal import Decimal
-from trytond.protocols.datatype import Float
 
 RE_FROM = re.compile('.* from "?([a-zA-Z_0-9]+)"?.*$')
 RE_INTO = re.compile('.* into "?([a-zA-Z_0-9]+)"?.*$')
@@ -229,13 +227,12 @@ class Database(DatabaseInterface):
         sql_file = os.path.join(os.path.dirname(__file__), 'init.sql')
         with open(sql_file) as fp:
             for line in fp.read().split(';'):
-                if (len(line)>0) and (not line.isspace()):
+                if (len(line) > 0) and (not line.isspace()):
                     cursor.execute(line)
 
-        for i in ('ir', 'workflow', 'res', 'webdav'):
+        for i in ('ir', 'res', 'webdav'):
             root_path = os.path.join(os.path.dirname(__file__), '..', '..')
             tryton_file = os.path.join(root_path, i, '__tryton__.py')
-            mod_path = os.path.join(root_path, i)
             with open(tryton_file) as fp:
                 info = safe_eval(fp.read())
             active = info.get('active', False)
@@ -386,12 +383,6 @@ class Cursor(CursorInterface):
                 "'ir_ui_menu', "
                 "'res_user', "
                 "'res_group', "
-                "'wkf', "
-                "'wkf_activity', "
-                "'wkf_transition', "
-                "'wkf_instance', "
-                "'wkf_workitem', "
-                "'wkf_witm_trans', "
                 "'ir_module_module', "
                 "'ir_module_module_dependency', "
                 "'ir_translation', "
@@ -404,7 +395,8 @@ class Cursor(CursorInterface):
         return self.cursor.fetchone()[0]
 
     def setnextid(self, table, value):
-        self.cursor.execute("SELECT SETVAL('" + table + "_id_seq', %d)" % value)
+        self.cursor.execute("SELECT SETVAL('" + table + "_id_seq', %d)"
+            % value)
 
     def currid(self, table):
         self.cursor.execute('SELECT last_value FROM "' + table + '_id_seq"')
@@ -434,7 +426,5 @@ if PYDATETIME:
     register_type(PYDATETIME)
 if PYTIME:
     register_type(PYTIME)
-register_adapter(Session, AsIs)
 register_adapter(float, lambda value: AsIs(repr(value)))
-register_adapter(Float, lambda value: AsIs(repr(value)))
 register_adapter(Decimal, lambda value: AsIs(str(value)))
