@@ -1,14 +1,14 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 from trytond.backend.database import DatabaseInterface, CursorInterface
 from trytond.config import CONFIG
-from trytond.session import Session
 import os
 import re
 from decimal import Decimal
 import datetime
 import time
+import sys
+import threading
 
 _FIX_ROWCOUNT = False
 try:
@@ -16,7 +16,7 @@ try:
     from pysqlite2.dbapi2 import IntegrityError as DatabaseIntegrityError
     from pysqlite2.dbapi2 import OperationalError as DatabaseOperationalError
     #pysqlite2 < 2.5 doesn't return correct rowcount
-    _FIX_ROWCOUNT = sqlite.version_info < (2 , 5, 0)
+    _FIX_ROWCOUNT = sqlite.version_info < (2, 5, 0)
 except ImportError:
     import sqlite3 as sqlite
     from sqlite3 import IntegrityError as DatabaseIntegrityError
@@ -24,6 +24,7 @@ except ImportError:
 QUOTE_SEPARATION = re.compile(r"(.*?)('.*?')", re.DOTALL)
 EXTRACT_PATTERN = re.compile(r'EXTRACT\s*\(\s*(\S*)\s+FROM', re.I)
 
+
 def extract(lookup_type, date):
     if date is None:
         return None
@@ -64,6 +65,7 @@ def extract(lookup_type, date):
         return date.isocalendar()[1]
     return getattr(date, lookup_type.lower())
 
+
 def date_trunc(_type, date):
     if _type == 'second':
         return date
@@ -79,28 +81,33 @@ def date_trunc(_type, date):
         return "%i-%02i-%02i 00:00:00" % (tm_tuple.tm_year, tm_tuple.tm_mon,
                 tm_tuple.tm_mday)
 
+
 def split_part(text, delimiter, count):
+    if text is None:
+        return None
     return (text.split(delimiter) + [''] * (count - 1))[count - 1]
 
+
 def replace(text, pattern, replacement):
     return str(text).replace(pattern, replacement)
 
 
 class Database(DatabaseInterface):
 
-    _memory_database = None
+    _local = threading.local()
     _conn = None
 
     def __new__(cls, database_name=':memory:'):
-        if database_name == ':memory:' \
-                and cls._memory_database:
-            return cls._memory_database
+        if (database_name == ':memory:'
+                and hasattr(cls._local, 'memory_database')
+                and cls._local.memory_database):
+            return cls._local.memory_database
         return DatabaseInterface.__new__(cls, database_name=database_name)
 
     def __init__(self, database_name=':memory:'):
         super(Database, self).__init__(database_name=database_name)
         if database_name == ':memory:':
-            Database._memory_database = self
+            Database._local.memory_database = self
 
     def connect(self):
         if self.database_name == ':memory:':
@@ -120,7 +127,7 @@ class Database(DatabaseInterface):
             self._conn.create_function('replace', 3, replace)
         return self
 
-    def cursor(self, autocommit=False):
+    def cursor(self, autocommit=False, readonly=False):
         if self._conn is None:
             self.connect()
         if autocommit:
@@ -144,10 +151,9 @@ class Database(DatabaseInterface):
                 return
             path = os.path.join(CONFIG['data_path'],
                     database_name + '.sqlite')
-        conn = sqlite.connect(path)
-        cursor = conn.cursor()
-        cursor.close()
-        conn.close()
+        with sqlite.connect(path) as conn:
+            cursor = conn.cursor()
+            cursor.close()
 
     def drop(self, cursor, database_name):
         if database_name == ':memory:':
@@ -213,13 +219,12 @@ class Database(DatabaseInterface):
         sql_file = os.path.join(os.path.dirname(__file__), 'init.sql')
         with open(sql_file) as fp:
             for line in fp.read().split(';'):
-                if (len(line)>0) and (not line.isspace()):
+                if (len(line) > 0) and (not line.isspace()):
                     cursor.execute(line)
 
-        for i in ('ir', 'workflow', 'res', 'webdav'):
+        for i in ('ir', 'res', 'webdav'):
             root_path = os.path.join(os.path.dirname(__file__), '..', '..')
             tryton_file = os.path.join(root_path, i, '__tryton__.py')
-            mod_path = os.path.join(root_path, i)
             with open(tryton_file) as fp:
                 info = safe_eval(fp.read())
             active = info.get('active', False)
@@ -273,7 +278,7 @@ class Cursor(CursorInterface):
         super(Cursor, self).__init__()
         self._conn = conn
         self.database_name = database_name
-        self.dbname = self.database_name #XXX to remove
+        self.dbname = self.database_name  # XXX to remove
         self.cursor = conn.cursor(_Cursor)
 
     def __getattr__(self, name):
@@ -283,7 +288,7 @@ class Cursor(CursorInterface):
 
     def execute(self, sql, params=None):
         buf = ""
-        for nquote, quote in QUOTE_SEPARATION.findall(sql+"''"):
+        for nquote, quote in QUOTE_SEPARATION.findall(sql + "''"):
             nquote = nquote.replace('?', '??')
             nquote = nquote.replace('%s', '?')
             nquote = nquote.replace('ilike', 'like')
@@ -292,8 +297,7 @@ class Cursor(CursorInterface):
         sql = buf[:-2]
         try:
             if params:
-                res = self.cursor.execute(sql, [isinstance(x, str) and \
-                        unicode(x, 'utf-8') or x for x in  params])
+                res = self.cursor.execute(sql, params)
             else:
                 res = self.cursor.execute(sql)
         except Exception:
@@ -323,12 +327,6 @@ class Cursor(CursorInterface):
                     "'ir_ui_menu', "
                     "'res_user', "
                     "'res_group', "
-                    "'wkf', "
-                    "'wkf_activity', "
-                    "'wkf_transition', "
-                    "'wkf_instance', "
-                    "'wkf_workitem', "
-                    "'wkf_witm_trans', "
                     "'ir_module_module', "
                     "'ir_module_module_dependency', "
                     "'ir_translation', "
@@ -358,8 +356,15 @@ class Cursor(CursorInterface):
         return select
 
 sqlite.register_converter('NUMERIC', lambda val: Decimal(val))
-sqlite.register_adapter(Decimal, lambda val: buffer(str(val)))
-sqlite.register_adapter(Session, int)
+if sys.version_info[0] == 2:
+    sqlite.register_adapter(Decimal, lambda val: buffer(str(val)))
+else:
+    sqlite.register_adapter(Decimal, lambda val: bytes(str(val)))
+
+
 def adapt_datetime(val):
     return val.replace(tzinfo=None).isoformat(" ")
 sqlite.register_adapter(datetime.datetime, adapt_datetime)
+sqlite.register_adapter(datetime.time, lambda val: val.isoformat())
+sqlite.register_converter('TIME', lambda val: datetime.time(*map(int,
+            val.split(':'))))
