@@ -1,6 +1,18 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-import ConfigParser, os, sys
+import sys
+try:
+    import cdecimal
+    # Use cdecimal globally
+    if 'decimal' not in sys.modules:
+        sys.modules['decimal'] = cdecimal
+except ImportError:
+    import decimal
+    sys.modules['cdecimal'] = decimal
+import os
+import ConfigParser
+import time
+
 
 def get_hostname(netloc):
     if '[' in netloc and ']' in netloc:
@@ -10,6 +22,7 @@ def get_hostname(netloc):
     else:
         return netloc
 
+
 def get_port(netloc, protocol):
     netloc = netloc.split(']')[-1]
     if ':' in netloc:
@@ -21,17 +34,19 @@ def get_port(netloc, protocol):
             'webdav': 8080,
         }.get(protocol)
 
+
 class ConfigManager(object):
     def __init__(self, fname=None):
         self.options = {
-            'hostname': None,
             'jsonrpc': [('localhost', 8000)],
             'ssl_jsonrpc': False,
+            'hostname_jsonrpc': None,
             'xmlrpc': [],
             'ssl_xmlrpc': False,
             'jsondata_path': '/var/www/localhost/tryton',
             'webdav': [],
             'ssl_webdav': False,
+            'hostname_webdav': None,
             'db_type': 'postgresql',
             'db_host': False,
             'db_port': False,
@@ -65,13 +80,21 @@ class ConfigManager(object):
             'cron': True,
             'unoconv': 'pipe,name=trytond;urp;StarOffice.ComponentContext',
             'retry': 5,
+            'language': 'en_US',
+            'timezone': time.tzname[0] or time.tzname[1],
         }
         self.configfile = None
 
+    def set_timezone(self):
+        os.environ['TZ'] = self.get('timezone')
+        if hasattr(time, 'tzset'):
+            time.tzset()
+
     def update_cmdline(self, cmdline_options):
         self.options.update(cmdline_options)
 
-        # Verify that we want to log or not, if not the output will go to stdout
+        # Verify that we want to log or not,
+        # if not the output will go to stdout
         if self.options['logfile'] in ('None', 'False'):
             self.options['logfile'] = False
         # the same for the pidfile
@@ -104,7 +127,7 @@ class ConfigManager(object):
                 value = True
             if value == 'False' or value == 'false':
                 value = False
-            if name in ('xmlrpc', 'jsonrpc', 'webdav'):
+            if name in ('xmlrpc', 'jsonrpc', 'webdav') and value:
                 value = [(get_hostname(netloc).replace('*', ''),
                     get_port(netloc, name)) for netloc in value.split(',')]
             self.options[name] = value
