@@ -15,6 +15,7 @@ from trytond.transaction import Transaction
 CDATA_START = re.compile('^\s*\<\!\[cdata\[', re.IGNORECASE)
 CDATA_END = re.compile('\]\]\>\s*$', re.IGNORECASE)
 
+
 class DummyTagHandler:
     """Dubhandler implementing empty methods. Will be used when whe
     want to ignore the xml content"""
@@ -55,7 +56,7 @@ class MenuitemTagHandler:
         if values.get('parent'):
             values['parent'] = self.mh.get_id(values['parent'])
 
-        action_name = False
+        action_name = None
         if values.get('action'):
             action_id = self.mh.get_id(values['action'])
 
@@ -63,9 +64,9 @@ class MenuitemTagHandler:
             cursor.execute(cursor.limit_clause(
             "SELECT a.name, a.type, v.type, v.field_childs, icon.name " \
             "FROM ir_action a " \
-                "LEFT JOIN ir_action_report report ON (a.id = report.action) " \
+                "LEFT JOIN ir_action_report report ON (a.id = report.action) "\
                 "LEFT JOIN ir_action_act_window act ON (a.id = act.action) " \
-                "LEFT JOIN ir_action_wizard wizard ON (a.id = wizard.action) " \
+                "LEFT JOIN ir_action_wizard wizard ON (a.id = wizard.action) "\
                 "LEFT JOIN ir_action_url url ON (a.id = url.action) " \
                 "LEFT JOIN ir_action_act_window_view wv ON " \
                     "(act.id = wv.act_window) " \
@@ -128,12 +129,11 @@ class MenuitemTagHandler:
         if name != "menuitem":
             return self
         else:
-            self.mh.import_record( 'ir.ui.menu', self.values, self.xml_id)
+            self.mh.import_record('ir.ui.menu', self.values, self.xml_id)
             return None
 
     def current_state(self):
-        return "Tag menuitem with id: %s"% \
-               (self.xml_id)
+        return "Tag menuitem with id: %s" % self.xml_id
 
 
 class RecordTagHandler:
@@ -153,7 +153,6 @@ class RecordTagHandler:
         self.cdata = None
         self.start_cdata = None
 
-
     def startElement(self, name, attributes):
 
         # Manage the top level tag
@@ -187,7 +186,7 @@ class RecordTagHandler:
                 self.cdata = "start"
 
             # Catch the known attributes
-            search_attr = attributes.get('search','')
+            search_attr = attributes.get('search', '')
             ref_attr = attributes.get('ref', '')
             eval_attr = attributes.get('eval', '')
 
@@ -226,7 +225,6 @@ class RecordTagHandler:
 
         self.values[self.current_field] += data
 
-
     def endElement(self, name):
 
         """Must return the object to use for the next call, if name is
@@ -261,10 +259,10 @@ class RecordTagHandler:
             self.xml_ids.append(self.xml_id)
             return None
         else:
-            raise Exception("Unexpected closing tag '%s'"% (name,))
+            raise Exception("Unexpected closing tag '%s'" % (name,))
 
     def current_state(self):
-        return "In tag record: model %s with id %s."% \
+        return "In tag record: model %s with id %s." % \
                (self.model and self.model._name or "?", self.xml_id)
 
 
@@ -275,6 +273,7 @@ class Unhandled_field(Exception):
     """
     pass
 
+
 class Fs2bdAccessor:
     """
     Used in TrytondXmlHandler.
@@ -342,7 +341,7 @@ class Fs2bdAccessor:
         self.fs2db[module] = {}
         module_data_ids = self.modeldata_obj.search([
                 ('module', '=', module),
-                ('inherit', '=', False),
+                ('inherit', '=', None),
                 ], order=[('db_id', 'ASC')])
 
         record_ids = {}
@@ -354,8 +353,12 @@ class Fs2bdAccessor:
             record_ids.setdefault(rec.model, [])
             record_ids[rec.model].append(rec.db_id)
 
+        object_name_list = set(self.pool.object_name_list())
+
         self.browserecord[module] = {}
         for model_name in record_ids.keys():
+            if model_name not in object_name_list:
+                continue
             model_obj = self.pool.get(model_name)
             self.browserecord[module][model_name] = {}
             for i in range(0, len(record_ids[model_name]), cursor.IN_MAX):
@@ -364,7 +367,8 @@ class Fs2bdAccessor:
                     ids = model_obj.search([
                         ('id', 'in', sub_record_ids),
                         ])
-                models = model_obj.browse(ids)
+                with Transaction().set_context(language='en_US'):
+                    models = model_obj.browse(ids)
                 for model in models:
                     self.browserecord[module][model_name][model.id] = model
         self.fetched_modules.append(module)
@@ -400,7 +404,6 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
         self.sax_parser.setFeature(sax.handler.feature_namespaces, 0)
         self.sax_parser.setContentHandler(self)
 
-
     def parse_xmlstream(self, stream):
         """
         Take a byte stream has input and parse the xml content.
@@ -442,8 +445,8 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                 pass
 
             else:
-                logging.getLogger("convert").info( "Tag %s not supported" %
-                        name)
+                logging.getLogger("convert").info("Tag %s not supported" %
+                    name)
                 return
         elif not self.skip_data:
             self.taghandler.startElement(name, attributes)
@@ -473,12 +476,10 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
             module = self.module
 
         if self.fs2db.get(module, xml_id) is None:
-            raise Exception("Reference to %s not found"% \
-                                ".".join([module,xml_id]))
+            raise Exception("Reference to %s not found" % \
+                ".".join([module, xml_id]))
         return self.fs2db.get(module, xml_id)["db_id"]
 
-
-
     @staticmethod
     def _clean_value(key, browse_record, object_ref):
         """
@@ -495,10 +496,10 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
 
         # handle the value regarding to the type
         if field_type == 'many2one':
-            return browse_record[key] and browse_record[key].id or False
+            return browse_record[key] and browse_record[key].id or None
         elif field_type == 'reference':
             if not browse_record[key]:
-                return False
+                return None
             ref_mode, ref_id = browse_record[key].split(',', 1)
             try:
                 ref_id = safe_eval(ref_id)
@@ -512,7 +513,6 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
         else:
             return browse_record[key]
 
-
     def populate_to_delete(self):
         """Create a list of all the records that whe should met in the update
         process. The records that are not encountered are deleted from the
@@ -522,13 +522,12 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
         # problem when the corresponding recordds will be deleted:
         module_data_ids = self.modeldata_obj.search([
                 ('module', '=', self.module),
-                ('inherit', '=', False),
+                ('inherit', '=', None),
                 ], order=[('id', 'DESC')])
         return set(rec.fs_id for rec in self.modeldata_obj.browse(
             module_data_ids))
 
     def import_record(self, model, values, fs_id):
-        cursor = Transaction().cursor
         module = self.module
 
         if not fs_id:
@@ -546,7 +545,6 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                         (module, fs_id))
 
         object_ref = self.pool.get(model)
-        translation_obj = self.pool.get('ir.translation')
 
         if self.fs2db.get(module, fs_id):
 
@@ -562,7 +560,7 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
             # XXX maybe use only one call to get()
             db_id, db_model, mdata_id, old_values = \
                     [self.fs2db.get(module, fs_id)[x] for x in \
-                    ["db_id","model","id","values"]]
+                    ["db_id", "model", "id", "values"]]
             inherit_db_ids = {}
             inherit_mdata_ids = []
 
@@ -584,8 +582,10 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                 "data with the wrong model: %s (module: %s)" % (fs_id, module))
 
             #Re-create record if it was deleted
-            if not self.fs2db.get_browserecord(module, object_ref._name, db_id):
-                with Transaction().set_context(module=module):
+            if not self.fs2db.get_browserecord(
+                    module, object_ref._name, db_id):
+                with Transaction().set_context(
+                        module=module, language='en_US'):
                     db_id = object_ref.create(values)
 
                 # reset_browsercord
@@ -597,46 +597,6 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                         object_ref._inherit_fields.values():
                     inherit_db_ids[table] = record[field_name].id
 
-                #Add a translation record for field translatable
-                for field_name in object_ref._columns.keys() + \
-                        object_ref._inherit_fields.keys():
-                    if field_name in object_ref._columns:
-                        field = object_ref._columns[field_name]
-                        table_name = object_ref._name
-                        res_id = db_id
-                    else:
-                        field = object_ref._inherit_fields[field_name][2]
-                        table_name = self.pool.get(
-                                object_ref._inherit_fields[field_name][0])._name
-                        res_id = inherit_db_ids[table_name]
-                    if getattr(field, 'translate', False) and \
-                            values.get(field_name):
-                        cursor.execute('SELECT id FROM ir_translation ' \
-                                'WHERE name = %s ' \
-                                    'AND lang = %s ' \
-                                    'AND type = %s ' \
-                                    'AND res_id = %s ' \
-                                    'AND module = %s',
-                                (table_name + ',' + field_name,
-                                    'en_US', 'model', res_id, module))
-                        fetchone = cursor.fetchone()
-                        src_md5 = translation_obj.get_src_md5(
-                            values[field_name])
-                        if fetchone:
-                            trans_id = fetchone[0]
-                            cursor.execute('UPDATE ir_translation '
-                                'SET src = %s, src_md5 = %s, module = %s '
-                                'WHERE id = %s',
-                                (values[field_name], src_md5, module, trans_id))
-                        else:
-                            cursor.execute('INSERT INTO ir_translation '
-                                '(name, lang, type, src, src_md5, res_id, '
-                                    'value, module, fuzzy) '
-                                'VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s)',
-                                (table_name + ',' + field_name, 'en_US',
-                                    'model', values[field_name], src_md5,
-                                    res_id, '', module, False))
-
                 for table in inherit_db_ids.keys():
                     data_id = self.modeldata_obj.search([
                         ('fs_id', '=', fs_id),
@@ -716,7 +676,7 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                 if db_field != expected_value and (db_field or expected_value):
                     logging.getLogger("convert").warning(
                         "Field %s of %s@%s not updated (id: %s), because "\
-                        "it has changed since the last update"% \
+                        "it has changed since the last update" % \
                         (key, db_id, model, fs_id))
                     continue
 
@@ -727,11 +687,11 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
             # if there is values to update:
             if to_update:
                 # write the values in the db:
-                with Transaction().set_context(module=module):
+                with Transaction().set_context(
+                        module=module, language='en_US'):
                     object_ref.write(db_id, to_update)
                 self.fs2db.reset_browsercord(module, object_ref._name, db_id)
 
-
             if not inherit_db_ids:
                 record = object_ref.browse(db_id)
                 for table, field_name, field in \
@@ -746,50 +706,6 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                         ], limit=1)
                     inherit_mdata_ids.append((table, data_id))
 
-            #Update/Create translation record for field translatable
-            if to_update:
-                for field_name in object_ref._columns.keys() + \
-                        object_ref._inherit_fields.keys():
-                    if field_name in object_ref._columns:
-                        field = object_ref._columns[field_name]
-                        table_name = object_ref._name
-                        res_id = db_id
-                    else:
-                        field = object_ref._inherit_fields[field_name][2]
-                        table_name = self.pool.get(
-                                object_ref._inherit_fields[field_name][0])._name
-                        res_id = inherit_db_ids[table_name]
-                    if getattr(field, 'translate', False):
-                        cursor.execute('SELECT id FROM ir_translation ' \
-                                'WHERE name = %s ' \
-                                    'AND lang = %s ' \
-                                    'AND type = %s ' \
-                                    'AND res_id = %s ' \
-                                    'AND module = %s',
-                                (table_name + ',' + field_name,
-                                    'en_US', 'model', res_id, module))
-                        fetchone = cursor.fetchone()
-                        if fetchone:
-                            if to_update.get(field_name):
-                                src_md5 = translation_obj.get_src_md5(
-                                    to_update[field_name])
-                                trans_id = fetchone[0]
-                                cursor.execute('UPDATE ir_translation '
-                                    'SET src = %s, src_md5 = %s, module = %s '
-                                    'WHERE id = %s',
-                                    (to_update[field_name], src_md5, module,
-                                        trans_id))
-                        elif values.get(field_name):
-                            src_md5 = translation_obj.get_src_md5(
-                                values[field_name])
-                            cursor.execute('INSERT INTO ir_translation '
-                                '(name, lang, type, src, src_md5, res_id, '
-                                        'value, module, fuzzy) '
-                                'VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s)',
-                                (table_name + ',' + field_name, 'en_US',
-                                    'model', values[field_name], src_md5,
-                                    res_id, '', module, False))
-
             if to_update:
                 # re-read it: this ensure that we store the real value
                 # in the model_data table:
@@ -798,7 +714,7 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                 if not db_val:
                     db_val = object_ref.browse(db_id)
                 for key in to_update:
-                    values[key] = self._clean_value( key, db_val, object_ref)
+                    values[key] = self._clean_value(key, db_val, object_ref)
 
             if module != self.module:
                 temp_values = old_values.copy()
@@ -811,7 +727,7 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                     'model': model,
                     'module': module,
                     'db_id': db_id,
-                    'values': values,
+                    'values': str(values),
                     'date_update': datetime.datetime.now(),
                     })
                 for table, inherit_mdata_id in inherit_mdata_ids:
@@ -820,7 +736,7 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                         'model': table,
                         'module': module,
                         'db_id': inherit_db_ids[table],
-                        'values': values,
+                        'values': str(values),
                         'date_update': datetime.datetime.now(),
                         })
             # reset_browsercord to keep cache memory low
@@ -828,59 +744,15 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
 
         else:
             # this record is new, create it in the db:
-            with Transaction().set_context(module=module):
+            with Transaction().set_context(module=module, language='en_US'):
                 db_id = object_ref.create(values)
             inherit_db_ids = {}
 
             record = object_ref.browse(db_id)
-            for table, field_name, field in object_ref._inherit_fields.values():
+            for table, field_name, field in (
+                    object_ref._inherit_fields.values()):
                 inherit_db_ids[table] = record[field_name].id
 
-            #Add a translation record for field translatable
-            for field_name in object_ref._columns.keys() + \
-                    object_ref._inherit_fields.keys():
-                if field_name in object_ref._columns:
-                    field = object_ref._columns[field_name]
-                    table_name = object_ref._name
-                    res_id = db_id
-                else:
-                    field = object_ref._inherit_fields[field_name][2]
-                    table_name = self.pool.get(
-                            object_ref._inherit_fields[field_name][0])._name
-                    res_id = inherit_db_ids[table_name]
-                if getattr(field, 'translate', False) and \
-                        values.get(field_name):
-                    cursor.execute('SELECT id FROM ir_translation ' \
-                            'WHERE name = %s' \
-                                'AND lang = %s ' \
-                                'AND type = %s ' \
-                                'AND res_id = %s ' \
-                                'AND module = %s',
-                            (table_name + ',' + field_name,
-                                'en_US', 'model', res_id, module))
-                    trans_id = None
-                    if cursor.rowcount == -1 or cursor.rowcount is None:
-                        data = cursor.fetchone()
-                        if data:
-                            trans_id, = data
-                    elif cursor.rowcount != 0:
-                        trans_id, = cursor.fetchone()
-                    src_md5 = translation_obj.get_src_md5(values[field_name]
-                        or '')
-                    if trans_id:
-                        cursor.execute('UPDATE ir_translation '
-                            'SET src = %s, src_md5 = %s, module = %s '
-                            'WHERE id = %s',
-                            (values[field_name], src_md5, module, trans_id))
-                    else:
-                        cursor.execute('INSERT INTO ir_translation '
-                            '(name, lang, type, src, src_md5, res_id, '
-                                'value, module, fuzzy) '
-                            'VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s)',
-                            (table_name + ',' + field_name, 'en_US', 'model',
-                                values[field_name], src_md5, res_id, '',
-                                module, False))
-
             # re-read it: this ensure that we store the real value
             # in the model_data table:
             db_val = object_ref.browse(db_id)
@@ -913,6 +785,7 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                     "id": mdata_id, "values": str(values)})
             self.fs2db.reset_browsercord(module, model, db_id)
 
+
 def post_import(pool, module, to_delete):
     """
     Remove the records that are given in to_delete.
@@ -920,7 +793,6 @@ def post_import(pool, module, to_delete):
     cursor = Transaction().cursor
     mdata_delete = []
     modeldata_obj = pool.get("ir.model.data")
-    transition_delete = []
 
     with Transaction().set_context(active_test=False):
         mdata_ids = modeldata_obj.search([
@@ -928,63 +800,26 @@ def post_import(pool, module, to_delete):
             ('module', '=', module),
             ], order=[('id', 'DESC')])
 
+    object_name_list = set(pool.object_name_list())
     for mrec in modeldata_obj.browse(mdata_ids):
         mdata_id, model, db_id = mrec.id, mrec.model, mrec.db_id
 
-        # Whe skip transitions, they will be deleted with the
-        # corresponding activity:
-        if model == 'workflow.transition':
-            transition_delete.append((mdata_id, db_id))
-            continue
-
-        if model == 'workflow.activity':
-
-            wkf_todo = []
-            # search for records that are in the state/activity that
-            # we want to delete...
-            cursor.execute('SELECT res_type, res_id ' \
-                    'FROM wkf_instance ' \
-                    'WHERE id IN (' \
-                        'SELECT instance FROM wkf_workitem ' \
-                        'WHERE activity = %s)', (db_id,))
-            #... connect the transitions backward...
-            wkf_todo.extend(cursor.fetchall())
-            cursor.execute("UPDATE wkf_transition " \
-                    'SET condition = \'True\', "group" = NULL, ' \
-                        "signal = NULL, act_to = act_from, " \
-                        "act_from = %s " \
-                    "WHERE act_to = %s", (db_id, db_id))
-            # ... and force the record to follow them:
-            for wkf_model, wkf_model_id in wkf_todo:
-                model_obj = pool.get(wkf_model)
-                #XXX must perhaps use workflow_trigger_trigger?
-                model_obj.workflow_trigger_write(wkf_model_id)
-
-            # Collect the ids of these transition in model_data
-            cursor.execute(
-                "SELECT md.id FROM ir_model_data md " \
-                    "JOIN wkf_transition t ON "\
-                    "(md.model='workflow.transition' and md.db_id=t.id)" \
-                    "WHERE t.act_to = %s", (db_id,))
-            mdata_delete.extend([x[0] for x in cursor.fetchall()])
-
-            # And finally delete the transitions
-            cursor.execute("DELETE FROM wkf_transition " \
-                    "WHERE act_to = %s", (db_id,))
-
-
         logging.getLogger("convert").info(
                 'Deleting %s@%s' % (db_id, model))
         try:
             # Deletion of the record
-            model_obj = pool.get(model)
-            model_obj.delete(db_id)
-            mdata_delete.append(mdata_id)
+            if model in object_name_list:
+                model_obj = pool.get(model)
+                model_obj.delete(db_id)
+                mdata_delete.append(mdata_id)
+            else:
+                logging.getLogger("convert").warning(
+                        'Could not delete id %d of model %s because model no '
+                        'longer exists.' % (db_id, model))
             cursor.commit()
         except Exception:
             cursor.rollback()
-            tb_s = reduce(lambda x, y: x + y,
-                    traceback.format_exception(*sys.exc_info()))
+            tb_s = ''.join(traceback.format_exception(*sys.exc_info()))
             logging.getLogger("convert").error(
                 'Could not delete id: %d of model %s\n'
                     'There should be some relation '
@@ -992,21 +827,7 @@ def post_import(pool, module, to_delete):
                     'You should manually fix this '
                     'and restart --update=module\n'
                     'Exception: %s' %
-                    (db_id, model, tb_s.decode('utf-8', 'ignore')))
-
-    transition_obj = pool.get('workflow.transition')
-    for mdata_id, db_id in transition_delete:
-        logging.getLogger("convert").info(
-            'Deleting %s@workflow.transition' % (db_id,))
-        try:
-            transition_obj.delete(db_id)
-            mdata_delete.append(mdata_id)
-            cursor.commit()
-        except Exception:
-            cursor.rollback()
-            logging.getLogger("convert").error(
-                'Could not delete id: %d of model workflow.transition' %
-                (db_id,))
+                    (db_id, model, tb_s))
 
     # Clean model_data:
     if mdata_delete:
