@@ -2,14 +2,17 @@
 #this repository contains the full copyright notices and license terms.
 import base64
 import os
-from trytond.model import ModelView, ModelSQL, ModelStorage, fields
-from trytond.tools import file_open, safe_eval
-from trytond.backend import TableHandler
-from trytond.pyson import PYSONEncoder, CONTEXT, PYSON
-from trytond.transaction import Transaction
-from trytond.cache import Cache
-from trytond.pool import Pool
-from trytond.exceptions import UserError
+from operator import itemgetter
+from ..model import ModelView, ModelSQL, ModelStorage, fields
+from ..tools import file_open, safe_eval
+from ..backend import TableHandler
+from ..pyson import PYSONEncoder, CONTEXT, PYSON
+from ..transaction import Transaction
+from ..pool import Pool
+from ..exceptions import UserError
+from ..cache import Cache
+
+EMAIL_REFKEYS = set(('cc', 'to', 'subject'))
 
 
 class Action(ModelSQL, ModelView):
@@ -24,7 +27,7 @@ class Action(ModelSQL, ModelView):
     groups = fields.Many2Many('ir.action-res.group', 'action', 'group',
             'Groups')
     icon = fields.Many2One('ir.ui.icon', 'Icon')
-    active = fields.Boolean('Active', select=2)
+    active = fields.Boolean('Active', select=True)
 
     def __init__(self):
         super(Action, self).__init__()
@@ -33,7 +36,7 @@ class Action(ModelSQL, ModelView):
         })
 
     def default_usage(self):
-        return False
+        return None
 
     def default_active(self):
         return True
@@ -58,7 +61,25 @@ class Action(ModelSQL, ModelView):
                 if action_id2:
                     action = action_obj.browse(action_id2[0])
                     return action.action.id
-            return False
+
+    def get_action_values(self, type_, action_id):
+        action_obj = Pool().get(type_)
+        action_id = action_obj.search([
+                ('action', '=', action_id),
+                ])
+        if not action_id:
+            return
+        columns = set(action_obj._columns.keys()
+            + action_obj._inherit_fields.keys())
+        columns.add('icon.rec_name')
+        to_remove = ()
+        if type_ == 'ir.action.report':
+            to_remove = ('report_content_custom', 'report_content',
+                'style_content')
+        elif type_ == 'ir.action.act_window':
+            to_remove = ('domain', 'context', 'search_value')
+        columns.difference_update(to_remove)
+        return action_obj.read(action_id[0], list(columns))
 
 Action()
 
@@ -77,7 +98,7 @@ class ActionKeyword(ModelSQL, ModelView):
             ], string='Keyword', required=True)
     model = fields.Reference('Model', selection='models_get')
     action = fields.Many2One('ir.action', 'Action',
-            ondelete='CASCADE')
+        ondelete='CASCADE', select=True)
 
     def __init__(self):
         super(ActionKeyword, self).__init__()
@@ -89,6 +110,12 @@ class ActionKeyword(ModelSQL, ModelView):
             'wrong_wizard_model': 'Wrong wizard model!',
         })
 
+    def init(self, module_name):
+        super(ActionKeyword, self).init(module_name)
+
+        table = TableHandler(Transaction().cursor, self, module_name)
+        table.index_action(['keyword', 'model'], 'add')
+
     def check_wizard_model(self, ids):
         pool = Pool()
         action_wizard_obj = pool.get('ir.action.wizard')
@@ -132,6 +159,12 @@ class ActionKeyword(ModelSQL, ModelView):
                         ).fields_view_get.reset()
             except Exception:
                 pass
+            try:
+                pool.get(keyword.model.split(',')[0]
+                    ).view_toolbar_get.reset()
+            except Exception:
+                pass
+        self.get_keyword.reset()
         return super(ActionKeyword, self).delete(ids)
 
     def create(self, vals):
@@ -144,6 +177,12 @@ class ActionKeyword(ModelSQL, ModelView):
                         ).fields_view_get.reset()
             except Exception:
                 pass
+            try:
+                pool.get(vals['model'].split(',')[0]
+                    ).view_toolbar_get.reset()
+            except Exception:
+                pass
+        self.get_keyword.reset()
         return super(ActionKeyword, self).create(vals)
 
     def write(self, ids, vals):
@@ -158,6 +197,12 @@ class ActionKeyword(ModelSQL, ModelView):
                         ).fields_view_get.reset()
             except Exception:
                 pass
+            try:
+                pool.get(keyword.model.split(',')[0]
+                    ).view_toolbar_get.reset()
+            except Exception:
+                pass
+        self.get_keyword.reset()
         res = super(ActionKeyword, self).write(ids, vals)
         for keyword in self.browse(ids):
             # Restart the cache view
@@ -166,10 +211,17 @@ class ActionKeyword(ModelSQL, ModelView):
                         ).fields_view_get.reset()
             except Exception:
                 pass
+            try:
+                pool.get(keyword.model.split(',')[0]
+                    ).view_toolbar_get.reset()
+            except Exception:
+                pass
         return res
 
+    @Cache('ir_action_keyword.get_keyword')
     def get_keyword(self, keyword, value):
         pool = Pool()
+        action_obj = pool.get('ir.action')
         res = []
         model, model_id = value
 
@@ -186,25 +238,14 @@ class ActionKeyword(ModelSQL, ModelView):
         for action_keyword_id in action_keyword_ids:
             action_keyword = self.browse(action_keyword_id)
             try:
-                action_obj = pool.get(action_keyword.action.type)
+                type_ = action_keyword.action.type
             except UserError:
                 continue
-            action_id = action_obj.search([
-                ('action', '=', action_keyword.action.id),
-                ])
-            if action_id:
-                columns = set(action_obj._columns.keys()
-                    + action_obj._inherit_fields.keys())
-                columns.add('icon.rec_name')
-                if action_keyword.action.type == 'ir.action.report':
-                    to_remove = ('report_content_custom', 'report_content',
-                        'style_content')
-                elif action_keyword.action.type == 'ir.action.act_window':
-                    to_remove = ('domain', 'context', 'search_value')
-                else:
-                    to_remove = set()
-                columns.difference_update(to_remove)
-                res.append(action_obj.read(action_id[0], list(columns)))
+            values = action_obj.get_action_values(type_,
+                action_keyword.action.id)
+            if values:
+                res.append(values)
+        sorted(res, key=itemgetter('name'))
         return res
 
 ActionKeyword()
@@ -299,8 +340,10 @@ class ActionReport(ModelSQL, ModelView):
             ], translate=False,
         string='Extension', help='Leave empty for the same as template, '
         'see unoconv documentation for compatible format')
-    module = fields.Char('Module', readonly=True, select=1)
-    email = fields.Char('Email')
+    module = fields.Char('Module', readonly=True, select=True)
+    email = fields.Char('Email',
+        help='Python dictonary where keys define "to" "cc" "subject"\n'
+        "Example: {'to': 'test@example.com', 'cc': 'user@example.com'}")
     pyson_email = fields.Function(fields.Char('PySON Email'), 'get_pyson')
 
     def __init__(self):
@@ -309,6 +352,12 @@ class ActionReport(ModelSQL, ModelView):
             ('report_name_module_uniq', 'UNIQUE(report_name, module)',
                 'The internal name must be unique by module!'),
         ]
+        self._constraints += [
+            ('check_email', 'invalid_email'),
+            ]
+        self._error_messages.update({
+                'invalid_email': 'Invalid email!',
+                })
 
     def init(self, module_name):
         super(ActionReport, self).init(module_name)
@@ -322,11 +371,10 @@ class ActionReport(ModelSQL, ModelView):
         # Migration from 1.0 output_format (m2o) is now extension (selection)
         if table.column_exist('output_format'):
             cursor.execute(
-                'SELECT report.id FROM "'+ self._table + '" report '\
-                'JOIN ir_action_report_outputformat of '\
-                    'ON (report.output_format = of.id) '\
-                'WHERE of.format = \'pdf\''
-                )
+                'SELECT report.id FROM "' + self._table + '" report '
+                'JOIN ir_action_report_outputformat of '
+                    'ON (report.output_format = of.id) '
+                'WHERE of.format = \'pdf\'')
 
             ids = [x[0] for x in cursor.fetchall()]
             with Transaction().set_user(0):
@@ -348,21 +396,29 @@ class ActionReport(ModelSQL, ModelView):
         # report_content_custom to remove base64 encoding
         if (table.column_exist('report_content_data')
                 and table.column_exist('report_content_custom')):
-            cursor.execute('SELECT id, report_content_data '
+            limit = cursor.IN_MAX
+            cursor.execute('SELECT COUNT(id) '
                 'FROM "' + self._table + '"')
-            for report_id, content in cursor.fetchall():
-                if content:
-                    content = base64.decodestring(str(content))
-                    cursor.execute('UPDATE "' + self._table + '" '
-                        'SET report_content_custom = %s '
-                        'WHERE id = %s', (content, report_id))
+            report_count, = cursor.fetchone()
+            for offset in range(0, report_count, limit):
+                cursor.execute(cursor.limit_clause(
+                    'SELECT id, report_content_data '
+                    'FROM "' + self._table + '"'
+                    'ORDER BY id',
+                    limit, offset))
+                for report_id, report in cursor.fetchall():
+                    if report:
+                        report = buffer(base64.decodestring(str(report)))
+                        cursor.execute('UPDATE "' + self._table + '" '
+                            'SET report_content_custom = %s '
+                            'WHERE id = %s', (report, report_id))
             table.drop_column('report_content_data')
 
     def default_type(self):
         return 'ir.action.report'
 
     def default_report_content(self):
-        return False
+        return None
 
     def default_direct_print(self):
         return False
@@ -376,10 +432,26 @@ class ActionReport(ModelSQL, ModelView):
     def default_module(self):
         return Transaction().context.get('module') or ''
 
+    def check_email(self, ids):
+        "Check email"
+        for report in self.browse(ids):
+            if report.email:
+                try:
+                    value = safe_eval(report.email, CONTEXT)
+                except Exception:
+                    return False
+                if isinstance(value, dict):
+                    inkeys = set(value)
+                    if not inkeys <= EMAIL_REFKEYS:
+                        return False
+                else:
+                    return False
+        return True
+
     def get_report_content(self, ids, name):
         res = {}
         converter = buffer
-        default = False
+        default = None
         format_ = Transaction().context.pop('%s.%s' % (self._name, name), '')
         if format_ == 'size':
             converter = len
@@ -392,7 +464,7 @@ class ActionReport(ModelSQL, ModelView):
                             mode='rb') as fp:
                         data = fp.read()
                 except Exception:
-                    data = False
+                    data = None
             res[report.id] = converter(data) if data else default
         return res
 
@@ -402,18 +474,18 @@ class ActionReport(ModelSQL, ModelView):
     def get_style_content(self, ids, name):
         res = {}
         converter = buffer
-        default = False
+        default = None
         format_ = Transaction().context.pop('%s.%s' % (self._name, name), '')
         if format_ == 'size':
             converter = len
             default = 0
         for report in self.browse(ids):
             try:
-                with file_open( report.style.replace('/', os.sep),
+                with file_open(report.style.replace('/', os.sep),
                         mode='rb') as fp:
                     data = fp.read()
             except Exception:
-                data = False
+                data = None
             res[report.id] = converter(data) if data else default
         return res
 
@@ -426,7 +498,7 @@ class ActionReport(ModelSQL, ModelView):
         }
         for act in self.browse(ids):
             res[act.id] = encoder.encode(safe_eval(act[field] or
-                defaults.get(field, 'False'), CONTEXT))
+                defaults.get(field, 'None'), CONTEXT))
         return res
 
     def copy(self, ids, default=None):
@@ -442,7 +514,7 @@ class ActionReport(ModelSQL, ModelView):
         reports = self.browse(ids)
         for report in reports:
             if report.report:
-                default['report_content'] = False
+                default['report_content'] = None
             default['report_name'] = report.report_name
             new_ids.append(super(ActionReport, self).copy(report.id,
                 default=default))
@@ -506,9 +578,9 @@ class ActionActWindow(ModelSQL, ModelView):
     act_window_views = fields.One2Many('ir.action.act_window.view',
             'act_window', 'Views')
     views = fields.Function(fields.Binary('Views'), 'get_views')
-    limit = fields.Integer('Limit',
+    limit = fields.Integer('Limit', required=True,
             help='Default limit for the list view')
-    auto_refresh = fields.Integer('Auto-Refresh',
+    auto_refresh = fields.Integer('Auto-Refresh', required=True,
             help='Add an auto-refresh on the view')
     action = fields.Many2One('ir.action', 'Action', required=True,
             ondelete='CASCADE')
@@ -578,7 +650,7 @@ class ActionActWindow(ModelSQL, ModelView):
                         return False
             else:
                 for act_window_view in action.act_window_views:
-                    view= act_window_view.view
+                    view = act_window_view.view
                     if view.model:
                         return False
                     if view.type != 'board':
@@ -645,7 +717,7 @@ class ActionActWindow(ModelSQL, ModelView):
         }
         for act in self.browse(ids):
             res[act.id] = encoder.encode(safe_eval(act[field] or
-                defaults.get(field, 'False'), CONTEXT))
+                defaults.get(field, 'None'), CONTEXT))
         return res
 
     def create(self, vals):
@@ -690,7 +762,7 @@ class ActionActWindowView(ModelSQL, ModelView):
     _name = 'ir.action.act_window.view'
     _rec_name = 'view'
     _description = __doc__
-    sequence = fields.Integer('Sequence')
+    sequence = fields.Integer('Sequence', required=True)
     view = fields.Many2One('ir.ui.view', 'View', required=True,
             ondelete='CASCADE')
     act_window = fields.Many2One('ir.action.act_window', 'Action',
