@@ -6,12 +6,13 @@ try:
 except ImportError:
     hashlib = None
     import md5
-from trytond.model import ModelView, ModelSQL, fields
-from trytond.config import CONFIG
-from trytond.backend import TableHandler
-from trytond.transaction import Transaction
-from trytond.pyson import Eval
-from trytond.pool import Pool
+from ..model import ModelView, ModelSQL, fields
+from ..config import CONFIG
+from ..backend import TableHandler
+from ..transaction import Transaction
+from ..pyson import Eval
+from ..pool import Pool
+
 
 def firstline(description):
     try:
@@ -35,7 +36,8 @@ class Attachment(ModelSQL, ModelView):
     description = fields.Text('Description')
     summary = fields.Function(fields.Char('Summary',
         on_change_with=['description']), 'get_summary')
-    resource = fields.Reference('Resource', selection='models_get', select=1)
+    resource = fields.Reference('Resource', selection='models_get',
+        select=True)
     link = fields.Char('Link', states={
             'invisible': Eval('type') != 'link',
             }, depends=['type'])
@@ -96,7 +98,7 @@ class Attachment(ModelSQL, ModelView):
         db_name = Transaction().cursor.dbname
         format_ = Transaction().context.pop('%s.%s' % (self._name, name), '')
         for attachment in self.browse(ids):
-            value = False
+            value = None
             if name == 'data_size' or format_ == 'size':
                 value = 0
             if attachment.digest:
@@ -121,7 +123,7 @@ class Attachment(ModelSQL, ModelView):
         return res
 
     def set_data(self, ids, name, value):
-        if value is False or value is None:
+        if value is None:
             return
         cursor = Transaction().cursor
         db_name = cursor.dbname
@@ -141,10 +143,11 @@ class Attachment(ModelSQL, ModelView):
             with open(filename, 'rb') as file_p:
                 data = file_p.read()
             if value != data:
-                cursor.execute('SELECT DISTINCT(collision) FROM ir_attachment ' \
-                        'WHERE digest = %s ' \
-                            'AND collision != 0 ' \
-                        'ORDER BY collision', (digest,))
+                cursor.execute('SELECT DISTINCT(collision) '
+                    'FROM ir_attachment '
+                    'WHERE digest = %s '
+                        'AND collision != 0 '
+                    'ORDER BY collision', (digest,))
                 collision2 = 0
                 for row in cursor.fetchall():
                     collision2 = row[0]
@@ -184,10 +187,9 @@ class Attachment(ModelSQL, ModelView):
 
     def get_last_user(self, ids, name):
         with Transaction().set_user(0):
-            return dict( (x.id, x.write_uid.rec_name if x.write_uid
+            return dict((x.id, x.write_uid.rec_name if x.write_uid
                 else x.create_uid.rec_name) for x in self.browse(ids))
 
-
     def check_access(self, ids, mode='read'):
         pool = Pool()
         model_access_obj = pool.get('ir.model.access')
