@@ -6,12 +6,12 @@ from dateutil.relativedelta import relativedelta
 import traceback
 import sys
 import logging
-from trytond.backend import Database
-from trytond.model import ModelView, ModelSQL, fields
-from trytond.tools import safe_eval
-from trytond.transaction import Transaction
-from trytond.pool import Pool
-from trytond.backend import TableHandler
+from ..config import CONFIG
+from ..model import ModelView, ModelSQL, fields
+from ..tools import safe_eval
+from ..transaction import Transaction
+from ..pool import Pool
+from ..backend import TableHandler
 
 _INTERVALTYPES = {
     'days': lambda interval: relativedelta(days=interval),
@@ -21,6 +21,7 @@ _INTERVALTYPES = {
     'minutes': lambda interval: relativedelta(minutes=interval),
 }
 
+
 class Cron(ModelSQL, ModelView):
     "Cron"
     _name = "ir.cron"
@@ -32,22 +33,22 @@ class Cron(ModelSQL, ModelView):
     request_user = fields.Many2One(
         'res.user', 'Request User', required=True,
         help="The user who will receive requests in case of failure")
-    active = fields.Boolean('Active', select=1)
-    interval_number = fields.Integer('Interval Number')
-    interval_type = fields.Selection( [
+    active = fields.Boolean('Active', select=True)
+    interval_number = fields.Integer('Interval Number', required=True)
+    interval_type = fields.Selection([
        ('minutes', 'Minutes'),
        ('hours', 'Hours'),
        ('days', 'Days'),
        ('weeks', 'Weeks'),
        ('months', 'Months'),
        ], 'Interval Unit')
-    number_calls = fields.Integer('Number of Calls', select=1,
+    number_calls = fields.Integer('Number of Calls', select=1, required=True,
        help=('Number of times the function is called, a negative '
            'number indicates that the function will always be '
            'called'))
     repeat_missed = fields.Boolean('Repeat Missed')
     next_call = fields.DateTime('Next Call', required=True,
-            select=1)
+            select=True)
     model = fields.Char('Model')
     function = fields.Char('Function')
     args = fields.Text('Arguments')
@@ -68,6 +69,7 @@ class Cron(ModelSQL, ModelView):
         table.column_rename('numbercall', 'number_calls')
         table.column_rename('doall', 'repeat_missed')
         table.column_rename('nextcall', 'next_call')
+        table.drop_column('running')
 
         super(Cron, self).init(module_name)
 
@@ -105,8 +107,7 @@ class Cron(ModelSQL, ModelView):
         return _INTERVALTYPES[cron.interval_type](cron.interval_number)
 
     def _get_request_values(self, cron):
-        tb_s = reduce(lambda x, y: x + y,
-                traceback.format_exception(*sys.exc_info()))
+        tb_s = ''.join(traceback.format_exception(*sys.exc_info()))
         tb_s = tb_s.decode('utf-8', 'ignore')
         name = self.raise_user_error('request_title',
             raise_exception=False)
@@ -136,17 +137,17 @@ class Cron(ModelSQL, ModelView):
             model_obj = pool.get(cron.model)
             with Transaction().set_user(cron.user.id):
                 getattr(model_obj, cron.function)(*args)
-        except Exception, error:
+        except Exception:
             Transaction().cursor.rollback()
 
             request_obj = pool.get('res.request')
             req_user = cron.request_user
             language = (req_user.language.code if req_user.language
-                    else 'en_US')
+                    else CONFIG['language'])
             with contextlib.nested(Transaction().set_user(cron.user.id),
                     Transaction().set_context(language=language)):
                 values = self._get_request_values(cron)
-                rid = request_obj.create(values)
+                request_obj.create(values)
             Transaction().cursor.commit()
 
     def run(self, db_name):
