@@ -1,7 +1,7 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 "Exports"
-from trytond.model import ModelView, ModelSQL, fields
+from ..model import ModelView, ModelSQL, fields
 
 
 class Export(ModelSQL, ModelView):
@@ -22,6 +22,6 @@ class ExportLine(ModelSQL, ModelView):
     _description = __doc__
     name = fields.Char('Name')
     export = fields.Many2One('ir.export', 'Export',
-       select=1)
+       select=True)
 
 ExportLine()
