@@ -2,12 +2,13 @@
 #this repository contains the full copyright notices and license terms.
 import datetime
 import re
-from trytond.model import ModelView, ModelSQL, fields
-from trytond.report import Report
-from trytond.wizard import Wizard
-from trytond.transaction import Transaction
-from trytond.cache import Cache
-from trytond.pool import Pool
+from ..model import ModelView, ModelSQL, fields
+from ..report import Report
+from ..wizard import Wizard, StateView, StateAction, Button
+from ..transaction import Transaction
+from ..cache import Cache
+from ..pool import Pool
+from ..pyson import Bool, Eval
 IDENTIFIER = re.compile(r'^[a-zA-z_][a-zA-Z0-9_]*$')
 
 
@@ -15,11 +16,23 @@ class Model(ModelSQL, ModelView):
     "Model"
     _name = 'ir.model'
     _description = __doc__
-    name = fields.Char('Model Description', translate=True, loading='lazy')
-    model = fields.Char('Model Name', required=True)
-    info = fields.Text('Information')
+    name = fields.Char('Model Description', translate=True, loading='lazy',
+        states={
+            'readonly': Bool(Eval('module')),
+            },
+        depends=['module'])
+    model = fields.Char('Model Name', required=True,
+        states={
+            'readonly': Bool(Eval('module')),
+            },
+        depends=['module'])
+    info = fields.Text('Information',
+        states={
+            'readonly': Bool(Eval('module')),
+            },
+        depends=['module'])
     module = fields.Char('Module',
-       help="Module in which this model is defined")
+       help="Module in which this model is defined", readonly=True)
     fields = fields.One2Many('ir.model.field', 'model', 'Fields',
        required=True)
 
@@ -77,16 +90,40 @@ class ModelField(ModelSQL, ModelView):
     "Model field"
     _name = 'ir.model.field'
     _description = __doc__
-    name = fields.Char('Name', required=True)
-    relation = fields.Char('Model Relation')
+    name = fields.Char('Name', required=True,
+        states={
+            'readonly': Bool(Eval('module')),
+            },
+        depends=['module'])
+    relation = fields.Char('Model Relation',
+        states={
+            'readonly': Bool(Eval('module')),
+            },
+        depends=['module'])
     model = fields.Many2One('ir.model', 'Model', required=True,
-       select=1, ondelete='CASCADE')
+        select=True, ondelete='CASCADE',
+        states={
+            'readonly': Bool(Eval('module')),
+            },
+        depends=['module'])
     field_description = fields.Char('Field Description', translate=True,
-            loading='lazy')
-    ttype = fields.Char('Field Type')
+        loading='lazy',
+        states={
+            'readonly': Bool(Eval('module')),
+            },
+        depends=['module'])
+    ttype = fields.Char('Field Type',
+        states={
+            'readonly': Bool(Eval('module')),
+            },
+        depends=['module'])
     groups = fields.Many2Many('ir.model.field-res.group', 'field_id',
             'group_id', 'Groups')
-    help = fields.Text('Help', translate=True, loading='lazy')
+    help = fields.Text('Help', translate=True, loading='lazy',
+        states={
+            'readonly': Bool(Eval('module')),
+            },
+        depends=['module'])
     module = fields.Char('Module',
        help="Module in which this field is defined")
 
@@ -137,6 +174,10 @@ class ModelField(ModelSQL, ModelView):
                     fields_names.append('name')
                     to_delete.append('name')
 
+        int_id = False
+        if isinstance(ids, (int, long)):
+            int_id = True
+            ids = [ids]
         res = super(ModelField, self).read(ids, fields_names=fields_names)
 
         if (Transaction().context.get('language')
@@ -166,20 +207,20 @@ class ModelField(ModelSQL, ModelView):
                 if 'help' in fields_names:
                     trans_args.append((id2model[model_id] + ',' + rec['name'],
                             'help', Transaction().language, None))
-            translation_obj._get_sources(trans_args)
+            translation_obj.get_sources(trans_args)
             for rec in res:
                 if isinstance(rec['model'], (list, tuple)):
                     model_id = rec['model'][0]
                 else:
                     model_id = rec['model']
                 if 'field_description' in fields_names:
-                    res_trans = translation_obj._get_source(
+                    res_trans = translation_obj.get_source(
                             id2model[model_id] + ',' + rec['name'],
                             'field', Transaction().language)
                     if res_trans:
                         rec['field_description'] = res_trans
                 if 'help' in fields_names:
-                    res_trans = translation_obj._get_source(
+                    res_trans = translation_obj.get_source(
                             id2model[model_id] + ',' + rec['name'],
                             'help', Transaction().language)
                     if res_trans:
@@ -189,6 +230,8 @@ class ModelField(ModelSQL, ModelView):
             for rec in res:
                 for field in to_delete:
                     del rec[field]
+        if int_id:
+            res = res[0]
         return res
 
 ModelField()
@@ -259,15 +302,14 @@ class ModelAccess(ModelSQL, ModelView):
         cursor = Transaction().cursor
 
         cursor.execute('SELECT MAX(CASE WHEN a.perm_%s THEN 1 ELSE 0 END) '
-                'FROM "%s" AS a '
-                'JOIN "%s" AS m '
-                    'ON (a.model = m.id) '
-                'LEFT JOIN "%s" AS gu '
-                    'ON (gu."group" = a."group") '
-                'WHERE m.model = %%s AND (gu."user" = %%s OR a."group" IS NULL)'
-                % (mode, self._table, ir_model_obj._table,
-                    user_group_obj._table),
-                (model_name, Transaction().user))
+            'FROM "%s" AS a '
+            'JOIN "%s" AS m '
+                'ON (a.model = m.id) '
+            'LEFT JOIN "%s" AS gu '
+                'ON (gu."group" = a."group") '
+            'WHERE m.model = %%s AND (gu."user" = %%s OR a."group" IS NULL)'
+            % (mode, self._table, ir_model_obj._table, user_group_obj._table),
+            (model_name, Transaction().user))
         access, = cursor.fetchone()
         if not access and access is not None:
             if raise_exception:
@@ -375,19 +417,19 @@ class ModelFieldAccess(ModelSQL, ModelView):
         cursor = Transaction().cursor
 
         cursor.execute('SELECT f.name, '
-                'MAX(CASE WHEN a.perm_%s THEN 1 ELSE 0 END) '
-                'FROM "%s" AS a '
-                'JOIN "%s" AS f '
-                    'ON (a.field = f.id) '
-                'JOIN "%s" AS m '
-                    'ON (f.model = m.id) '
-                'LEFT JOIN "%s" AS gu '
-                    'ON (gu."group" = a."group") '
-                'WHERE m.model = %%s AND (gu."user" = %%s OR a."group" IS NULL) '
-                'GROUP BY f.name'
-                % (mode, self._table, ir_model_field_obj._table,
-                    ir_model_obj._table, user_group_obj._table),
-                (model_name, Transaction().user))
+            'MAX(CASE WHEN a.perm_%s THEN 1 ELSE 0 END) '
+            'FROM "%s" AS a '
+            'JOIN "%s" AS f '
+                'ON (a.field = f.id) '
+            'JOIN "%s" AS m '
+                'ON (f.model = m.id) '
+            'LEFT JOIN "%s" AS gu '
+                'ON (gu."group" = a."group") '
+            'WHERE m.model = %%s AND (gu."user" = %%s OR a."group" IS NULL) '
+            'GROUP BY f.name'
+            % (mode, self._table, ir_model_field_obj._table,
+                ir_model_obj._table, user_group_obj._table), (model_name,
+                Transaction().user))
         accesses = dict(cursor.fetchall())
         if access:
             return accesses
@@ -438,17 +480,72 @@ class ModelFieldAccess(ModelSQL, ModelView):
 ModelFieldAccess()
 
 
+class ModelButton(ModelSQL, ModelView):
+    "Model Button"
+    _name = 'ir.model.button'
+    _description = __doc__
+    name = fields.Char('Name', required=True, readonly=True)
+    model = fields.Many2One('ir.model', 'Model', required=True, readonly=True,
+        ondelete='CASCADE', select=True)
+    groups = fields.Many2Many('ir.model.button-res.group', 'button', 'group',
+        'Groups')
+
+    def __init__(self):
+        super(ModelButton, self).__init__()
+        self._sql_constraints += [
+            ('name_model_uniq', 'UNIQUE(name, model)',
+                'The button name in model must be unique!'),
+            ]
+        self._order.insert(0, ('model', 'ASC'))
+
+    def create(self, values):
+        result = super(ModelButton, self).create(values)
+        # Restart the cache for get_groups
+        self.get_groups.reset()
+        return result
+
+    def write(self, ids, values):
+        result = super(ModelButton, self).write(ids, values)
+        # Restart the cache for get_groups
+        self.get_groups.reset()
+        return result
+
+    def delete(self, ids):
+        result = super(ModelButton, self).delete(ids)
+        # Restart the cache for get_groups
+        self.get_groups.reset()
+        return result
+
+    @Cache('ir.model.button')
+    def get_groups(self, model, name):
+        '''
+        Return a set of group ids for the named button on the model.
+        '''
+        button_ids = self.search([
+                ('model', '=', model),
+                ('name', '=', name),
+                ])
+        if not button_ids:
+            return set()
+        button_id, = button_ids
+        button = self.browse(button_id)
+        return set(g.id for g in button.groups)
+
+ModelButton()
+
+
 class ModelData(ModelSQL, ModelView):
     "Model data"
     _name = 'ir.model.data'
     _description = __doc__
     fs_id = fields.Char('Identifier on File System', required=True,
-            help="The id of the record as known on the file system.",
-            select=1)
-    model = fields.Char('Model', required=True, select=1)
-    module = fields.Char('Module', required=True, select=1)
+        help="The id of the record as known on the file system.",
+        select=True)
+    model = fields.Char('Model', required=True, select=True)
+    module = fields.Char('Module', required=True, select=True)
     db_id = fields.Integer('Resource ID',
-       help="The id of the record in the database.", select=1)
+        help="The id of the record in the database.", select=True,
+        required=True)
     date_update = fields.DateTime('Update Date')
     date_init = fields.DateTime('Init Date')
     values = fields.Text('Values')
@@ -471,6 +568,13 @@ class ModelData(ModelSQL, ModelView):
     def default_noupdate(self):
         return False
 
+    def write(self, ids, values):
+        result = super(ModelData, self).write(ids, values)
+        # Restart the cache for get_id
+        self.get_id.reset()
+        return result
+
+    @Cache('ir_model_data.get_id')
     def get_id(self, module, fs_id):
         """
         Return for an fs_id the corresponding db_id.
@@ -486,48 +590,47 @@ class ModelData(ModelSQL, ModelView):
             ('inherit', '=', False),
             ], limit=1)
         if not ids:
-            raise Exception("Reference to %s not found" % \
-                                ".".join([module,fs_id]))
+            raise Exception("Reference to %s not found"
+                % ".".join([module, fs_id]))
         return self.read(ids[0], ['db_id'])['db_id']
 
 ModelData()
 
 
-class PrintModelGraphInit(ModelView):
-    'Print Model Graph Init'
-    _name = 'ir.model.print_model_graph.init'
+class PrintModelGraphStart(ModelView):
+    'Print Model Graph'
+    _name = 'ir.model.print_model_graph.start'
     _description = __doc__
-    level = fields.Integer('Level')
+    level = fields.Integer('Level', required=True)
     filter = fields.Text('Filter', help="Entering a Python "
             "Regular Expression will exclude matching models from the graph.")
 
     def default_level(self):
         return 1
 
-PrintModelGraphInit()
+PrintModelGraphStart()
 
 
 class PrintModelGraph(Wizard):
     _name = 'ir.model.print_model_graph'
-    states = {
-        'init': {
-            'result': {
-                'type': 'form',
-                'object': 'ir.model.print_model_graph.init',
-                'state': [
-                    ('end', 'Cancel', 'tryton-cancel'),
-                    ('print', 'Print', 'tryton-ok', True),
-                ],
-            },
-        },
-        'print': {
-            'result': {
-                'type': 'print',
-                'report': 'ir.model.graph',
-                'state': 'end',
-            },
-        },
-    }
+
+    start = StateView('ir.model.print_model_graph.start',
+        'ir.print_model_graph_start_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('Print', 'print_', 'tryton-ok', default=True),
+            ])
+    print_ = StateAction('ir.report_model_graph')
+
+    def transition_print_(self, session):
+        return 'end'
+
+    def do_print_(self, session, action):
+        return action, {
+            'id': Transaction().context.get('active_id'),
+            'ids': Transaction().context.get('active_ids'),
+            'level': session.start.level,
+            'filter': session.start.filter,
+            }
 
 PrintModelGraph()
 
@@ -535,16 +638,16 @@ PrintModelGraph()
 class ModelGraph(Report):
     _name = 'ir.model.graph'
 
-    def execute(self, ids, datas):
+    def execute(self, ids, data):
         import pydot
         pool = Pool()
         model_obj = pool.get('ir.model')
         action_report_obj = pool.get('ir.action.report')
 
-        if not datas['form']['filter']:
+        if not data['filter']:
             filter = None
         else:
-            filter = re.compile(datas['form']['filter'], re.VERBOSE)
+            filter = re.compile(data['filter'], re.VERBOSE)
         action_report_ids = action_report_obj.search([
             ('report_name', '=', self._name)
             ])
@@ -557,8 +660,7 @@ class ModelGraph(Report):
         graph = pydot.Dot(fontsize="8")
         graph.set('center', '1')
         graph.set('ratio', 'auto')
-        self.fill_graph(models, graph, level=datas['form']['level'],
-                filter=filter)
+        self.fill_graph(models, graph, level=data['level'], filter=filter)
         data = graph.create(prog='dot', format='png')
         return ('png', buffer(data), False, action_report.name)
 
