@@ -3,11 +3,13 @@
 import os
 from trytond.model import ModelView, ModelSQL, fields
 import trytond.tools as tools
-from trytond.modules import MODULES_PATH, create_graph, get_module_list
-from trytond.wizard import Wizard
-from trytond.backend import Database
+from trytond.modules import create_graph, get_module_list
+from trytond.wizard import Wizard, StateView, Button, StateTransition, \
+    StateAction
+from trytond.backend import TableHandler
 from trytond.pool import Pool
 from trytond.transaction import Transaction
+from trytond.pyson import Eval
 
 
 class Module(ModelSQL, ModelView):
@@ -22,6 +24,10 @@ class Module(ModelSQL, ModelView):
     version = fields.Function(fields.Char('Version'), 'get_version')
     dependencies = fields.One2Many('ir.module.module.dependency',
         'module', 'Dependencies', readonly=True)
+    parents = fields.Function(fields.One2Many('ir.module.module', None,
+            'Parents'), 'get_parents')
+    childs = fields.Function(fields.One2Many('ir.module.module', None,
+            'Childs'), 'get_childs')
     state = fields.Selection([
         ('uninstalled', 'Not Installed'),
         ('installed', 'Installed'),
@@ -38,12 +44,6 @@ class Module(ModelSQL, ModelView):
         ]
         self._order.insert(0, ('name', 'ASC'))
         self._rpc.update({
-            'button_install': True,
-            'button_install_cancel': True,
-            'button_uninstall': True,
-            'button_uninstall_cancel': True,
-            'button_upgrade': True,
-            'button_upgrade_cancel': True,
             'on_write': False,
         })
         self._error_messages.update({
@@ -53,6 +53,26 @@ class Module(ModelSQL, ModelView):
             'uninstall_dep': 'The modules you are trying to uninstall ' \
                     'depends on installed modules:',
             })
+        self._buttons.update({
+                'install': {
+                    'invisible': Eval('state') != 'uninstalled',
+                    },
+                'install_cancel': {
+                    'invisible': Eval('state') != 'to install',
+                    },
+                'uninstall': {
+                    'invisible': Eval('state') != 'installed',
+                    },
+                'uninstall_cancel': {
+                    'invisible': Eval('state') != 'to remove',
+                    },
+                'upgrade': {
+                    'invisible': Eval('state') != 'installed',
+                    },
+                'upgrade_cancel': {
+                    'invisible': Eval('state') != 'to upgrade',
+                    },
+                })
 
     def default_state(self):
         return 'uninstalled'
@@ -61,7 +81,7 @@ class Module(ModelSQL, ModelView):
     def get_module_info(name):
         "Return the content of the __tryton__.py"
         try:
-            if name in ['ir', 'workflow', 'res', 'webdav']:
+            if name in ['ir', 'res', 'webdav']:
                 file_p = tools.file_open(os.path.join(name, '__tryton__.py'))
             else:
                 file_p = tools.file_open(os.path.join(name, '__tryton__.py'))
@@ -79,6 +99,32 @@ class Module(ModelSQL, ModelView):
                     module.name).get('version', '')
         return res
 
+    def get_parents(self, ids, name):
+        parents = {}
+        modules = self.browse(ids)
+        parent_names = list(set(d.name for m in modules
+                    for d in m.dependencies))
+        parent_ids = self.search([
+                ('name', 'in', parent_names),
+                ])
+        name2id = dict((m.name, m.id) for m in self.browse(parent_ids))
+        for module in self.browse(ids):
+            parents[module.id] = [name2id[d.name] for d in module.dependencies]
+        return parents
+
+    def get_childs(self, ids, name):
+        childs = dict((i, []) for i in ids)
+        modules = self.browse(ids)
+        names = [m.name for m in modules]
+        child_ids = self.search([
+                ('dependencies.name', 'in', names),
+                ])
+        for child in self.browse(child_ids):
+            for dep in child.dependencies:
+                if dep.module.id in childs:
+                    childs[dep.module.id].append(child.id)
+        return childs
+
     def delete(self, ids):
         if not ids:
             return True
@@ -102,23 +148,20 @@ class Module(ModelSQL, ModelView):
         for module in self.browse(ids):
             if module.name not in graph:
                 continue
-            def get_parents(name, graph):
-                parents = set()
-                for node in graph:
-                    if graph[name] in node.childs:
-                        parents.add(node.name)
-                for parent in parents.copy():
-                    parents.update(get_parents(parent, graph))
+
+            def get_parents(module):
+                parents = set(p.name for p in module.parents)
+                for p in module.parents:
+                    parents.update(get_parents(p))
                 return parents
-            dependencies = get_parents(module.name, graph)
-            def get_childs(name, graph):
-                childs = set(x.name for x in graph[name].childs)
-                childs2 = set()
-                for child in childs:
-                    childs2.update(get_childs(child, graph))
-                childs.update(childs2)
+            dependencies = get_parents(module)
+
+            def get_childs(module):
+                childs = set(c.name for c in module.childs)
+                for c in module.childs:
+                    childs.update(get_childs(c))
                 return childs
-            dependencies.update(get_childs(module.name, graph))
+            dependencies.update(get_childs(module))
             res += self.search([
                 ('name', 'in', list(dependencies)),
                 ])
@@ -129,19 +172,17 @@ class Module(ModelSQL, ModelView):
         for module in self.browse(ids):
             if module.name not in graph:
                 missings = []
-                for package, deps, datas in packages:
+                for package, deps, xdep, info in packages:
                     if package == module.name:
                         missings = [x for x in deps if x not in graph]
                 self.raise_user_error('missing_dep', (missings, module.name))
-            def get_parents(name, graph):
-                parents = set()
-                for node in graph:
-                    if graph[name] in node.childs:
-                        parents.add(node.name)
-                for parent in parents.copy():
-                    parents.update(get_parents(parent, graph))
+
+            def get_parents(module):
+                parents = set(p.name for p in module.parents)
+                for p in module.parents:
+                    parents.update(get_parents(p))
                 return parents
-            dependencies = list(get_parents(module.name, graph))
+            dependencies = list(get_parents(module))
             module_install_ids = self.search([
                 ('name', 'in', dependencies),
                 ('state', '=', 'uninstalled'),
@@ -155,10 +196,11 @@ class Module(ModelSQL, ModelView):
         for module in self.browse(ids):
             if module.name not in graph:
                 missings = []
-                for package, deps, datas in packages:
+                for package, deps, xdep, info in packages:
                     if package == module.name:
                         missings = [x for x in deps if x not in graph]
                 self.raise_user_error('missing_dep', (missings, module.name))
+
             def get_childs(name, graph):
                 childs = set(x.name for x in graph[name].childs)
                 childs2 = set()
@@ -175,16 +217,19 @@ class Module(ModelSQL, ModelView):
                 'state': 'to upgrade',
                 })
 
-    def button_install(self, ids):
+    @ModelView.button
+    def install(self, ids):
         return self.state_install(ids)
 
-    def button_install_cancel(self, ids):
+    @ModelView.button
+    def install_cancel(self, ids):
         self.write(ids, {
             'state': 'uninstalled',
             })
         return True
 
-    def button_uninstall(self, ids):
+    @ModelView.button
+    def uninstall(self, ids):
         cursor = Transaction().cursor
         for module in self.browse(ids):
             cursor.execute('SELECT m.state, m.name ' \
@@ -202,14 +247,17 @@ class Module(ModelSQL, ModelView):
         self.write(ids, {'state': 'to remove'})
         return True
 
-    def button_uninstall_cancel(self, ids):
+    @ModelView.button
+    def uninstall_cancel(self, ids):
         self.write(ids, {'state': 'installed'})
         return True
 
-    def button_upgrade(self, ids):
+    @ModelView.button
+    def upgrade(self, ids):
         return self.state_upgrade(ids)
 
-    def button_upgrade_cancel(self, ids):
+    @ModelView.button
+    def upgrade_cancel(self, ids):
         self.write(ids, {'state': 'installed'})
         return True
 
@@ -281,12 +329,6 @@ class Module(ModelSQL, ModelView):
                     self._update_dependencies(mod, tryton.get('depends', []))
                     continue
 
-                if name in ['ir', 'workflow', 'res', 'webdav']:
-                    mod_path = os.path.join(
-                            os.path.dirname(MODULES_PATH), name)
-                else:
-                    mod_path = os.path.join(MODULES_PATH, name)
-
                 tryton = Module.get_module_info(mod_name)
                 if not tryton:
                     continue
@@ -319,6 +361,8 @@ class Module(ModelSQL, ModelView):
             if x.name not in depends])
         if depends is None:
             depends = []
+        # Restart Browse Cache for deleted dependencies
+        module = self.browse(module.id)
         dependency_names = [x.name for x in module.dependencies]
         for depend in depends:
             if depend not in dependency_names:
@@ -335,16 +379,16 @@ class ModuleDependency(ModelSQL, ModelView):
     _name = "ir.module.module.dependency"
     _description = __doc__
     name = fields.Char('Name')
-    module = fields.Many2One('ir.module.module', 'Module', select=1,
+    module = fields.Many2One('ir.module.module', 'Module', select=True,
        ondelete='CASCADE', required=True)
     state = fields.Function(fields.Selection([
-        ('uninstalled','Not Installed'),
-        ('installed','Installed'),
-        ('to upgrade','To be upgraded'),
-        ('to remove','To be removed'),
-        ('to install','To be installed'),
-        ('unknown', 'Unknown'),
-        ], 'State', readonly=True), 'get_state')
+                ('uninstalled', 'Not Installed'),
+                ('installed', 'Installed'),
+                ('to upgrade', 'To be upgraded'),
+                ('to remove', 'To be removed'),
+                ('to install', 'To be installed'),
+                ('unknown', 'Unknown'),
+                ], 'State', readonly=True), 'get_state')
 
     def __init__(self):
         super(ModuleDependency, self).__init__()
@@ -374,17 +418,27 @@ class ModuleConfigWizardItem(ModelSQL, ModelView):
     "Config wizard to run after installing module"
     _name = 'ir.module.module.config_wizard.item'
     _description = __doc__
-    name = fields.Char('Name', required=True, readonly=True)
-    sequence= fields.Integer('Sequence')
+    _rec_name = 'action'
+    action = fields.Many2One('ir.action', 'Action', required=True,
+        readonly=True)
+    sequence = fields.Integer('Sequence', required=True)
     state = fields.Selection([
         ('open', 'Open'),
         ('done', 'Done'),
-        ], string='State', required=True, select=1)
+        ], string='State', required=True, select=True)
 
     def __init__(self):
         super(ModuleConfigWizardItem, self).__init__()
         self._order.insert(0, ('sequence', 'ASC'))
 
+    def init(self, module_name):
+        table = TableHandler(Transaction().cursor, self, module_name)
+
+        # Migrate from 2.2 remove name
+        table.drop_column('name')
+
+        super(ModuleConfigWizardItem, self).init(module_name)
+
     def default_state(self):
         return 'open'
 
@@ -402,99 +456,115 @@ class ModuleConfigWizardFirst(ModelView):
 ModuleConfigWizardFirst()
 
 
+class ModuleConfigWizardOther(ModelView):
+    'Module Config Wizard Other'
+    _name = 'ir.module.module.config_wizard.other'
+    _description = __doc__
+
+    percentage = fields.Float('Percentage', readonly=True)
+
+    def default_percentage(self):
+        pool = Pool()
+        item_obj = pool.get('ir.module.module.config_wizard.item')
+        done = item_obj.search([
+            ('state', '=', 'done'),
+            ], count=True)
+        all = item_obj.search([], count=True)
+        return 100.0 * done / all
+
+ModuleConfigWizardOther()
+
+
 class ModuleConfigWizard(Wizard):
     'Run config wizards'
     _name = 'ir.module.module.config_wizard'
-    states = {
-        'init': {
-            'result': {
-                'type': 'choice',
-                'next_state': '_first',
-            },
-        },
-        'first': {
-            'result': {
-                'type': 'form',
-                'object': 'ir.module.module.config_wizard.first',
-                'state': [
-                    ('end', 'Cancel', 'tryton-cancel'),
-                    ('wizard', 'Ok', 'tryton-ok', True),
-                ],
-            },
-        },
-        'wizard': {
-            'result': {
-                'type': 'action',
-                'action': '_action_wizard',
-                'state': 'next',
-            },
-        },
-        'next': {
-            'result': {
-                'type': 'choice',
-                'next_state': '_next',
-            },
-        },
-    }
 
-    def _first(self, data):
-        res = self._next(data)
-        if res == 'wizard':
+    class ConfigStateAction(StateAction):
+
+        def __init__(self):
+            StateAction.__init__(self, None)
+
+        def get_action(self):
+            pool = Pool()
+            item_obj = pool.get('ir.module.module.config_wizard.item')
+            action_obj = pool.get('ir.action')
+            item_ids = item_obj.search([
+                ('state', '=', 'open'),
+                ], limit=1)
+            if item_ids:
+                item = item_obj.browse(item_ids[0])
+                item_obj.write(item.id, {
+                    'state': 'done',
+                    })
+                return action_obj.get_action_values(item.action.type,
+                    item.action.id)
+
+    start = StateTransition()
+    first = StateView('ir.module.module.config_wizard.first',
+        'ir.module_config_wizard_first_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('Ok', 'action', 'tryton-ok', default=True),
+            ])
+    other = StateView('ir.module.module.config_wizard.other',
+        'ir.module_config_wizard_other_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('Next', 'action', 'tryton-go-next', default=True),
+            ])
+    action = ConfigStateAction()
+
+    def transition_start(self, session):
+        res = self.transition_action(session)
+        if res == 'other':
             return 'first'
         return res
 
-    def _action_wizard(self, data):
-        pool = Pool()
-        item_obj = pool.get('ir.module.module.config_wizard.item')
-        item_ids = item_obj.search([
-            ('state', '=', 'open'),
-            ], limit=1)
-        if item_ids:
-            item = item_obj.browse(item_ids[0])
-            item_obj.write(item.id, {
-                'state': 'done',
-                })
-            return {
-                    'type': 'ir.action.wizard',
-                    'wiz_name': item.name,
-                    }
-        return {}
-
-    def _next(self, data):
+    def transition_action(self, session):
         pool = Pool()
         item_obj = pool.get('ir.module.module.config_wizard.item')
         item_ids = item_obj.search([
             ('state', '=', 'open'),
             ])
         if item_ids:
-            return 'wizard'
+            return 'other'
         return 'end'
 
 ModuleConfigWizard()
 
 
-class ModuleInstallUpgradeInit(ModelView):
-    'Module Install Upgrade Init'
-    _name = 'ir.module.module.install_upgrade.init'
-    _description = __doc__
-    module_info = fields.Text('Modules to update', readonly=True)
-
-ModuleInstallUpgradeInit()
-
-
 class ModuleInstallUpgradeStart(ModelView):
     'Module Install Upgrade Start'
     _name = 'ir.module.module.install_upgrade.start'
     _description = __doc__
+    module_info = fields.Text('Modules to update', readonly=True)
 
 ModuleInstallUpgradeStart()
 
 
+class ModuleInstallUpgradeDone(ModelView):
+    'Module Install Upgrade Done'
+    _name = 'ir.module.module.install_upgrade.done'
+    _description = __doc__
+
+ModuleInstallUpgradeDone()
+
+
 class ModuleInstallUpgrade(Wizard):
     "Install / Upgrade modules"
     _name = 'ir.module.module.install_upgrade'
 
-    def _get_install(self, data):
+    start = StateView('ir.module.module.install_upgrade.start',
+        'ir.module_install_upgrade_start_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('Start Upgrade', 'upgrade', 'tryton-ok', default=True),
+            ])
+    upgrade = StateTransition()
+    done = StateView('ir.module.module.install_upgrade.done',
+        'ir.module_install_upgrade_done_view_form', [
+            Button('Ok', 'config', 'tryton-ok', default=True),
+            ])
+    config = StateAction('ir.act_module_config_wizard')
+
+    def default_start(self, session, fields):
         pool = Pool()
         module_obj = pool.get('ir.module.module')
         module_ids = module_obj.search([
@@ -506,7 +576,7 @@ class ModuleInstallUpgrade(Wizard):
                     for x in modules),
         }
 
-    def _upgrade_module(self, data):
+    def transition_upgrade(self, session):
         pool = Pool()
         module_obj = pool.get('ir.module.module')
         lang_obj = pool.get('ir.lang')
@@ -521,49 +591,11 @@ class ModuleInstallUpgrade(Wizard):
             transaction.cursor.commit()
         if module_ids:
             pool.init(update=True, lang=lang)
-            new_wizard = pool.get('ir.module.module.install_upgrade',
-                    type='wizard')
-            new_wizard._lock.acquire()
-            new_wizard._datas[data['_wiz_id']] = self._datas[data['_wiz_id']]
-            new_wizard._lock.release()
-        return {}
-
-    states = {
-        'init': {
-            'actions': ['_get_install'],
-            'result': {
-                'type': 'form',
-                'object': 'ir.module.module.install_upgrade.init',
-                'state': [
-                    ('end', 'Cancel', 'tryton-cancel'),
-                    ('start', 'Start Upgrade', 'tryton-ok', True),
-                ],
-            },
-        },
-        'start': {
-            'actions': ['_upgrade_module'],
-            'result': {
-                'type': 'form',
-                'object': 'ir.module.module.install_upgrade.start',
-                'state': [
-                    ('config', 'Ok', 'tryton-ok', True),
-                ],
-            },
-        },
-        'config': {
-            'result': {
-                'type': 'action',
-                'action': '_config',
-                'state': 'end',
-            },
-        },
-    }
-
-    def _config(self, data):
-        return {
-                'type': 'ir.action.wizard',
-                'wiz_name': 'ir.module.module.config_wizard',
-                }
+        if session:
+            # Don't store session to prevent concurrent update
+            for state_name in session.data:
+                getattr(session, state_name).dirty = False
+        return 'done'
 
 ModuleInstallUpgrade()
 
@@ -571,22 +603,10 @@ ModuleInstallUpgrade()
 class ModuleConfig(Wizard):
     'Configure Modules'
     _name = 'ir.module.module.config'
-    states = {
-        'init': {
-            'result': {
-                'type': 'action',
-                'action': '_action_open',
-                'state': 'end',
-            },
-        },
-    }
 
-    def _action_open(self, datas):
-        pool = Pool()
-        model_data_obj = pool.get('ir.model.data')
-        act_window_obj = pool.get('ir.action.act_window')
-        act_window_id = model_data_obj.get_id('ir', 'act_module_form')
-        res = act_window_obj.read(act_window_id)
-        return res
+    start = StateAction('ir.act_module_form')
+
+    def transition_start(self, session):
+        return 'end'
 
 ModuleConfig()
