@@ -2,26 +2,26 @@
 #this repository contains the full copyright notices and license terms.
 import contextlib
 import time
-from trytond.model import ModelView, ModelSQL, fields
-from trytond.tools import safe_eval
-from trytond.pyson import Eval
-from trytond.transaction import Transaction
-from trytond.cache import Cache
-from trytond.const import OPERATORS
-from trytond.pool import Pool
+from ..model import ModelView, ModelSQL, fields
+from ..tools import safe_eval
+from ..pyson import Eval
+from ..transaction import Transaction
+from ..cache import Cache
+from ..const import OPERATORS
+from ..pool import Pool
 
 
 class RuleGroup(ModelSQL, ModelView):
     "Rule group"
     _name = 'ir.rule.group'
     _description = __doc__
-    name = fields.Char('Name', select=1)
-    model = fields.Many2One('ir.model', 'Model', select=1,
+    name = fields.Char('Name', select=True)
+    model = fields.Many2One('ir.model', 'Model', select=True,
             required=True)
-    global_p = fields.Boolean('Global', select=1,
+    global_p = fields.Boolean('Global', select=True,
             help="Make the rule global \n" \
                     "so every users must follow this rule")
-    default_p = fields.Boolean('Default', select=1,
+    default_p = fields.Boolean('Default', select=True,
             help="Add this rule to all users by default")
     rules = fields.One2Many('ir.rule', 'rule_group', 'Tests',
             help="The rule is satisfied if at least one test is True")
@@ -90,11 +90,11 @@ class Rule(ModelSQL, ModelView):
     _description = __doc__
     field = fields.Many2One('ir.model.field', 'Field',
         domain=[('model', '=', Eval('_parent_rule_group', {}).get('model'))],
-        select=1, required=True)
+        select=True, required=True)
     operator = fields.Selection([(x, x) for x in OPERATORS], 'Operator',
         required=True, translate=False)
-    operand = fields.Selection('get_operand','Operand', required=True)
-    rule_group = fields.Many2One('ir.rule.group', 'Group', select=2,
+    operand = fields.Selection('get_operand', 'Operand', required=True)
+    rule_group = fields.Many2One('ir.rule.group', 'Group', select=True,
        required=True, ondelete="CASCADE")
 
     def init(self, module_name):
@@ -107,7 +107,8 @@ class Rule(ModelSQL, ModelView):
             'SET operator = %%s '
             'WHERE operator = %%s' % self._table, ('!=', '<>'))
 
-    def _operand_get(self, obj_name='', level=3, recur=None, root_tech='', root=''):
+    def _operand_get(self, obj_name='', level=3, recur=None, root_tech='',
+            root=''):
         res = {}
         if not obj_name:
             obj_name = 'res.user'
@@ -121,8 +122,9 @@ class Rule(ModelSQL, ModelView):
         for k in key:
 
             if obj_fields[k]['type'] in ('many2one'):
-                res[root + '/' + obj_fields[k]['string']] = \
-                        root_tech + '.' + k + '.id'
+                res[root + '/' + obj_fields[k]['string']] = (
+                    '(' + root_tech + '.' + k + '.id '
+                    'if ' + root_tech + ' else None)')
 
             elif obj_fields[k]['type'] in ('many2many', 'one2many'):
                 res[root + '/' + obj_fields[k]['string']] = \
@@ -131,9 +133,9 @@ class Rule(ModelSQL, ModelView):
                 res[root + '/' + obj_fields[k]['string']] = \
                         root_tech + '.' + k
 
-            if (obj_fields[k]['type'] in recur) and (level>0):
+            if (obj_fields[k]['type'] in recur) and (level > 0):
                 res.update(self._operand_get(obj_fields[k]['relation'],
-                    level-1, recur, root_tech + '.' + k,
+                    level - 1, recur, root_tech + '.' + k,
                     root + '/' + obj_fields[k]['string']))
 
         return res
@@ -167,23 +169,23 @@ class Rule(ModelSQL, ModelView):
         user_obj = pool.get('res.user')
 
         cursor = Transaction().cursor
-        cursor.execute('SELECT r.id FROM "' + self._table + '" r ' \
-                'JOIN "' + rule_group_obj._table + '" g ' \
-                    "ON (g.id = r.rule_group) " \
-                'JOIN "' + model_obj._table + '" m ON (g.model = m.id) ' \
-                "WHERE m.model = %s "
-                    "AND g.perm_" + mode + " "
-                    "AND (g.id IN (" \
-                            'SELECT rule_group ' \
-                            'FROM "' + rule_group_user_obj._table + '" ' \
-                                'WHERE "user" = %s ' \
-                            "UNION SELECT rule_group " \
-                            'FROM "' + rule_group_group_obj._table + '" g_rel ' \
-                                'JOIN "' + user_group_obj._table + '" u_rel ' \
-                                    'ON (g_rel."group" = u_rel."group") ' \
-                                'WHERE u_rel."user" = %s) ' \
-                        "OR default_p " \
-                        "OR g.global_p)",
+        cursor.execute('SELECT r.id FROM "' + self._table + '" r '
+            'JOIN "' + rule_group_obj._table + '" g '
+                "ON (g.id = r.rule_group) "
+            'JOIN "' + model_obj._table + '" m ON (g.model = m.id) '
+            "WHERE m.model = %s "
+                "AND g.perm_" + mode + " "
+                "AND (g.id IN ("
+                        'SELECT rule_group '
+                        'FROM "' + rule_group_user_obj._table + '" '
+                        'WHERE "user" = %s '
+                        "UNION SELECT rule_group "
+                        'FROM "' + rule_group_group_obj._table + '" g_rel '
+                        'JOIN "' + user_group_obj._table + '" u_rel '
+                            'ON (g_rel."group" = u_rel."group") '
+                        'WHERE u_rel."user" = %s) '
+                    "OR default_p "
+                    "OR g.global_p)",
                 (model_name, Transaction().user, Transaction().user))
         ids = [x[0] for x in cursor.fetchall()]
         if not ids:
