@@ -1,15 +1,14 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 from string import Template
-import datetime
 import time
-from trytond.model import ModelView, ModelSQL, fields
-from trytond.tools import datetime_strftime
-from trytond.pyson import Eval
-from trytond.transaction import Transaction
-from trytond.pool import Pool
-from trytond.config import CONFIG
-from trytond.backend import TableHandler
+from ..model import ModelView, ModelSQL, fields
+from ..tools import datetime_strftime
+from ..pyson import Eval, And
+from ..transaction import Transaction
+from ..pool import Pool
+from ..config import CONFIG
+from ..backend import TableHandler
 
 sql_sequence = CONFIG.options['db_type'] == 'postgresql'
 
@@ -45,23 +44,27 @@ class Sequence(ModelSQL, ModelView):
     number_next_internal = fields.Integer('Next Number',
         states={
             'invisible': ~Eval('type').in_(['incremental']),
+            'required': And(Eval('type').in_(['incremental']),
+                not sql_sequence),
             }, depends=['type'])
     number_next = fields.Function(number_next_internal, 'get_number_next',
         'set_number_next')
     number_increment = fields.Integer('Increment Number',
         states={
             'invisible': ~Eval('type').in_(['incremental']),
+            'required': Eval('type').in_(['incremental']),
             }, depends=['type'])
     padding = fields.Integer('Number padding',
         states={
             'invisible': ~Eval('type').in_(['incremental']),
+            'required': Eval('type').in_(['incremental']),
             }, depends=['type'])
     timestamp_rounding = fields.Float('Timestamp Rounding', required=True,
         states={
             'invisible': ~Eval('type').in_(
                 ['decimal timestamp', 'hexadecimal timestamp']),
             }, depends=['type'])
-    timestamp_offset = fields.Float('Timestamp Offset',
+    timestamp_offset = fields.Float('Timestamp Offset', required=True,
         states={
             'invisible': ~Eval('type').in_(
                 ['decimal timestamp', 'hexadecimal timestamp']),
@@ -70,6 +73,8 @@ class Sequence(ModelSQL, ModelView):
         states={
             'invisible': ~Eval('type').in_(
                 ['decimal timestamp', 'hexadecimal timestamp']),
+            'required': Eval('type').in_(
+                ['decimal timestamp', 'hexadecimal timestamp']),
             }, depends=['type'])
 
     def __init__(self):
@@ -78,6 +83,10 @@ class Sequence(ModelSQL, ModelView):
             ('check_prefix_suffix', 'invalid_prefix_suffix'),
             ('check_last_timestamp', 'future_last_timestamp'),
         ]
+        self._sql_constraints += [
+            ('check_timestamp_rounding', 'CHECK(timestamp_rounding > 0)',
+                'Timestamp rounding should be greater than 0'),
+            ]
         self._error_messages.update({
             'missing': 'Missing sequence!',
             'invalid_prefix_suffix': 'Invalid prefix/suffix!',
@@ -123,13 +132,13 @@ class Sequence(ModelSQL, ModelView):
         return 1.0
 
     def default_timestamp_offset(self):
-        return 946681200.0 # Offset for 2000-01-01
+        return 946681200.0  # Offset for 2000-01-01
 
     def default_last_timestamp(self):
         return 0.0
 
     def default_code(self):
-        return Transaction().context.get('code', False)
+        return Transaction().context.get('code')
 
     def get_number_next(self, ids, name):
         cursor = Transaction().cursor
@@ -148,7 +157,6 @@ class Sequence(ModelSQL, ModelView):
         return result
 
     def set_number_next(self, ids, name, value):
-        cursor = Transaction().cursor
         super(Sequence, self).write(ids, {
                 'number_next_internal': value,
                 })
@@ -295,7 +303,6 @@ class Sequence(ModelSQL, ModelView):
                 return hex(timestamp)[2:].upper()
         return ''
 
-
     def get_id(self, domain):
         '''
         Return sequence value for the domain
