@@ -2,13 +2,14 @@
 #this repository contains the full copyright notices and license terms.
 import datetime
 import time
-from trytond.model import ModelView, ModelSQL, fields
-from trytond.pyson import Eval, Or
-from trytond.tools import safe_eval
-from trytond.backend import TableHandler
-from trytond.tools import reduce_ids
-from trytond.transaction import Transaction
-from trytond.cache import Cache
+from ..model import ModelView, ModelSQL, fields
+from ..pyson import Eval
+from ..tools import safe_eval
+from ..backend import TableHandler
+from ..tools import reduce_ids
+from ..transaction import Transaction
+from ..cache import Cache
+from ..pool import Pool
 
 
 class Trigger(ModelSQL, ModelView):
@@ -16,34 +17,35 @@ class Trigger(ModelSQL, ModelView):
     _name = 'ir.trigger'
     _description = __doc__
     name = fields.Char('Name', required=True, translate=True)
-    active = fields.Boolean('Active', select=2)
-    model = fields.Many2One('ir.model', 'Model', required=True, select=1)
-    on_time = fields.Boolean('On Time', select=1, states={
-        'invisible': Or(Eval('on_create', False), Eval('on_write', False),
-            Eval('on_delete', False)),
+    active = fields.Boolean('Active', select=True)
+    model = fields.Many2One('ir.model', 'Model', required=True, select=True)
+    on_time = fields.Boolean('On Time', select=True, states={
+            'invisible': (Eval('on_create', False)
+                or Eval('on_write', False)
+                or Eval('on_delete', False)),
         }, depends=['on_create', 'on_write', 'on_delete'],
         on_change=['on_time'])
-    on_create = fields.Boolean('On Create', select=1, states={
+    on_create = fields.Boolean('On Create', select=True, states={
         'invisible': Eval('on_time', False),
         }, depends=['on_time'],
         on_change=['on_create'])
-    on_write = fields.Boolean('On Write', select=1, states={
+    on_write = fields.Boolean('On Write', select=True, states={
         'invisible': Eval('on_time', False),
         }, depends=['on_time'],
         on_change=['on_write'])
-    on_delete = fields.Boolean('On Delete', select=1, states={
+    on_delete = fields.Boolean('On Delete', select=True, states={
         'invisible': Eval('on_time', False),
         }, depends=['on_time'],
         on_change=['on_delete'])
     condition = fields.Char('Condition', required=True,
-            help='A Python statement evaluated with record represented by '
-            '"self"\nIt triggers the action if true.')
-    limit_number = fields.Integer('Limit Number', help='Limit the number of '
-            'call to "Action Function" by records.\n'
-            '0 for no limit.')
+        help='A Python statement evaluated with record represented by '
+        '"self"\nIt triggers the action if true.')
+    limit_number = fields.Integer('Limit Number', required=True,
+        help='Limit the number of call to "Action Function" by records.\n'
+        '0 for no limit.')
     minimum_delay = fields.Float('Minimum Delay', help='Set a minimum delay '
-            'in minutes between call to "Action Function" for the same record.\n'
-            '0 for no delay.')
+        'in minutes between call to "Action Function" for the same record.\n'
+        '0 for no delay.')
     action_model = fields.Many2One('ir.model', 'Action Model', required=True)
     action_function = fields.Char('Action Function', required=True)
 
@@ -76,6 +78,9 @@ class Trigger(ModelSQL, ModelView):
     def default_active(self):
         return True
 
+    def default_limit_number(self):
+        return 0
+
     def on_change_on_time(self, values):
         if values.get('on_time'):
             return {
@@ -119,7 +124,7 @@ class Trigger(ModelSQL, ModelView):
                 'Invalid trigger mode'
 
         if Transaction().user == 0:
-            return [] # XXX is it want we want?
+            return []  # XXX is it want we want?
 
         trigger_ids = self.search([
             ('model.model', '=', model_name),
@@ -135,7 +140,6 @@ class Trigger(ModelSQL, ModelView):
         :param record: a BrowseRecord of the tested model
         :return: a boolean
         """
-        model_obj = self.pool.get(trigger.model.model)
         env = {}
         env['current_date'] = datetime.datetime.today()
         env['time'] = time
@@ -150,9 +154,10 @@ class Trigger(ModelSQL, ModelView):
         :param ids: the list of record ids triggered
         :param trigger_id: the trigger id
         """
-        trigger_log_obj = self.pool.get('ir.trigger.log')
+        pool = Pool()
+        trigger_log_obj = pool.get('ir.trigger.log')
         trigger = self.browse(trigger_id)
-        model_obj = self.pool.get(trigger.action_model.model)
+        model_obj = pool.get(trigger.action_model.model)
         cursor = Transaction().cursor
 
         # Filter on limit_number
@@ -197,13 +202,15 @@ class Trigger(ModelSQL, ModelView):
                         datepart, timepart = delay[record_id].split(" ")
                         year, month, day = map(int, datepart.split("-"))
                         timepart_full = timepart.split(".")
-                        hours, minutes, seconds = map(int, timepart_full[0].split(":"))
+                        hours, minutes, seconds = map(int,
+                            timepart_full[0].split(":"))
                         if len(timepart_full) == 2:
                             microseconds = int(timepart_full[1])
                         delay[record_id] = datetime.datetime(year, month, day,
                                 hours, minutes, seconds, microseconds)
-                    if datetime.datetime.now() - delay[record_id] \
-                            >= datetime.timedelta(minutes=trigger.minimum_delay):
+                    if (datetime.datetime.now() - delay[record_id]
+                            >= datetime.timedelta(
+                                minutes=trigger.minimum_delay)):
                         new_ids.append(record_id)
             ids = new_ids
 
@@ -220,11 +227,12 @@ class Trigger(ModelSQL, ModelView):
         '''
         Trigger time actions
         '''
+        pool = Pool()
         trigger_ids = self.search([
             ('on_time', '=', True),
             ])
         for trigger in self.browse(trigger_ids):
-            model_obj = self.pool.get(trigger.model.model)
+            model_obj = pool.get(trigger.model.model)
             triggered_ids = []
             # TODO add a domain
             record_ids = model_obj.search([])
