@@ -6,10 +6,11 @@ from lxml import etree
 from difflib import SequenceMatcher
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.backend import TableHandler
-from trytond.pyson import PYSONEncoder, CONTEXT, Eval
+from trytond.pyson import CONTEXT, Eval
 from trytond.tools import safe_eval
 from trytond.transaction import Transaction
-from trytond.wizard import Wizard
+from trytond.wizard import Wizard, StateView, StateAction, StateTransition, \
+    Button
 from trytond.pool import Pool
 
 
@@ -18,17 +19,17 @@ class View(ModelSQL, ModelView):
     _name = 'ir.ui.view'
     _description = __doc__
     _rec_name = 'model'
-    model = fields.Char('Model', required=True, select=1)
-    priority = fields.Integer('Priority', required=True, select=1)
+    model = fields.Char('Model', required=True, select=True)
+    priority = fields.Integer('Priority', required=True, select=True)
     type = fields.Selection([
-        (False, ''),
-        ('tree','Tree'),
-        ('form','Form'),
-        ('graph', 'Graph'),
-        ('board', 'Board'),
-        ], 'View Type', select=1)
+            (None, ''),
+            ('tree', 'Tree'),
+            ('form', 'Form'),
+            ('graph', 'Graph'),
+            ('board', 'Board'),
+            ], 'View Type', select=True)
     arch = fields.Text('View Architecture')
-    inherit = fields.Many2One('ir.ui.view', 'Inherited View', select=1,
+    inherit = fields.Many2One('ir.ui.view', 'Inherited View', select=True,
             ondelete='CASCADE')
     field_childs = fields.Char('Children Field', states={
             'invisible': Eval('type') != 'tree',
@@ -96,8 +97,8 @@ class View(ModelSQL, ModelView):
                     logger = logging.getLogger('ir')
                     error_log = reduce(lambda x, y: str(x) + '\n' + str(y),
                             validator.error_log.filter_from_errors())
-                    logger.error(
-                        'Invalid xml view:\n%s' %  (str(error_log) + '\n' + xml))
+                    logger.error('Invalid xml view:\n%s'
+                        % (str(error_log) + '\n' + xml))
                     return False
             root_element = tree.getroottree().getroot()
 
@@ -109,6 +110,7 @@ class View(ModelSQL, ModelView):
                 'digits': fields.digits_validate,
                 'add_remove': fields.add_remove_validate,
             }
+
             def encode(element):
                 for attr in ('states', 'domain', 'context', 'digits',
                         'add_remove', 'spell', 'colors'):
@@ -118,10 +120,10 @@ class View(ModelSQL, ModelView):
                             validates.get(attr, lambda a: True)(value)
                         except Exception, e:
                             logger = logging.getLogger('ir')
-                            logger.error('Invalid pyson view element "%s:%s":' \
-                                    '\n%s\n%s' % \
-                                    (element.get('id') or element.get('name'),
-                                        attr, str(e), xml))
+                            logger.error('Invalid pyson view element "%s:%s":'
+                                '\n%s\n%s'
+                                % (element.get('id') or element.get('name'),
+                                    attr, str(e), xml))
                             return False
                 for child in element:
                     if not encode(child):
@@ -233,53 +235,38 @@ class View(ModelSQL, ModelView):
 View()
 
 
-class ShowViewInit(ModelView):
-    'Show view init'
-    _name = 'ir.ui.view.show.init'
+class ShowViewStart(ModelView):
+    'Show view'
+    _name = 'ir.ui.view.show.start'
 
-ShowViewInit()
+ShowViewStart()
 
 
 class ShowView(Wizard):
     'Show view'
     _name = 'ir.ui.view.show'
 
-    states = {
-        'init': {
-            'result': {
-                'type': 'form',
-                'object': 'ir.ui.view.show.init',
-                'state': [
-                    ('end', 'Close', 'tryton-cancel', True),
-                    ],
-                },
-            },
-        }
+    class ShowStateView(StateView):
 
-    def __init__(self):
-        super(ShowView, self).__init__()
-        self._error_messages.update({
-                'view_type': 'Only "form" view can be shown!',
-                })
+        def __init__(self, model_name, buttons):
+            StateView.__init__(self, model_name, None, buttons)
 
-    def execute(self, wiz_id, data, state='init'):
-        pool = Pool()
-        view_obj = pool.get('ir.ui.view')
-        result = super(ShowView, self).execute(wiz_id, data, state=state)
-        view = view_obj.browse(data['id'])
-        view_id = view.id
-        view_type = view.type
-        if view.inherit:
-            view_id = view.inherit.id
-            view_type = view.inherit.type
-        if view_type != 'form':
-            self.raise_user_error('view_type')
-        model_obj = pool.get(view.model)
-        fields_view = model_obj.fields_view_get(view_id=view.id)
-        result['fields'] = fields_view['fields']
-        result['arch'] = fields_view['arch']
-        result['object'] = view.model
-        return result
+        def get_view(self):
+            pool = Pool()
+            view_obj = pool.get('ir.ui.view')
+            view_id = Transaction().context.get('active_id')
+            view = view_obj.browse(view_id)
+            model_obj = pool.get(view.model)
+            if view.type != 'form':
+                return model_obj.fields_view_get(view_type='form')
+            return model_obj.fields_view_get(view_id=view_id)
+
+        def get_defaults(self, wizard, session, state_name, fields):
+            return {}
+
+    start = ShowStateView('ir.ui.view.show.start', [
+            Button('Close', 'end', 'tryton-close', default=True),
+            ])
 
 ShowView()
 
@@ -290,7 +277,7 @@ class ViewShortcut(ModelSQL, ModelView):
     _description = __doc__
     name = fields.Char('Shortcut Name', required=True)
     res_id = fields.Integer('Resource Ref.', required=True)
-    sequence = fields.Integer('Sequence')
+    sequence = fields.Integer('Sequence', required=True)
     user_id = fields.Many2One('res.user', 'User Ref.', required=True,
        ondelete='CASCADE')
     resource = fields.Char('Resource Name', required=True)
@@ -304,9 +291,9 @@ class ViewShortcut(ModelSQL, ModelView):
         "Provide user's shortcuts"
         result = []
         ids = self.search([
-            ('user_id','=',user_id),
-            ('resource','=',model),
-            ])
+                ('user_id', '=', user_id),
+                ('resource', '=', model),
+                ])
         for shorcut in self.browse(ids):
             result.append({
                     'res_id': shorcut.res_id,
@@ -320,57 +307,31 @@ class ViewShortcut(ModelSQL, ModelView):
 ViewShortcut()
 
 
-class AddShortcut(Wizard):
-    'Add shortcut'
-    _name = 'ir.ui.view_sc.add'
-
-    states = {
-        'init': {
-            'result': {
-                'type': 'action',
-                'action': '_add_shortcut',
-                'state': 'end',
-            },
-        },
-    }
-
-    def _add_shortcut(self, data):
-        pool = Pool()
-        view_sc_obj = pool.get('ir.ui.view_sc')
-        model_obj = pool.get(data['model'])
-
-        record = model_obj.browse(data['id'])
-        view_sc_obj.create({
-            'name': record.rec_name,
-            'res_id': record.id,
-            'user_id': Transaction().user,
-            'resource': model_obj._name,
-            })
-        return {}
-
-AddShortcut()
-
-
 class OpenShortcut(Wizard):
     'Open a shortcut'
     _name = 'ir.ui.view_sc.open'
 
-    states = {
-        'init': {
-            'result': {
-                'type': 'action',
-                'action': '_open',
-                'state': 'end',
-            }
-        }
-    }
+    start_state = 'open_'
 
-    def _open(self, data):
+    class OpenStateAction(StateAction):
+        def __init__(self):
+            StateAction.__init__(self, None)
+
+        def get_action(self):
+            pass
+
+    open_ = OpenStateAction()
+
+    def transition_open_(self, session):
+        return 'end'
+
+    def do_open_(self, session, action):
         pool = Pool()
         view_sc_obj = pool.get('ir.ui.view_sc')
         action_keyword_obj = pool.get('ir.action.keyword')
+        action_obj = pool.get('ir.action')
 
-        view_sc = view_sc_obj.browse(data['id'])
+        view_sc = view_sc_obj.browse(Transaction().context.get('active_id'))
         models = (
                 '%s,%d' % (view_sc.resource, view_sc.res_id),
                 '%s,0' % (view_sc.resource),
@@ -384,15 +345,10 @@ class OpenShortcut(Wizard):
             if action_keyword_ids:
                 break
         if not action_keyword_ids:
-            return {}
+            return {}, {}
         action_keyword = action_keyword_obj.browse(action_keyword_ids[0])
-        action_obj = pool.get(action_keyword.action.type)
-        action_ids = action_obj.search([
-            ('action.id', '=', action_keyword.action.id),
-            ])
-        if not action_ids:
-            return {}
-        return action_obj.read(action_ids[0])
+        return action_obj.get_action_values(action_keyword.action.type,
+            action_keyword.action.id), {}
 
 OpenShortcut()
 
@@ -402,11 +358,11 @@ class ViewTreeWidth(ModelSQL, ModelView):
     _name = 'ir.ui.view_tree_width'
     _description = __doc__
     _rec_name = 'model'
-    model = fields.Char('Model', required=True, select=1)
-    field = fields.Char('Field', required=True, select=1)
+    model = fields.Char('Model', required=True, select=True)
+    field = fields.Char('Field', required=True, select=True)
     user = fields.Many2One('res.user', 'User', required=True,
-            ondelete='CASCADE', select=1)
-    width = fields.Integer('Width')
+            ondelete='CASCADE', select=True)
+    width = fields.Integer('Width', required=True)
 
     def __init__(self):
         super(ViewTreeWidth, self).__init__()
