@@ -2,6 +2,7 @@
 #this repository contains the full copyright notices and license terms.
 import contextlib
 from itertools import islice, ifilter, ifilterfalse, chain
+from trytond.config import CONFIG
 from trytond.model import fields
 from trytond.transaction import Transaction
 from trytond.pool import Pool
@@ -29,16 +30,16 @@ class BrowseRecordNull(object):
     '''
 
     def __init__(self):
-        self.id = False
+        self.id = None
 
     def __getitem__(self, name):
-        return False
+        return None
 
     def __int__(self):
-        return False
+        return None
 
     def __str__(self):
-        return ''
+        return None
 
     def __nonzero__(self):
         return False
@@ -113,10 +114,12 @@ class BrowseRecord(object):
                 fname, field = item
                 return (fname not in self._data.get(self.id, {})
                     and fname not in self._local_data.get(self.id, {}))
+
             def to_load(item):
                 fname, field = item
                 return (field.loading == 'eager'
                     and fname not in to_remove)
+
             def overrided(item):
                 fname, field = item
                 return fname in self._model._columns
@@ -196,9 +199,7 @@ class BrowseRecord(object):
                                     LRUDict(RECORD_CACHE_SIZE))
                                 data[i] = BrowseRecord(data[i], model,
                                     ids, local_cache)
-                    elif (model
-                            and j._type in ('one2many', 'many2many')
-                            and len(data[i])):
+                    elif (model and j._type in ('one2many', 'many2many')):
                         _datetime = None
                         if hasattr(j, 'datetime_field') and j.datetime_field:
                             _datetime = data[j.datetime_field]
@@ -216,7 +217,8 @@ class BrowseRecord(object):
                                     BrowseRecordList))):
                         if data['id'] == self.id and i == name:
                             result = data[i]
-                        self._local_data.setdefault(data['id'], {})[i] = data[i]
+                        self._local_data.setdefault(data['id'], {})[i] = \
+                            data[i]
                         del data[i]
                 self._data.setdefault(data['id'], {}).update(data)
                 if data['id'] == self.id and name in data:
@@ -262,7 +264,7 @@ class BrowseRecord(object):
 
     def setLang(self, lang):
         self._context = self._context.copy()
-        prev_lang = self._context.get('language') or 'en_US'
+        prev_lang = self._context.get('language') or CONFIG['language']
         self._context['language'] = lang
         for cache in (self._cache, {self._model_name: self._local_data}):
             language_cache = cache.setdefault('_language_cache', {})
