@@ -4,18 +4,21 @@
 from trytond.pyson import PYSON
 from trytond.const import OPERATORS
 
+
 def domain_validate(value):
     assert isinstance(value, list), 'domain must be a list'
+
     def test_domain(dom):
         for arg in dom:
             if isinstance(arg, basestring):
                 if arg not in ('AND', 'OR'):
                     return False
-            elif isinstance(arg, tuple) or \
-                    (isinstance(arg, list) and len(arg) > 2 and \
-                    (arg[1] in OPERATORS) or \
-                    (isinstance(arg[1], PYSON) and \
-                    arg[1].types() == set([str]))):
+            elif (isinstance(arg, tuple)
+                or (isinstance(arg, list)
+                    and len(arg) > 2
+                    and ((arg[1] in OPERATORS)
+                        or (isinstance(arg[1], PYSON)
+                            and arg[1].types() == set([str]))))):
                 pass
             elif isinstance(arg, list):
                 if not test_domain(arg):
@@ -23,6 +26,7 @@ def domain_validate(value):
         return True
     assert test_domain(value), 'invalid domain'
 
+
 def states_validate(value):
     assert isinstance(value, dict), 'states must be a dict'
     for state in value:
@@ -34,17 +38,21 @@ def states_validate(value):
             assert value[state].types() == set([bool]), \
                     'values of states must return boolean'
 
+
 def on_change_validate(value):
     if value:
         assert isinstance(value, list), 'on_change must be a list'
 
+
 def on_change_with_validate(value):
     if value:
         assert isinstance(value, list), 'on_change_with must be a list'
 
+
 def depends_validate(value):
     assert isinstance(value, list), 'depends must be a list'
 
+
 def context_validate(value):
     assert isinstance(value, dict), 'context must be a dict'
 
@@ -53,9 +61,9 @@ class Field(object):
     _type = None
 
     def __init__(self, string='', help='', required=False, readonly=False,
-            domain=None, states=None, change_default=False, select=0,
-            on_change=None, on_change_with=None, depends=None,
-            order_field=None, context=None, loading='eager'):
+            domain=None, states=None, select=False, on_change=None,
+            on_change_with=None, depends=None, order_field=None, context=None,
+            loading='eager'):
         '''
         :param string: A string for label of the field.
         :param help: A multi-line help string.
@@ -67,18 +75,14 @@ class Field(object):
             ``readonly`` and ``invisible``. Values are pyson expressions that
             will be evaluated with record values. This allows to change
             dynamically the attributes of the field.
-        :param change_default: A boolean. If ``True`` the field can be used as
-        condition for a custom default value.
-        :param select: An integer. When equal to ``1``, the field appears in the
-            search box in list view and the search will be optimized. When equal to
-            ``2`` the field appears in the *Advanced Search* in the search box.
+        :param select: An boolean. When True search will be optimized.
         :param on_change: A list of values. If set, the client will call the
             method ``on_change_<field_name>`` when the user changes the field
             value. It then passes this list of values as arguments to the
             function.
-        :param on_change_with: A list of values. Like ``on_change``, but defined
-            the other way around. The list contains all the fields that must
-            update the current field.
+        :param on_change_with: A list of values. Like ``on_change``, but
+            defined the other way around. The list contains all the fields that
+            must update the current field.
         :param depends: A list of field name on which this one depends.
         :param order_field: A string. If set it will use the string when
             ordering records on the field.
@@ -96,8 +100,7 @@ class Field(object):
         self.domain = domain or []
         self.__states = None
         self.states = states or {}
-        self.change_default = change_default
-        self.select = select
+        self.select = bool(select)
         self.__on_change = None
         self.on_change = on_change
         self.__on_change_with = None
@@ -107,7 +110,8 @@ class Field(object):
         self.order_field = order_field
         self.__context = None
         self.context = context or {}
-        assert loading in ('lazy', 'eager'), 'loading must be "lazy" or "eager"'
+        assert loading in ('lazy', 'eager'), \
+            'loading must be "lazy" or "eager"'
         self.loading = loading
 
     def _get_domain(self):
