@@ -4,9 +4,6 @@
 import inspect
 import copy
 from trytond.model.fields.field import Field
-from trytond.model.fields.float import digits_validate
-from trytond.model.fields.one2many import add_remove_validate
-from trytond.pyson import PYSON
 
 
 class Function(Field):
@@ -32,7 +29,8 @@ class Function(Field):
         if not self.setter:
             self._field.readonly = True
         self.searcher = searcher
-        assert loading in ('lazy', 'eager'), 'loading must be "lazy" or "eager"'
+        assert loading in ('lazy', 'eager'), \
+            'loading must be "lazy" or "eager"'
         self.loading = loading
 
     __init__.__doc__ += Field.__init__.__doc__
@@ -94,7 +92,6 @@ class Function(Field):
                 name = [name]
             return getattr(model, self.getter)(ids, name)
 
-
     def set(self, ids, model, name, value):
         '''
         Call the setter.
