@@ -12,8 +12,8 @@ class Many2One(Field):
 
     def __init__(self, model_name, string='', left=None, right=None,
             ondelete='SET NULL', datetime_field=None, help='', required=False,
-            readonly=False, domain=None, states=None, change_default=False,
-            select=False, on_change=None, on_change_with=None, depends=None,
+            readonly=False, domain=None, states=None, select=False,
+            on_change=None, on_change_with=None, depends=None,
             order_field=None, context=None, loading='eager'):
         '''
         :param model_name: The name of the target model.
@@ -37,11 +37,10 @@ class Many2One(Field):
             else:
                 depends = [datetime_field]
         super(Many2One, self).__init__(string=string, help=help,
-                required=required, readonly=readonly, domain=domain,
-                states=states, change_default=change_default, select=select,
-                on_change=on_change, on_change_with=on_change_with,
-                depends=depends, order_field=order_field, context=context,
-                loading=loading)
+            required=required, readonly=readonly, domain=domain, states=states,
+            select=select, on_change=on_change, on_change_with=on_change_with,
+            depends=depends, order_field=order_field, context=context,
+            loading=loading)
         self.model_name = model_name
         self.left = left
         self.right = right
