@@ -12,9 +12,8 @@ class Selection(Field):
 
     def __init__(self, selection, string='', sort=True, translate=True,
             help='', required=False, readonly=False, domain=None, states=None,
-            change_default=False, select=0, on_change=None,
-            on_change_with=None, depends=None, order_field=None, context=None,
-            loading='eager'):
+            select=False, on_change=None, on_change_with=None, depends=None,
+            order_field=None, context=None, loading='eager'):
         '''
         :param selection: A list or a function name that returns a list.
             The list must be a list of tuples. First member is the value
@@ -22,11 +21,10 @@ class Selection(Field):
         :param sort: A boolean to sort or not the selections.
         '''
         super(Selection, self).__init__(string=string,  help=help,
-                required=required, readonly=readonly, domain=domain,
-                states=states, change_default=change_default, select=select,
-                on_change=on_change, on_change_with=on_change_with,
-                depends=depends, order_field=order_field, context=context,
-                loading=loading)
+            required=required, readonly=readonly, domain=domain, states=states,
+            select=select, on_change=on_change, on_change_with=on_change_with,
+            depends=depends, order_field=order_field, context=context,
+            loading=loading)
         if hasattr(selection, 'copy'):
             self.selection = selection.copy()
         else:
