@@ -19,7 +19,6 @@ class Model(WarningErrorMixin, URLMixin):
     _inherits = {}
     _description = ''
     __columns = None
-    __xxx2many_targets = None
     __defaults = None
     _rec_name = 'name'
 
@@ -27,7 +26,7 @@ class Model(WarningErrorMixin, URLMixin):
 
     def _reset_columns(self):
         self.__columns = None
-        self._reset_xxx2many_targets()
+        self.__defaults = None
 
     def _getcolumns(self):
         if self.__columns:
@@ -70,6 +69,7 @@ class Model(WarningErrorMixin, URLMixin):
         self._rpc = {
             'default_get': False,
             'fields_get': False,
+            'on_change_with': False,
         }
         self._inherit_fields = []
         self._error_messages = {}
@@ -85,7 +85,9 @@ class Model(WarningErrorMixin, URLMixin):
             assert (k in self._columns) or (k in self._inherit_fields), \
             'Default function defined in %s but field %s does not exist!' % \
                 (self._name, k,)
+        self._update_rpc()
 
+    def _update_rpc(self):
         for field_name in self._columns.keys() + self._inherit_fields.keys():
             if field_name in self._columns:
                 field = self._columns[field_name]
@@ -111,7 +113,7 @@ class Model(WarningErrorMixin, URLMixin):
             if (hasattr(model_obj, name)
                     and isinstance(getattr(model_obj, name),
                         collections.Callable)):
-               return getattr(model_obj, name)
+                return getattr(model_obj, name)
         raise AttributeError(name)
 
     def _inherits_reload(self):
@@ -129,62 +131,14 @@ class Model(WarningErrorMixin, URLMixin):
                 res[field_name] = (model, self._inherits[model],
                         pool.get(model)._inherit_fields[field_name][2])
         self._inherit_fields = res
-        self._reset_xxx2many_targets()
+        self._reset_columns()
+        self._update_rpc()
         # Update objects that uses this one to update their _inherits fields
         for obj_name in pool.object_name_list():
             obj = pool.get(obj_name)
             if self._name in obj._inherits:
                 obj._inherits_reload()
 
-    def _reset_xxx2many_targets(self):
-        self.__xxx2many_targets = None
-
-    def _getxxx2many_targets(self):
-        if self.__xxx2many_targets:
-            return self.__xxx2many_targets
-        to_cache = True
-
-        res = [(field_name, field.model_name)
-                for field_name, field in self._columns.iteritems()
-                if field._type == 'one2many']
-
-        for field_name, field in self._columns.iteritems():
-            if field._type != 'many2many':
-                continue
-            if hasattr(field, 'get_target'):
-                try:
-                    model_name = field.get_target()._name
-                except KeyError:
-                    to_cache = False
-                    continue
-            else:
-                model_name = field.model_name
-            res.append((field_name, model_name))
-
-        res += [(field_name, field.model_name)
-                for _, (_, field_name, field) in \
-                        self._inherit_fields.iteritems()
-                if field._type == 'one2many']
-
-        for _, (_, field_name, field) in self._inherit_fields.iteritems():
-            if field._type != 'many2many':
-                continue
-            if hasattr(field, 'get_target'):
-                try:
-                    model_name = field.get_target()._name
-                except KeyError:
-                    to_cache = False
-                    continue
-            else:
-                model_name = field.model_name
-            res.append((field_name, model_name))
-
-        if to_cache:
-            self.__xxx2many_targets = res
-        return res
-
-    _xxx2many_targets = property(fget=_getxxx2many_targets)
-
     def init(self, module_name):
         """
         Add model in ir.model and ir.model.field.
@@ -221,12 +175,12 @@ class Model(WarningErrorMixin, URLMixin):
 
         # Update translation of model
         for name, src in [(self._name + ',name', self._description)]:
-            cursor.execute('SELECT id FROM ir_translation ' \
-                    'WHERE lang = %s ' \
-                        'AND type = %s ' \
-                        'AND name = %s ' \
-                        'AND res_id = %s',
-                    ('en_US', 'model', name, 0))
+            cursor.execute('SELECT id FROM ir_translation '
+                'WHERE lang = %s '
+                    'AND type = %s '
+                    'AND name = %s '
+                    'AND (res_id IS NULL OR res_id = %s)',
+                ('en_US', 'model', name, 0))
             trans_id = None
             if cursor.rowcount == -1 or cursor.rowcount is None:
                 data = cursor.fetchone()
@@ -235,7 +189,7 @@ class Model(WarningErrorMixin, URLMixin):
             elif cursor.rowcount != 0:
                 trans_id, = cursor.fetchone()
             src_md5 = translation_obj.get_src_md5(src)
-            if not trans_id:
+            if trans_id is None:
                 cursor.execute('INSERT INTO ir_translation '
                     '(name, lang, type, src, src_md5, value, module, fuzzy) '
                     'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
@@ -296,10 +250,10 @@ class Model(WarningErrorMixin, URLMixin):
                         "VALUES (%s, %s, %s, %s, %s, %s, %s)",
                         (model_id, field_name, field.string, field._type,
                             relation, field.help, module_name))
-            elif model_fields[field_name]['field_description'] != field.string \
-                    or model_fields[field_name]['ttype'] != field._type \
-                    or model_fields[field_name]['relation'] != relation \
-                    or model_fields[field_name]['help'] != field.help:
+            elif (model_fields[field_name]['field_description'] != field.string
+                    or model_fields[field_name]['ttype'] != field._type
+                    or model_fields[field_name]['relation'] != relation
+                    or model_fields[field_name]['help'] != field.help):
                 cursor.execute('UPDATE ir_model_field ' \
                         'SET field_description = %s, ' \
                             'ttype = %s, ' \
@@ -311,13 +265,11 @@ class Model(WarningErrorMixin, URLMixin):
             trans_name = self._name + ',' + field_name
             string_md5 = translation_obj.get_src_md5(field.string)
             if trans_name not in trans_fields:
-                if field_name not in ('create_uid', 'create_date',
-                            'write_uid', 'write_date', 'id'):
-                    cursor.execute('INSERT INTO ir_translation '
-                        '(name, lang, type, src, src_md5, value, module, fuzzy) '
-                        'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
-                        (trans_name, 'en_US', 'field', field.string,
-                            string_md5, '', module_name, False))
+                cursor.execute('INSERT INTO ir_translation '
+                    '(name, lang, type, src, src_md5, value, module, fuzzy) '
+                    'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
+                    (trans_name, 'en_US', 'field', field.string,
+                        string_md5, '', module_name, False))
             elif trans_fields[trans_name]['src'] != field.string:
                 cursor.execute('UPDATE ir_translation '
                     'SET src = %s, src_md5 = %s '
@@ -327,7 +279,8 @@ class Model(WarningErrorMixin, URLMixin):
             if trans_name not in trans_help:
                 if field.help:
                     cursor.execute('INSERT INTO ir_translation '
-                        '(name, lang, type, src, src_md5, value, module, fuzzy) '
+                        '(name, lang, type, src, src_md5, value, module, '
+                            'fuzzy) '
                         'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
                         (trans_name, 'en_US', 'help', field.help, help_md5, '',
                             module_name, False))
@@ -475,7 +428,7 @@ class Model(WarningErrorMixin, URLMixin):
                 raise_exception=False)
 
         #Add translation to cache
-        language = Transaction().context.get('language') or 'en_US'
+        language = Transaction().language
         trans_args = []
         for field in (x for x in self._columns.keys()
                 if ((not fields_names) or x in fields_names)):
@@ -494,7 +447,7 @@ class Model(WarningErrorMixin, URLMixin):
                     for (key, val) in sel:
                         trans_args.append((self._name + ',' + field,
                             'selection', language, val))
-        translation_obj._get_sources(trans_args)
+        translation_obj.get_sources(trans_args)
 
         encoder = PYSONEncoder()
 
@@ -512,7 +465,6 @@ class Model(WarningErrorMixin, URLMixin):
                     'states',
                     'size',
                     'required',
-                    'change_default',
                     'translate',
                     'help',
                     'select',
@@ -552,12 +504,12 @@ class Model(WarningErrorMixin, URLMixin):
 
             if Transaction().context.get('language'):
                 # translate the field label
-                res_trans = translation_obj._get_source(
+                res_trans = translation_obj.get_source(
                         self._name + ',' + field, 'field',
                         Transaction().context['language'])
                 if res_trans:
                     res[field]['string'] = res_trans
-                help_trans = translation_obj._get_source(
+                help_trans = translation_obj.get_source(
                         self._name + ',' + field, 'help',
                         Transaction().context['language'])
                 if help_trans:
@@ -575,7 +527,7 @@ class Model(WarningErrorMixin, URLMixin):
                         # translate each selection option
                         sel2 = []
                         for (key, val) in sel:
-                            val2 = translation_obj._get_source(
+                            val2 = translation_obj.get_source(
                                     self._name + ',' + field, 'selection',
                                     language, val)
                             sel2.append((key, val2 or val))
@@ -605,7 +557,8 @@ class Model(WarningErrorMixin, URLMixin):
                         self._columns[field].field)
 
             # convert attributes into pyson
-            for attr in ('states', 'domain', 'context', 'digits', 'add_remove'):
+            for attr in ('states', 'domain', 'context', 'digits',
+                    'add_remove'):
                 if attr in res[field]:
                     res[field][attr] = encoder.encode(res[field][attr])
 
@@ -615,3 +568,10 @@ class Model(WarningErrorMixin, URLMixin):
                 if i not in fields_names:
                     del res[i]
         return res
+
+    def on_change_with(self, fieldnames, values):
+        changes = {}
+        for fieldname in fieldnames:
+            changes[fieldname] = getattr(self,
+                'on_change_with_%s' % fieldname)(values)
+        return changes
