@@ -1,7 +1,6 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 
-from __future__ import with_statement
 import datetime
 import time
 from decimal import Decimal
@@ -12,6 +11,9 @@ try:
 except ImportError:
     import StringIO
 import csv
+from functools import reduce
+import traceback
+import sys
 from trytond.model import Model
 from trytond.model import fields
 from trytond.model.browse import BrowseRecordList, BrowseRecord, \
@@ -19,8 +21,12 @@ from trytond.model.browse import BrowseRecordList, BrowseRecord, \
 from trytond.model.browse import EvalEnvironment
 from trytond.tools import safe_eval, reduce_domain
 from trytond.pyson import PYSONEncoder, PYSONDecoder, PYSON
-from trytond.const import OPERATORS
+from trytond.const import OPERATORS, RECORD_CACHE_SIZE
 from trytond.transaction import Transaction
+from trytond.pool import Pool
+from trytond.cache import LRUDict
+from trytond.config import CONFIG
+from .modelview import ModelView
 
 
 class ModelStorage(Model):
@@ -37,18 +43,19 @@ class ModelStorage(Model):
 
     def __init__(self):
         super(ModelStorage, self).__init__()
-        self._rpc.update({
-            'create': True,
-            'read': False,
-            'write': True,
-            'delete': True,
-            'copy': True,
-            'search': False,
-            'search_count': False,
-            'search_read': False,
-            'export_data': False,
-            'import_data': True,
-        })
+        if isinstance(self, ModelView):
+            self._rpc.update({
+                'create': True,
+                'read': False,
+                'write': True,
+                'delete': True,
+                'copy': True,
+                'search': False,
+                'search_count': False,
+                'search_read': False,
+                'export_data': False,
+                'import_data': True,
+            })
         self._constraints = []
 
     def default_create_uid(self):
@@ -59,19 +66,16 @@ class ModelStorage(Model):
         "Default value for create_date field."
         return datetime.datetime.today()
 
-    def __clean_xxx2many_cache(self):
-        # Clean cursor cache
-        to_clean = [(model._name, field_name)
-                for model_name, model in self.pool.iterobject(type='model')
-                for field_name, target_name in model._xxx2many_targets
-                if target_name == self._name]
-        for cache in Transaction().cursor.cache.values():
-            for cache in (cache, cache.get('_language_cache', {}).values()):
-                for model_name, field_name in to_clean:
-                    if model_name in cache:
-                        for model_id in cache[model_name]:
-                            if field_name in cache[model_name][model_id]:
-                                del cache[model_name][model_id][field_name]
+    def __fields_by_model(self, values):
+        "Return a dictionary with list of fields from values grouped by model"
+        fields_by_model = {}
+        for field in values:
+            if field in self._columns:
+                model = self._name
+            else:
+                model = self._inherit_fields[field][0]
+            fields_by_model.setdefault(model, []).append(field)
+        return fields_by_model
 
     def create(self, values):
         '''
@@ -81,13 +85,13 @@ class ModelStorage(Model):
                 and created values as value
         :return: the id of the created record
         '''
-        model_access_obj = self.pool.get('ir.model.access')
-        model_field_access_obj = self.pool.get('ir.model.field.access')
+        pool = Pool()
+        model_access_obj = pool.get('ir.model.access')
+        model_field_access_obj = pool.get('ir.model.field.access')
 
         model_access_obj.check(self._name, 'create')
-        model_field_access_obj.check(self._name,
-                [x for x in values if x in self._columns], 'write')
-        self.__clean_xxx2many_cache()
+        for model, fields in self.__fields_by_model(values).iteritems():
+            model_field_access_obj.check(model, fields, 'write')
         return False
 
     def trigger_create(self, id):
@@ -96,7 +100,7 @@ class ModelStorage(Model):
 
         :param id: the created id
         '''
-        trigger_obj = self.pool.get('ir.trigger')
+        trigger_obj = Pool().get('ir.trigger')
         trigger_ids = trigger_obj.get_triggers(self._name, 'create')
         if not trigger_ids:
             return
@@ -116,8 +120,9 @@ class ModelStorage(Model):
             the dictionnaries will have fields names as key
             and fields value as value. The list will not be in the same order.
         '''
-        model_access_obj = self.pool.get('ir.model.access')
-        model_field_access_obj = self.pool.get('ir.model.field.access')
+        pool = Pool()
+        model_access_obj = pool.get('ir.model.access')
+        model_field_access_obj = pool.get('ir.model.field.access')
 
         model_access_obj.check(self._name, 'read')
         model_field_access_obj.check(self._name,
@@ -135,12 +140,13 @@ class ModelStorage(Model):
                 and written values as value
         :return: True if succeed
         '''
-        model_access_obj = self.pool.get('ir.model.access')
-        model_field_access_obj = self.pool.get('ir.model.field.access')
+        pool = Pool()
+        model_access_obj = pool.get('ir.model.access')
+        model_field_access_obj = pool.get('ir.model.field.access')
 
         model_access_obj.check(self._name, 'write')
-        model_field_access_obj.check(self._name,
-                [x for x in values if x in self._columns], 'write')
+        for model, fields in self.__fields_by_model(values).iteritems():
+            model_field_access_obj.check(model, fields, 'write')
         if not self.check_xml_record(ids, values):
             self.raise_user_error('write_xml_record',
                     error_description='xml_record_desc')
@@ -154,8 +160,6 @@ class ModelStorage(Model):
                     for i in ids:
                         if i in cache[self._name]:
                             cache[self._name][i] = {}
-        if ids:
-            self.__clean_xxx2many_cache()
         return False
 
     def trigger_write_get_eligibles(self, ids):
@@ -165,7 +169,7 @@ class ModelStorage(Model):
         :param ids: a list of ids
         :return: a dictionary of the lists of eligible ids by triggers
         '''
-        trigger_obj = self.pool.get('ir.trigger')
+        trigger_obj = Pool().get('ir.trigger')
         trigger_ids = trigger_obj.get_triggers(self._name, 'write')
         if not trigger_ids:
             return {}
@@ -185,11 +189,11 @@ class ModelStorage(Model):
 
         :param eligibles: a dictionary of the lists of eligible ids by triggers
         '''
-        trigger_obj = self.pool.get('ir.trigger')
+        trigger_obj = Pool().get('ir.trigger')
         trigger_ids = eligibles.keys()
         if not trigger_ids:
             return
-        records = self.browse(chain(*eligibles.values()))
+        records = self.browse(list(chain(*eligibles.values())))
         id2record = dict((x.id, x) for x in records)
         triggers = trigger_obj.browse(trigger_ids)
         for trigger in triggers:
@@ -208,7 +212,7 @@ class ModelStorage(Model):
         :param ids: a list of ids or an id
         :return: True if succeed
         '''
-        model_access_obj = self.pool.get('ir.model.access')
+        model_access_obj = Pool().get('ir.model.access')
 
         model_access_obj.check(self._name, 'delete')
         if not self.check_xml_record(ids, None):
@@ -224,8 +228,6 @@ class ModelStorage(Model):
                     for i in ids:
                         if i in cache[self._name]:
                             del cache[self._name][i]
-        if ids:
-            self.__clean_xxx2many_cache()
         return False
 
     def trigger_delete(self, ids):
@@ -234,7 +236,7 @@ class ModelStorage(Model):
 
         :param ids: the deleted ids
         '''
-        trigger_obj = self.pool.get('ir.trigger')
+        trigger_obj = Pool().get('ir.trigger')
         trigger_ids = trigger_obj.get_triggers(self._name, 'delete')
         if not trigger_ids:
             return
@@ -257,7 +259,8 @@ class ModelStorage(Model):
             new value for the field as value
         :return: a list of new ids or the new id
         '''
-        lang_obj = self.pool.get('ir.lang')
+        pool = Pool()
+        lang_obj = pool.get('ir.lang')
         if default is None:
             default = {}
 
@@ -314,7 +317,7 @@ class ModelStorage(Model):
             data, data_o2m = convert_data(field_defs, data)
             new_ids[data_id] = self.create(data)
             for field_name in data_o2m:
-                relation_model = self.pool.get(
+                relation_model = pool.get(
                         field_defs[field_name]['relation'])
                 relation_field = field_defs[field_name]['relation_field']
                 if relation_field:
@@ -333,7 +336,7 @@ class ModelStorage(Model):
                 ])
             if lang_ids:
                 lang_ids += lang_obj.search([
-                    ('code', '=', 'en_US'),
+                    ('code', '=', CONFIG['language']),
                     ])
                 langs = lang_obj.browse(lang_ids)
                 for lang in langs:
@@ -467,9 +470,9 @@ class ModelStorage(Model):
                 if domain and ((isinstance(domain[0], basestring) \
                         and domain[0] == 'AND') \
                         or (not isinstance(domain[0], basestring))):
-                    domain.append(('active', '=', 1))
+                    domain.append(('active', '=', True))
                 else:
-                    domain = ['AND', domain, ('active', '=', 1)]
+                    domain = ['AND', domain, ('active', '=', True)]
             return domain
         return process(domain)
 
@@ -486,8 +489,8 @@ class ModelStorage(Model):
             return {}
         res = {}
         rec_name = self._rec_name
-        if rec_name not in self._columns \
-                and rec_name not in self._inherit_fields.keys():
+        if (rec_name not in self._columns
+                and rec_name not in self._inherit_fields):
             rec_name = 'id'
         for record in self.browse(ids):
             res[record.id] = unicode(record[rec_name])
@@ -501,7 +504,11 @@ class ModelStorage(Model):
         :param clause: a domain clause
         :return: a list of domain clause
         '''
-        return [(self._rec_name,) + clause[1:]]
+        rec_name = self._rec_name
+        if (rec_name not in self._columns
+                and rec_name not in self._inherit_fields):
+            return []
+        return [(rec_name,) + clause[1:]]
 
     def browse(self, ids):
         '''
@@ -511,13 +518,14 @@ class ModelStorage(Model):
         :param ids: a list of ids or an id
         :return: a BrowseRecordList or a BrowseRecord
         '''
+        local_cache = LRUDict(RECORD_CACHE_SIZE)
         if isinstance(ids, (int, long)):
-            return BrowseRecord(ids, self)
-        local_cache = {}
-        return BrowseRecordList((BrowseRecord(x, self, local_cache=local_cache)
+            return BrowseRecord(ids, self, [ids], local_cache)
+        return BrowseRecordList((BrowseRecord(x, self, ids, local_cache)
             for x in ids))
 
     def __export_row(self, record, fields_names):
+        pool = Pool()
         lines = []
         data = ['' for x in range(len(fields_names))]
         done = []
@@ -531,8 +539,14 @@ class ModelStorage(Model):
                 if not isinstance(value, BrowseRecord):
                     break
                 field_name = fields_tree[i]
-                model_obj = self.pool.get(value._model_name)
-                field = model_obj._columns[field_name]
+                model_obj = pool.get(value._model_name)
+                if field_name in model_obj._columns:
+                    field = model_obj._columns[field_name]
+                elif field_name in model_obj._inherit_fields:
+                    field = model_obj._inherit_fields[field_name][2]
+                else:
+                    raise Exception('Field %s not available on object "%s"'
+                        % (field_name, model_obj._name))
                 if field.states and 'invisible' in field.states:
                     pyson_invisible = PYSONEncoder().encode(
                             field.states['invisible'])
@@ -603,6 +617,8 @@ class ModelStorage(Model):
             - the exception if failed
             - the warning if failed
         '''
+        pool = Pool()
+
         def process_lines(self, datas, prefix, fields_def, position=0):
 
             def warn(msgname, *args):
@@ -611,7 +627,7 @@ class ModelStorage(Model):
                 warnings.warn(msg)
 
             def get_selection(selection, value):
-                res = False
+                res = None
                 if not isinstance(selection, (tuple, list)):
                     selection = getattr(self, selection)()
                 for key, _ in selection:
@@ -624,26 +640,26 @@ class ModelStorage(Model):
 
             def get_many2one(relation, value):
                 if not value:
-                    return False
-                relation_obj = self.pool.get(relation)
+                    return None
+                relation_obj = pool.get(relation)
                 res = relation_obj.search([
                     ('rec_name', '=', value),
                     ], limit=2)
                 if len(res) < 1:
                     warn('relation_not_found', value, relation)
-                    res = False
+                    res = None
                 elif len(res) > 1:
                     warn('too_many_relations_found', value, relation)
-                    res = False
+                    res = None
                 else:
                     res = res[0]
                 return res
 
             def get_many2many(relation, value):
                 if not value:
-                    return False
+                    return None
                 res = []
-                relation_obj = self.pool.get(relation)
+                relation_obj = pool.get(relation)
                 for word in csv.reader(StringIO.StringIO(value), delimiter=',',
                         quoting=csv.QUOTE_NONE, escapechar='\\').next():
                     res2 = relation_obj.search([
@@ -664,29 +680,29 @@ class ModelStorage(Model):
 
             def get_reference(value):
                 if not value:
-                    return False
+                    return None
                 try:
                     relation, value = value.split(',', 1)
                 except Exception:
                     warn('reference_syntax_error', value, '/'.join(field))
-                    return False
-                relation_obj = self.pool.get(relation)
+                    return None
+                relation_obj = pool.get(relation)
                 res = relation_obj.search([
                     ('rec_name', '=', value),
                     ], limit=2)
                 if len(res) < 1:
                     warn('relation_not_found', value, relation)
-                    res = False
+                    res = None
                 elif len(res) > 1:
                     warn('too_many_relations_found', value, relation)
-                    res = False
+                    res = None
                 else:
                     res = '%s,%s' % (relation, str(res[0]))
                 return res
 
             def get_by_id(value):
                 if not value:
-                    return False
+                    return None
                 relation = None
                 ftype = fields_def[field[-1][:-3]]['type']
                 if ftype == 'many2many':
@@ -697,7 +713,7 @@ class ModelStorage(Model):
                         relation, value = value.split(',', 1)
                     except Exception:
                         warn('reference_syntax_error', value, '/'.join(field))
-                        return False
+                        return None
                     value = [value]
                 else:
                     value = [value]
@@ -725,8 +741,8 @@ class ModelStorage(Model):
             for i, field in enumerate(fields_names):
                 if i >= len(line):
                     raise Exception('ImportError',
-                            'Please check that all your lines have %d cols.' % \
-                            (len(fields_names),))
+                        'Please check that all your lines have %d cols.'
+                        % len(fields_names))
                 is_prefix_len = (len(field) == (prefix_len + 1))
                 value = line[i]
                 if is_prefix_len and field[-1].endswith(':id'):
@@ -737,20 +753,22 @@ class ModelStorage(Model):
                 elif is_prefix_len and prefix == field[:-1]:
                     this_field_def = fields_def[field[-1]]
                     field_type = this_field_def['type']
-                    res = False
+                    res = None
                     if field_type == 'boolean':
                         if value.lower() == 'true':
                             res = True
                         elif value.lower() == 'false':
                             res = False
+                        elif not value:
+                            res = False
                         else:
-                            res = value and bool(int(value))
+                            res = bool(int(value))
                     elif field_type == 'integer':
-                        res = value and int(value)
+                        res = value and int(value) or None
                     elif field_type == 'float':
-                        res = value and float(value)
+                        res = value and float(value) or None
                     elif field_type == 'numeric':
-                        res = value and Decimal(value)
+                        res = value and Decimal(value) or None
                     elif field_type == 'date':
                         res = value and datetime.date(*time.strptime(value,
                             '%Y-%m-%d')[:3])
@@ -768,14 +786,14 @@ class ModelStorage(Model):
                     elif field_type == 'reference':
                         res = get_reference(value)
                     else:
-                        res = value or False
+                        res = value or None
                     row[field[-1]] = res
                 elif prefix == field[0:prefix_len]:
                     todo.add(field[prefix_len])
             # Import one2many fields
             nbrmax = 1
             for field in todo:
-                newfd = self.pool.get(fields_def[field]['relation']
+                newfd = pool.get(fields_def[field]['relation']
                         ).fields_get()
                 res = process_lines(self, datas, prefix + [field], newfd,
                         position)
@@ -806,7 +824,7 @@ class ModelStorage(Model):
                     datas.pop(0)
             return (row, nbrmax, translate)
 
-        ir_model_data_obj = self.pool.get('ir.model.data')
+        ir_model_data_obj = pool.get('ir.model.data')
 
         # logger for collecting warnings for the client
         warnings = logging.Logger("import")
@@ -840,7 +858,8 @@ class ModelStorage(Model):
                 logger.error(exp)
                 # XXX should raise Exception
                 Transaction().cursor.rollback()
-                warning = '\n'.join(map(str, exp[1:]) + [warning])
+                tb_s = ''.join(traceback.format_exception(*sys.exc_info()))
+                warning = '%s\n%s' % (tb_s, warning)
                 return (-1, res, exp, warning)
             done += 1
         return (done, 0, 0, 0)
@@ -861,7 +880,7 @@ class ModelStorage(Model):
             written values as value
         :return: True or False
         """
-        model_data_obj = self.pool.get('ir.model.data')
+        model_data_obj = Pool().get('ir.model.data')
         # Allow root user to update/delete
         if Transaction().user == 0:
             return True
@@ -934,7 +953,8 @@ class ModelStorage(Model):
         return True
 
     def _get_error_args(self, field_name):
-        model_field_obj = self.pool.get('ir.model.field')
+        pool = Pool()
+        model_field_obj = pool.get('ir.model.field')
         error_args = (field_name, self._name)
         if model_field_obj:
             model_field_ids = model_field_obj.search([
@@ -947,8 +967,8 @@ class ModelStorage(Model):
                         model_field.model.name)
         return error_args
 
-
     def _validate(self, ids):
+        pool = Pool()
         if (Transaction().user == 0
                 and Transaction().context.get('user')):
             with Transaction().set_user(Transaction().context.get('user')):
@@ -958,12 +978,12 @@ class ModelStorage(Model):
             if not getattr(self, field[0])(ids):
                 self.raise_user_error(field[1])
 
-        if not 'res.user' in self.pool.object_name_list() \
+        if not 'res.user' in pool.object_name_list() \
                 or Transaction().user == 0:
             ctx_pref = {
             }
         else:
-            user_obj = self.pool.get('res.user')
+            user_obj = pool.get('res.user')
             ctx_pref = user_obj.get_preferences(context_only=True)
 
         def is_pyson(test):
@@ -989,9 +1009,9 @@ class ModelStorage(Model):
                         ('many2one', 'many2many', 'one2many', 'one2one')
                     and field.domain):
                     if field._type in ('many2one', 'one2many'):
-                        relation_obj = self.pool.get(field.model_name)
+                        relation_obj = pool.get(field.model_name)
                     else:
-                        relation_obj = field.get_target(self.pool)
+                        relation_obj = field.get_target()
                     if is_pyson(field.domain):
                         pyson_domain = PYSONEncoder().encode(field.domain)
                         for record in records:
@@ -1051,7 +1071,7 @@ class ModelStorage(Model):
                             env['context'] = Transaction().context
                             env['active_id'] = record.id
                             required = PYSONDecoder(env).decode(pyson_required)
-                            if required and not record[field_name]:
+                            if required and record[field_name] is None:
                                 self.raise_user_error(
                                         'required_validation_record',
                                         error_args=self._get_error_args(
@@ -1067,28 +1087,31 @@ class ModelStorage(Model):
                 # validate required
                 if field.required:
                     for record in records:
-                        if isinstance(record[field_name], (BrowseRecordNull,
-                            type(None), type(False))) and not record[field_name]:
+                        if (isinstance(record[field_name], (BrowseRecordNull,
+                                        type(None), type(False)))
+                                and not record[field_name]):
                             self.raise_user_error(
-                                    'required_validation_record',
-                                    error_args=self._get_error_args(field_name))
+                                'required_validation_record',
+                                error_args=self._get_error_args(field_name))
                 # validate size
                 if hasattr(field, 'size') and field.size:
                     for record in records:
                         if len(record[field_name] or '') > field.size:
                             self.raise_user_error(
-                                    'size_validation_record',
-                                    error_args=self._get_error_args(field_name))
+                                'size_validation_record',
+                                error_args=self._get_error_args(field_name))
 
                 def digits_test(value, digits, field_name):
                     def raise_user_error():
                         self.raise_user_error('digits_validation_record',
                             error_args=self._get_error_args(field_name))
+                    if value is None:
+                        return
                     if isinstance(value, Decimal):
-                        if not (value.quantize(Decimal(str(10.0**-digits[1])))
-                                == value):
+                        if (value.quantize(Decimal(str(10.0 ** -digits[1])))
+                                != value):
                             raise_user_error()
-                    else:
+                    elif CONFIG.options['db_type'] != 'mysql':
                         if not (round(value, digits[1]) == float(value)):
                             raise_user_error()
                 # validate digits
@@ -1110,6 +1133,7 @@ class ModelStorage(Model):
                                 field_name)
 
     def _clean_defaults(self, defaults):
+        pool = Pool()
         vals = {}
         for field in defaults.keys():
             fld_def = (field in self._columns) and self._columns[field] \
@@ -1120,7 +1144,7 @@ class ModelStorage(Model):
                 else:
                     vals[field] = defaults[field]
             elif fld_def._type in ('one2many',):
-                obj = self.pool.get(fld_def.model_name)
+                obj = pool.get(fld_def.model_name)
                 vals[field] = []
                 for defaults2 in defaults[field]:
                     vals2 = obj._clean_defaults(defaults2)
@@ -1132,25 +1156,3 @@ class ModelStorage(Model):
             else:
                 vals[field] = defaults[field]
         return vals
-
-    def workflow_trigger_trigger(self, ids):
-        '''
-        Trigger a trigger event.
-
-        :param ids: a list of id or an id
-        '''
-        trigger_obj = self.pool.get('workflow.trigger')
-        instance_obj = self.pool.get('workflow.instance')
-
-        if isinstance(ids, (int, long)):
-            ids = [ids]
-
-        with Transaction().set_user(0):
-            trigger_ids = trigger_obj.search([
-                ('res_id', 'in', ids),
-                ('model', '=', self._name),
-                ])
-            instances = set([trigger.instance for trigger in
-                trigger_obj.browse(trigger_ids)])
-        for instance in instances:
-            instance_obj.update(instance)
