@@ -6,13 +6,15 @@ try:
 except ImportError:
     hashlib = None
     import md5
-import copy
+from functools import wraps
 from trytond.model import Model
 from trytond.tools import safe_eval
 from trytond.pyson import PYSONEncoder, CONTEXT
 from trytond.transaction import Transaction
 from trytond.cache import Cache
 from trytond.pool import Pool
+from trytond.exceptions import UserError
+
 
 def _find(tree, element):
     if element.tag == 'xpath':
@@ -76,7 +78,7 @@ class ModelView(Model):
     """
     Define a model with views in Tryton.
     """
-    __modules_list = None # Cache for the modules list sorted by dependency
+    __modules_list = None  # Cache for the modules list sorted by dependency
 
     @staticmethod
     def _reset_modules_list():
@@ -96,6 +98,7 @@ class ModelView(Model):
         super(ModelView, self).__init__()
         self._rpc['fields_view_get'] = False
         self._rpc['view_toolbar_get'] = False
+        self._buttons = {}
 
     @Cache('modelview.fields_view_get')
     def fields_view_get(self, view_id=None, view_type='form', hexmd5=None):
@@ -231,6 +234,7 @@ class ModelView(Model):
             return True
         return result
 
+    @Cache('modelview.view_toolbar_get')
     def view_toolbar_get(self):
         """
         Returns the model specific actions.
@@ -384,10 +388,12 @@ class ModelView(Model):
     def __view_look_dom(self, element, type, fields_width=None):
         pool = Pool()
         translation_obj = pool.get('ir.translation')
+        model_data_obj = pool.get('ir.model.data')
+        button_obj = pool.get('ir.model.button')
+        user_obj = pool.get('res.user')
 
         if fields_width is None:
             fields_width = {}
-        result = False
         fields_attrs = {}
         childs = True
 
@@ -410,41 +416,40 @@ class ModelView(Model):
                     if relation and element.tag == 'field':
                         childs = False
                         views = {}
-                        for field in element:
-                            if field.tag in ('form', 'tree', 'graph'):
-                                field2 = copy.copy(field)
-
-                                def _translate_field(field):
-                                    if field.get('string'):
-                                        trans = translation_obj._get_source(
-                                                self._name, 'view',
-                                                Transaction().language,
-                                                field.get('string'))
-                                        if trans:
-                                            field.set('string', trans)
-                                    if field.get('sum'):
-                                        trans = translation_obj._get_source(
-                                                self._name, 'view',
-                                                Transaction().language,
-                                                field.get('sum'))
-                                        if trans:
-                                            field.set('sum', trans)
-                                    for field_child in field:
-                                        _translate_field(field_child)
-                                if Transaction().language != 'en_US':
-                                    _translate_field(field2)
-
-                                relation_obj = pool.get(relation)
-                                if hasattr(relation_obj, '_view_look_dom_arch'):
-                                    xarch, xfields = \
-                                            relation_obj._view_look_dom_arch(
-                                                    field2, field.tag)
-                                    views[field.tag] = {
-                                        'arch': xarch,
-                                        'fields': xfields
-                                    }
-                                element.remove(field)
-                        attrs = {'views': views}
+                        mode = (element.attrib.pop('mode', None)
+                            or 'tree,form').split(',')
+                        view_ids = []
+                        if element.get('view_ids'):
+                            for view_id in element.get('view_ids').split(','):
+                                try:
+                                    view_ids.append(int(view_id))
+                                except ValueError:
+                                    view_ids.append(model_data_obj.get_id(
+                                            *view_id.split('.')))
+                        relation_obj = pool.get(relation)
+                        if (not len(element)
+                                and type == 'form'
+                                and field._type in ('one2many', 'many2many')):
+                            # Prefetch only the first view to prevent infinite
+                            # loop
+                            if view_ids:
+                                for view_id in view_ids:
+                                    view = relation_obj.fields_view_get(
+                                        view_id=view_id)
+                                    views[view['type']] = view
+                                    break
+                            else:
+                                for view_type in mode:
+                                    views[view_type] = \
+                                        relation_obj.fields_view_get(
+                                            view_type=view_type)
+                                    break
+                        element.attrib['mode'] = ','.join(mode)
+                        element.attrib['view_ids'] = ','.join(
+                            map(str, view_ids))
+                        attrs = {
+                            'views': views,
+                            }
                     fields_attrs[element.get(attr)] = attrs
             if element.get('name') in fields_width:
                 element.set('width', str(fields_width[element.get('name')]))
@@ -457,11 +462,26 @@ class ModelView(Model):
                 element.set(attr, encoder.encode(safe_eval(element.get(attr),
                     CONTEXT)))
 
+        if element.tag == 'button':
+            if element.get('type', 'object') == 'object':
+                assert not element.get('states')
+                button_name = element.attrib['name']
+                if button_name in self._buttons:
+                    states = self._buttons[button_name]
+                else:
+                    states = {}
+                groups = set(user_obj.get_groups())
+                button_groups = button_obj.get_groups(self._name, button_name)
+                if button_groups and not groups & button_groups:
+                    states = states.copy()
+                    states['readonly'] = True
+                element.set('states', encoder.encode(states))
+
         # translate view
-        if Transaction().language != 'en_US' and not result:
+        if Transaction().language != 'en_US':
             for attr in ('string', 'sum', 'confirm', 'help'):
                 if element.get(attr):
-                    trans = translation_obj._get_source(self._name, 'view',
+                    trans = translation_obj.get_source(self._name, 'view',
                             Transaction().language, element.get(attr))
                     if trans:
                         element.set(attr, trans)
@@ -479,3 +499,21 @@ class ModelView(Model):
                 fields_attrs.update(self.__view_look_dom(field, type,
                     fields_width=fields_width))
         return fields_attrs
+
+    @staticmethod
+    def button(func):
+        @wraps(func)
+        def wrapper(self, *args, **kwargs):
+            pool = Pool()
+            button_obj = pool.get('ir.model.button')
+            user_obj = pool.get('res.user')
+
+            if Transaction().user != 0:
+                groups = set(user_obj.get_groups())
+                button_groups = button_obj.get_groups(self._name,
+                    func.__name__)
+                if button_groups and not groups & button_groups:
+                    raise UserError('Calling button %s on %s is not allowed!'
+                        % (func.__name__, self._name))
+            return func(self, *args, **kwargs)
+        return wrapper
