@@ -3,8 +3,8 @@
 from trytond.protocols.sslsocket import SSLSocket
 from trytond.protocols.dispatcher import dispatch
 from trytond.config import CONFIG
-from trytond.protocols.datatype import Float
-from trytond.protocols.common import daemon, GZipRequestHandlerMixin
+from trytond.protocols.common import daemon, GZipRequestHandlerMixin, \
+    RegisterHandlerMixin
 from trytond.exceptions import UserError, UserWarning, NotLogged, \
     ConcurrencyException
 import SimpleXMLRPCServer
@@ -27,6 +27,12 @@ try:
 except ImportError:
     import json
 import base64
+import encodings
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from StringIO import StringIO
+
 
 def object_hook(dct):
     if '__class__' in dct:
@@ -35,12 +41,15 @@ def object_hook(dct):
                     dct['hour'], dct['minute'], dct['second'])
         elif dct['__class__'] == 'date':
             return datetime.date(dct['year'], dct['month'], dct['day'])
+        elif dct['__class__'] == 'time':
+            return datetime.time(dct['hour'], dct['minute'], dct['second'])
         elif dct['__class__'] == 'buffer':
             return buffer(base64.decodestring(dct['base64']))
         elif dct['__class__'] == 'Decimal':
             return Decimal(dct['decimal'])
     return dct
 
+
 class JSONEncoder(json.JSONEncoder):
 
     def __init__(self, *args, **kwargs):
@@ -64,6 +73,12 @@ class JSONEncoder(json.JSONEncoder):
                     'month': obj.month,
                     'day': obj.day,
                     }
+        elif isinstance(obj, datetime.time):
+            return {'__class__': 'time',
+                'hour': obj.hour,
+                'minute': obj.minute,
+                'second': obj.second,
+                }
         elif isinstance(obj, buffer):
             return {'__class__': 'buffer',
                 'base64': base64.encodestring(obj),
@@ -112,13 +127,7 @@ class SimpleJSONRPCDispatcher(SimpleXMLRPCServer.SimpleXMLRPCDispatcher):
                 ConcurrencyException), exception:
             response['error'] = exception.args
         except Exception:
-            tb_s = ''
-            for line in traceback.format_exception(*sys.exc_info()):
-                try:
-                    line = line.encode('utf-8', 'ignore')
-                except Exception:
-                    continue
-                tb_s += line
+            tb_s = ''.join(traceback.format_exception(*sys.exc_info()))
             for path in sys.path:
                 tb_s = tb_s.replace(path, '')
             if CONFIG['debug_mode']:
@@ -149,6 +158,7 @@ class GenericJSONRPCRequestHandler:
 
 
 class SimpleJSONRPCRequestHandler(GZipRequestHandlerMixin,
+        RegisterHandlerMixin,
         GenericJSONRPCRequestHandler,
         SimpleXMLRPCServer.SimpleXMLRPCRequestHandler,
         SimpleHTTPServer.SimpleHTTPRequestHandler):
@@ -159,7 +169,7 @@ class SimpleJSONRPCRequestHandler(GZipRequestHandlerMixin,
     """
     protocol_version = "HTTP/1.1"
     rpc_paths = None
-    encode_threshold = 1400 # common MTU
+    encode_threshold = 1400  # common MTU
 
     def send_header(self, keyword, value):
         if keyword == 'Content-type' and value == 'text/xml':
@@ -167,6 +177,18 @@ class SimpleJSONRPCRequestHandler(GZipRequestHandlerMixin,
         SimpleXMLRPCServer.SimpleXMLRPCRequestHandler.send_header(self,
             keyword, value)
 
+    def do_GET(self):
+        if self.is_tryton_url(self.path):
+            self.send_tryton_url(self.path)
+            return
+        SimpleHTTPServer.SimpleHTTPRequestHandler.do_GET(self)
+
+    def do_HEAD(self):
+        if self.is_tryton_url(self.path):
+            self.send_tryton_url(self.path)
+            return
+        SimpleHTTPServer.SimpleHTTPRequestHandler.do_HEAD(self)
+
     def translate_path(self, path):
         """Translate a /-separated PATH to the local filename syntax.
 
@@ -185,14 +207,55 @@ class SimpleJSONRPCRequestHandler(GZipRequestHandlerMixin,
         for word in words:
             drive, word = os.path.splitdrive(word)
             head, word = os.path.split(word)
-            if word in (os.curdir, os.pardir): continue
+            if word in (os.curdir, os.pardir):
+                continue
             path = os.path.join(path, word)
         return path
 
+    def is_tryton_url(self, path):
+        words = path.split('/')
+        try:
+            return words[2] in ('model', 'wizard', 'report')
+        except IndexError:
+            return False
+
+    def send_tryton_url(self, path):
+        self.send_response(300)
+        hostname = CONFIG['hostname'] or unicode(socket.getfqdn(), 'utf8')
+        hostname = '.'.join(encodings.idna.ToASCII(part) for part in
+            hostname.split('.'))
+        values = {
+            'hostname': hostname,
+            'path': path,
+            }
+        content = StringIO()
+        content.write('<html')
+        content.write('<head>')
+        content.write('<meta http-equiv="Refresh" '
+            'content="0;url=tryton://%(hostname)s%(path)s"/>' % values)
+        content.write('<title>Moved</title>')
+        content.write('</head>')
+        content.write('<body>')
+        content.write('<h1>Moved</h1>')
+        content.write('<p>This page has moved to '
+            '<a href="tryton://%(hostname)s%(path)s">'
+            'tryton://%(hostname)s%(path)s</a>.</p>' % values)
+        content.write('</body>')
+        content.write('</html>')
+        length = content.tell()
+        content.seek(0)
+        self.send_header('Location', 'tryton://%(hostname)s%(path)s' % values)
+        self.send_header('Content-type', 'text/html')
+        self.send_header('Content-Length', str(length))
+        self.end_headers()
+        self.copyfile(content, self.wfile)
+        content.close()
+
 
 class SecureJSONRPCRequestHandler(SimpleJSONRPCRequestHandler):
 
     def setup(self):
+        self.server.handlers.add(self)
         self.connection = SSLSocket(self.request)
         self.rfile = socket._fileobject(self.request, "rb", self.rbufsize)
         self.wfile = socket._fileobject(self.request, "wb", self.wbufsize)
@@ -220,6 +283,7 @@ class SimpleJSONRPCServer(SocketServer.TCPServer,
     def __init__(self, addr, requestHandler=SimpleJSONRPCRequestHandler,
             logRequests=True, allow_none=False, encoding=None,
             bind_and_activate=True):
+        self.handlers = set()
         self.logRequests = logRequests
 
         SimpleJSONRPCDispatcher.__init__(self, allow_none, encoding)
@@ -237,6 +301,11 @@ class SimpleJSONRPCServer(SocketServer.TCPServer,
             flags |= fcntl.FD_CLOEXEC
             fcntl.fcntl(self.fileno(), fcntl.F_SETFD, flags)
 
+    def server_close(self):
+        SocketServer.TCPServer.server_close(self)
+        for handler in self.handlers:
+            self.shutdown_request(handler.request)
+
 
 class SimpleThreadedJSONRPCServer(SocketServer.ThreadingMixIn,
         SimpleJSONRPCServer):
@@ -246,6 +315,8 @@ class SimpleThreadedJSONRPCServer(SocketServer.ThreadingMixIn,
     def server_bind(self):
         self.socket.setsockopt(socket.SOL_SOCKET,
                 socket.SO_REUSEADDR, 1)
+        self.socket.setsockopt(socket.SOL_SOCKET,
+            socket.SO_KEEPALIVE, 1)
         SimpleJSONRPCServer.server_bind(self)
 
 
@@ -256,8 +327,8 @@ class SimpleThreadedJSONRPCServer6(SimpleThreadedJSONRPCServer):
 class SecureThreadedJSONRPCServer(SimpleThreadedJSONRPCServer):
 
     def __init__(self, server_address, HandlerClass, logRequests=1):
-        SimpleThreadedJSONRPCServer.__init__(self, server_address, HandlerClass,
-                logRequests)
+        SimpleThreadedJSONRPCServer.__init__(self, server_address,
+            HandlerClass, logRequests)
         self.socket = SSLSocket(socket.socket(self.address_family,
             self.socket_type))
         self.server_bind()
