@@ -3,8 +3,8 @@
 from trytond.protocols.sslsocket import SSLSocket
 from trytond.protocols.dispatcher import dispatch
 from trytond.config import CONFIG
-from trytond.protocols.datatype import Float
-from trytond.protocols.common import daemon, GZipRequestHandlerMixin
+from trytond.protocols.common import daemon, GZipRequestHandlerMixin, \
+    RegisterHandlerMixin
 from trytond.exceptions import UserError, UserWarning, NotLogged, \
     ConcurrencyException
 from trytond import security
@@ -14,7 +14,6 @@ import xmlrpclib
 import traceback
 import socket
 import sys
-import os
 import base64
 import datetime
 from types import DictType
@@ -22,10 +21,13 @@ from types import DictType
 # convert decimal to float before marshalling:
 from decimal import Decimal
 
+
 def dump_decimal(self, value, write):
-    write("<value><double>")
-    write(str(value))
-    write("</double></value>\n")
+    value = {'__class__': 'Decimal',
+        'decimal': str(value),
+        }
+    self.dump_struct(value, write)
+
 
 def dump_buffer(self, value, write):
     self.write = write
@@ -33,15 +35,32 @@ def dump_buffer(self, value, write):
     value.encode(self)
     del self.write
 
+
+def dump_date(self, value, write):
+    value = {'__class__': 'date',
+        'year': value.year,
+        'month': value.month,
+        'day': value.day,
+        }
+    self.dump_struct(value, write)
+
+
+def dump_time(self, value, write):
+    value = {'__class__': 'time',
+        'hour': value.hour,
+        'minute': value.minute,
+        'second': value.second,
+        }
+    self.dump_struct(value, write)
+
 xmlrpclib.Marshaller.dispatch[Decimal] = dump_decimal
-xmlrpclib.Marshaller.dispatch[Float] = dump_decimal
 xmlrpclib.Marshaller.dispatch[type(None)] = \
         lambda self, value, write: self.dump_bool(bool(value), write)
-xmlrpclib.Marshaller.dispatch[datetime.date] = \
-        lambda self, value, write: self.dump_datetime(
-                datetime.datetime.combine(value, datetime.time()), write)
+xmlrpclib.Marshaller.dispatch[datetime.date] = dump_date
+xmlrpclib.Marshaller.dispatch[datetime.time] = dump_time
 xmlrpclib.Marshaller.dispatch[buffer] = dump_buffer
 
+
 def dump_struct(self, value, write, escape=xmlrpclib.escape):
     converted_value = {}
     for k, v in value.items():
@@ -55,10 +74,26 @@ def dump_struct(self, value, write, escape=xmlrpclib.escape):
 xmlrpclib.Marshaller.dispatch[DictType] = dump_struct
 
 
-def _end_double(self, data):
-    self.append(Float(data))
+def end_struct(self, data):
+    mark = self._marks.pop()
+    # map structs to Python dictionaries
+    dct = {}
+    items = self._stack[mark:]
+    for i in range(0, len(items), 2):
+        dct[xmlrpclib._stringify(items[i])] = items[i + 1]
+    if '__class__' in dct:
+        if dct['__class__'] == 'date':
+            dct = datetime.date(dct['year'], dct['month'], dct['day'])
+        elif dct['__class__'] == 'time':
+            dct = datetime.time(dct['hour'], dct['minute'], dct['second'])
+        elif dct['__class__'] == 'Decimal':
+            dct = Decimal(dct['decimal'])
+    self._stack[mark:] = [dct]
     self._value = 0
-xmlrpclib.Unmarshaller.dispatch["double"] = _end_double
+
+xmlrpclib.Unmarshaller.dispatch['struct'] = end_struct
+
+
 def _end_dateTime(self, data):
     value = xmlrpclib.DateTime()
     value.decode(data)
@@ -98,13 +133,7 @@ class GenericXMLRPCRequestHandler:
                 raise xmlrpclib.Fault(exception.code,
                     '\n'.join((error,) + description))
             except Exception:
-                tb_s = ''
-                for line in traceback.format_exception(*sys.exc_info()):
-                    try:
-                        line = line.encode('utf-8', 'ignore')
-                    except Exception:
-                        continue
-                    tb_s += line
+                tb_s = ''.join(traceback.format_exception(*sys.exc_info()))
                 for path in sys.path:
                     tb_s = tb_s.replace(path, '')
                 if CONFIG['debug_mode']:
@@ -117,14 +146,17 @@ class GenericXMLRPCRequestHandler:
 
 
 class SimpleXMLRPCRequestHandler(GZipRequestHandlerMixin,
+        RegisterHandlerMixin,
         GenericXMLRPCRequestHandler,
         SimpleXMLRPCServer.SimpleXMLRPCRequestHandler):
     protocol_version = "HTTP/1.1"
     rpc_paths = None
-    encode_threshold = 1400 # common MTU
+    encode_threshold = 1400  # common MTU
 
     def parse_request(self):
         res = SimpleXMLRPCServer.SimpleXMLRPCRequestHandler.parse_request(self)
+        if not res:
+            return res
         database_name = self.path[1:]
         if not database_name:
             self.tryton = {'user': None, 'session': None}
@@ -157,11 +189,23 @@ class SimpleThreadedXMLRPCServer(SocketServer.ThreadingMixIn,
     timeout = 1
     daemon_threads = True
 
+    def __init__(self, server_address, HandlerClass, logRequests=1):
+        self.handlers = set()
+        SimpleXMLRPCServer.SimpleXMLRPCServer.__init__(self, server_address,
+            HandlerClass, logRequests)
+
     def server_bind(self):
         self.socket.setsockopt(socket.SOL_SOCKET,
                 socket.SO_REUSEADDR, 1)
+        self.socket.setsockopt(socket.SOL_SOCKET,
+            socket.SO_KEEPALIVE, 1)
         SimpleXMLRPCServer.SimpleXMLRPCServer.server_bind(self)
 
+    def server_close(self):
+        SimpleXMLRPCServer.SimpleXMLRPCServer.server_close(self)
+        for handler in self.handlers:
+            self.shutdown_request(handler.request)
+
 
 class SimpleThreadedXMLRPCServer6(SimpleThreadedXMLRPCServer):
     address_family = socket.AF_INET6
