@@ -1,9 +1,5 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-import copy
-import xml
-import sys
-import base64
 try:
     import cStringIO as StringIO
 except ImportError:
@@ -12,32 +8,49 @@ import zipfile
 import time
 import os
 import datetime
-from base64 import decodestring
-import traceback
 import inspect
-import logging
 import tempfile
 import warnings
+import subprocess
 warnings.simplefilter("ignore")
 import relatorio.reporting
 warnings.resetwarnings()
 try:
     from relatorio.templates.opendocument import Manifest, MANIFEST
 except ImportError:
-    Manifest = None
+    Manifest, MANIFEST = None, None
 from genshi.filters import Translator
 import lxml.etree
 from trytond.config import CONFIG
-from trytond.backend import DatabaseIntegrityError
 from trytond.pool import Pool
 from trytond.transaction import Transaction
+from trytond.url import URLMixin
 
-PARENTS = {
-    'table-row': 1,
-    'list-item': 1,
-    'body': 0,
-    'section': 0,
-}
+MIMETYPES = {
+    'odt': 'application/vnd.oasis.opendocument.text',
+    'odp': 'application/vnd.oasis.opendocument.presentation',
+    'ods': 'application/vnd.oasis.opendocument.spreadsheet',
+    'odg': 'application/vnd.oasis.opendocument.graphics',
+    }
+FORMAT2EXT = {
+    'doc6': 'doc',
+    'doc95': 'doc',
+    'docbook': 'xml',
+    'ooxml': 'xml',
+    'latex': 'ltx',
+    'sdc4': 'sdc',
+    'sdc3': 'sdc',
+    'sdd3': 'sdd',
+    'sdd4': 'sdd',
+    'sdw4': 'sdw',
+    'sdw3': 'sdw',
+    'sxd3': 'sxd',
+    'sxd5': 'sxd',
+    'text': 'txt',
+    'xhtml': 'html',
+    'xls5': 'xls',
+    'xls95': 'xls',
+    }
 
 
 class ReportFactory:
@@ -77,7 +90,7 @@ class TranslateFactory:
         self.language = language
 
 
-class Report(object):
+class Report(URLMixin):
     _name = ""
 
     def __new__(cls):
@@ -100,11 +113,12 @@ class Report(object):
             local context of the report
         :return: a tuple with:
             report type,
-            base64 encoded data,
+            data,
             a boolean to direct print,
             the report name
         '''
-        action_report_obj = self.pool.get('ir.action.report')
+        pool = Pool()
+        action_report_obj = pool.get('ir.action.report')
         action_report_ids = action_report_obj.search([
             ('report_name', '=', self._name)
             ])
@@ -115,11 +129,12 @@ class Report(object):
         if action_report.model:
             objects = self._get_objects(ids, action_report.model, datas)
         type, data = self.parse(action_report, objects, datas, {})
-        return (type, base64.encodestring(data), action_report.direct_print,
+        return (type, buffer(data), action_report.direct_print,
                 action_report.name)
 
     def _get_objects(self, ids, model, datas):
-        model_obj = self.pool.get(model)
+        pool = Pool()
+        model_obj = pool.get(model)
         return model_obj.browse(ids)
 
     def parse(self, report, objects, datas, localcontext):
@@ -135,36 +150,45 @@ class Report(object):
             report type
             report
         '''
+        pool = Pool()
         localcontext['datas'] = datas
-        localcontext['user'] = self.pool.get('res.user'
+        localcontext['user'] = pool.get('res.user'
                 ).browse(Transaction().user)
         localcontext['formatLang'] = lambda *args, **kargs: \
                 self.format_lang(*args, **kargs)
-        localcontext['decodestring'] = decodestring
         localcontext['StringIO'] = StringIO.StringIO
         localcontext['time'] = time
         localcontext['datetime'] = datetime
         localcontext['context'] = Transaction().context
 
         translate = TranslateFactory(self._name, Transaction().language,
-                self.pool.get('ir.translation'))
-        localcontext['setLang'] = lambda language: translate.set_language(language)
+                pool.get('ir.translation'))
+        localcontext['setLang'] = lambda language: translate.set_language(
+            language)
 
-        if not report.report_content:
+        # Convert to str as buffer from DB is not supported by StringIO
+        report_content = (str(report.report_content) if report.report_content
+            else False)
+        style_content = (str(report.style_content) if report.style_content
+            else False)
+
+        if not report_content:
             raise Exception('Error', 'Missing report file!')
 
-        fd, path = tempfile.mkstemp(suffix='.odt', prefix='trytond')
+        fd, path = tempfile.mkstemp(
+            suffix=(os.extsep + report.template_extension),
+            prefix='trytond_')
         outzip = zipfile.ZipFile(path, mode='w')
 
         content_io = StringIO.StringIO()
-        content_io.write(base64.decodestring(report.report_content))
+        content_io.write(report_content)
         content_z = zipfile.ZipFile(content_io, mode='r')
 
         style_info = None
         style_xml = None
         manifest = None
         for f in content_z.infolist():
-            if f.filename == 'styles.xml' and report.style_content:
+            if f.filename == 'styles.xml' and style_content:
                 style_info = f
                 style_xml = content_z.read(f.filename)
                 continue
@@ -173,13 +197,14 @@ class Report(object):
                 continue
             outzip.writestr(f, content_z.read(f.filename))
 
-        if report.style_content:
+        if style_content:
             pictures = []
 
-            #cStringIO difference:
-            #calling StringIO() with a string parameter creates a read-only object
+            # cStringIO difference:
+            # calling StringIO() with a string parameter creates a read-only
+            # object
             new_style_io = StringIO.StringIO()
-            new_style_io.write(base64.decodestring(report.style_content))
+            new_style_io.write(style_content)
             new_style_z = zipfile.ZipFile(new_style_io, mode='r')
             new_style_xml = new_style_z.read('styles.xml')
             for file in new_style_z.namelist():
@@ -223,7 +248,8 @@ class Report(object):
         # Since Genshi >= 0.6, Translator requires a function type
         translator = Translator(lambda text: translate(text))
 
-        rel_report = relatorio.reporting.Report(path, 'application/vnd.oasis.opendocument.text',
+        mimetype = MIMETYPES[report.template_extension]
+        rel_report = relatorio.reporting.Report(path, mimetype,
                 ReportFactory(), relatorio.reporting.MIMETemplateLoader())
         rel_report.filters.insert(0, translator)
         #convert unicode key into str
@@ -239,65 +265,36 @@ class Report(object):
                 data = data.getvalue()
         os.close(fd)
         os.remove(path)
-        output_format = report.extension
-        if output_format == 'pdf':
-            data = self.convert_pdf(data)
-        return (output_format, data)
+        output_format = report.extension or report.template_extension
+        if output_format not in MIMETYPES:
+            data = self.unoconv(data, report.template_extension, output_format)
+        oext = FORMAT2EXT.get(output_format, output_format)
+        return (oext, data)
 
-    def convert_pdf(self, data):
-        """
-        Convert report to PDF using OpenOffice.org.
-        This requires OpenOffice.org, pyuno and openoffice-python to
-        be installed.
-        """
-        import tempfile
+    def unoconv(self, data, input_format, output_format):
+        '''
+        Call unoconv to convert the OpenDocument
+        '''
+        fd, path = tempfile.mkstemp(suffix=(os.extsep + input_format),
+            prefix='trytond_')
+        oext = FORMAT2EXT.get(output_format, output_format)
+        with os.fdopen(fd, 'wb+') as fp:
+            fp.write(data)
+        cmd = ['unoconv', '--connection=%s' % CONFIG['unoconv'],
+            '-f', oext, '--stdout', path]
         try:
-            import unohelper # installs import-hook
-            import openoffice.interact
-            import openoffice.officehelper as officehelper
-            from openoffice.streams import OutputStream
-            from com.sun.star.beans import PropertyValue
-        except ImportError, exception:
-            raise Exception('ImportError', str(exception))
-        try:
-            # connect to OOo
-            desktop = openoffice.interact.Desktop()
-        except officehelper.BootstrapException:
-            raise Exception('Error', "Can't connect to (bootstrap) OpenOffice.org")
-
-        res_data = None
-        # Create temporary file (with name) and write data there.
-        # We can not use NamedTemporaryFile here, since this would be
-        # deleted as soon as we close it to allow OOo reading.
-        #TODO use an input stream here
-        fd_odt, odt_name = tempfile.mkstemp()
-        fh_odt = os.fdopen(fd_odt, 'wb+')
-        try:
-            fh_odt.write(data)
-            del data # save memory
-            fh_odt.close()
-            doc = desktop.openFile(odt_name, hidden=False)
-            # Export as PDF
-            buffer = StringIO.StringIO()
-            out_props = (
-                PropertyValue("FilterName", 0, "writer_pdf_Export", 0),
-                PropertyValue("Overwrite", 0, True, 0),
-                PropertyValue("OutputStream", 0, OutputStream(buffer), 0),
-                )
-            doc.storeToURL("private:stream", out_props)
-            res_data = buffer.getvalue()
-            del buffer
-            doc.dispose()
+            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE)
+            stdoutdata, stderrdata = proc.communicate()
+            if proc.wait() != 0:
+                raise Exception(stderrdata)
+            return stdoutdata
         finally:
-            fh_odt.close()
-            os.remove(odt_name)
-        if not res_data:
-            Exception('Error', 'Error converting to PDF')
-        return res_data
+            os.remove(path)
 
     def format_lang(self, value, lang, digits=2, grouping=True, monetary=False,
             date=False, currency=None, symbol=True):
-        lang_obj = self.pool.get('ir.lang')
+        pool = Pool()
+        lang_obj = pool.get('ir.lang')
 
         if date:
             if lang:
@@ -305,7 +302,7 @@ class Report(object):
                 code = lang.code
             else:
                 locale_format = lang_obj.default_date()
-                code = lang_obj.default_code()
+                code = CONFIG['language']
             if not isinstance(value, time.struct_time):
                 # assume string, parse it
                 if len(str(value)) == 10:
