@@ -9,6 +9,8 @@
     'xml': [
         'import_data.xml',
         'sequence.xml',
+        'workflow.xml',
+        'wizard.xml',
     ],
     'translation': [
     ],
