@@ -2,13 +2,20 @@
 # -*- coding: utf-8 -*-
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
+import sys
+try:
+    import cdecimal
+    if 'decimal' not in sys.modules:
+        sys.modules['decimal'] = cdecimal
+except ImportError:
+    import decimal
+    sys.modules['cdecimal'] = decimal
 import unittest
 import datetime
 from decimal import Decimal
 from trytond.tests.test_tryton import POOL, DB_NAME, USER, CONTEXT, \
         install_module
 from trytond.transaction import Transaction
-from trytond.protocols.datatype import Float
 
 
 class FieldsTestCase(unittest.TestCase):
@@ -59,10 +66,16 @@ class FieldsTestCase(unittest.TestCase):
         self.datetime_default = POOL.get('test.datetime_default')
         self.datetime_required = POOL.get('test.datetime_required')
 
+        self.time = POOL.get('test.time')
+        self.time_default = POOL.get('test.time_default')
+        self.time_required = POOL.get('test.time_required')
+
         self.one2one = POOL.get('test.one2one')
         self.one2one_target = POOL.get('test.one2one.target')
         self.one2one_required = POOL.get('test.one2one_required')
 
+        self.property_ = POOL.get('test.property')
+
     def test0010boolean(self):
         '''
         Test Boolean.
@@ -307,7 +320,7 @@ class FieldsTestCase(unittest.TestCase):
             self.assert_(integer3_id)
 
             integer3 = self.integer.read(integer3_id, ['integer'])
-            self.assert_(integer3['integer'] == 0)
+            self.assert_(integer3['integer'] is None)
 
             integer4_id = self.integer_default.create({})
             self.assert_(integer4_id)
@@ -335,7 +348,14 @@ class FieldsTestCase(unittest.TestCase):
                 'integer': 'test',
                 })
 
-            integer5_id = self.integer_required.create({})
+            # We should catch UserError but mysql does not raise an
+            # IntegrityError but an OperationalError
+            self.assertRaises(Exception, self.integer_required.create, {})
+            transaction.cursor.rollback()
+
+            integer5_id = self.integer_required.create({
+                    'integer': 0,
+                    })
             self.assert_(integer5_id)
 
             integer5 = self.integer_required.read(integer5_id, ['integer'])
@@ -494,7 +514,7 @@ class FieldsTestCase(unittest.TestCase):
             self.assert_(float3_id)
 
             float3 = self.float.read(float3_id, ['float'])
-            self.assert_(float3['float'] == 0)
+            self.assert_(float3['float'] is None)
 
             float4_id = self.float_default.create({})
             self.assert_(float4_id)
@@ -522,11 +542,14 @@ class FieldsTestCase(unittest.TestCase):
                 'float': 'test',
                 })
 
-            float5_id = self.float_required.create({})
-            self.assert_(float5_id)
+            self.assertRaises(Exception, self.float_required.create, {})
+            transaction.cursor.rollback()
 
-            float5 = self.float_required.read(float5_id, ['float'])
-            self.assert_(float5['float'] == 0)
+            float5_id = self.float_required.create({
+                    'float': 0.0,
+                    })
+            float5 = self.float_required.read(float5_id)
+            self.assert_(float5['float'] == 0.0)
 
             float6_id = self.float_digits.create({
                 'digits': 1,
@@ -539,13 +562,15 @@ class FieldsTestCase(unittest.TestCase):
                 'float': 1.11,
                 })
 
-            self.failUnlessRaises(Exception, self.float_digits.write, float6_id, {
-                'float': 1.11,
-                })
+            self.failUnlessRaises(Exception, self.float_digits.write,
+                float6_id, {
+                    'float': 1.11,
+                    })
 
-            self.failUnlessRaises(Exception, self.float_digits.write, float6_id, {
-                'digits': 0,
-                })
+            self.failUnlessRaises(Exception, self.float_digits.write,
+                float6_id, {
+                    'digits': 0,
+                    })
 
             float7_id = self.float.create({
                 'float': 0.123456789012345,
@@ -554,13 +579,6 @@ class FieldsTestCase(unittest.TestCase):
             float7 = self.float.read(float7_id, ['float'])
             self.assert_(float7['float'] == 0.123456789012345)
 
-            float8_id = self.float.create({
-                'float': Float('1.1'),
-                })
-
-            float8 = self.float.read(float8_id, ['float'])
-            self.assertEqual(float8['float'], 1.1)
-
             transaction.cursor.rollback()
 
     def test0040numeric(self):
@@ -714,7 +732,7 @@ class FieldsTestCase(unittest.TestCase):
             self.assert_(numeric3_id)
 
             numeric3 = self.numeric.read(numeric3_id, ['numeric'])
-            self.assert_(numeric3['numeric'] == Decimal('0'))
+            self.assert_(numeric3['numeric'] is None)
 
             numeric4_id = self.numeric_default.create({})
             self.assert_(numeric4_id)
@@ -742,11 +760,14 @@ class FieldsTestCase(unittest.TestCase):
                 'numeric': 'test',
                 })
 
-            numeric5_id = self.numeric_required.create({})
-            self.assert_(numeric5_id)
+            self.assertRaises(Exception, self.numeric_required.create, {})
+            transaction.cursor.rollback()
 
-            numeric5 = self.numeric_required.read(numeric5_id, ['numeric'])
-            self.assert_(numeric5['numeric'] == Decimal('0'))
+            numeric5_id = self.numeric_required.create({
+                    'numeric': Decimal(0),
+                    })
+            numeric5 = self.numeric_required.read(numeric5_id)
+            self.assert_(numeric5['numeric'] == 0)
 
             numeric6_id = self.numeric_digits.create({
                 'digits': 1,
@@ -782,13 +803,6 @@ class FieldsTestCase(unittest.TestCase):
             self.assert_(numeric7['numeric'] ==
                     Decimal('0.1234567890123456789'))
 
-            numeric8_id = self.numeric.create({
-                'numeric': Float('1.1'),
-                })
-
-            numeric8 = self.numeric.read(numeric8_id, ['numeric'])
-            self.assertEqual(numeric8['numeric'], Decimal('1.1'))
-
             transaction.cursor.rollback()
 
     def test0050char(self):
@@ -957,7 +971,7 @@ class FieldsTestCase(unittest.TestCase):
                 self.assert_(char_ids == [char1_id])
 
                 char2_id = char.create({
-                    'char': False,
+                    'char': None,
                     })
                 self.assert_(char2_id)
 
@@ -992,7 +1006,7 @@ class FieldsTestCase(unittest.TestCase):
             self.assert_(char4['char'] == 'Test')
 
             self.char.write(char1_id, {
-                'char': False,
+                'char': None,
                 })
             char1 = self.char.read(char1_id, ['char'])
             self.assert_(char1['char'] == None)
@@ -1217,7 +1231,7 @@ class FieldsTestCase(unittest.TestCase):
                 self.assert_(text_ids == [text1_id])
 
                 text2_id = text.create({
-                    'text': False,
+                    'text': None,
                     })
                 self.assert_(text2_id)
 
@@ -1252,7 +1266,7 @@ class FieldsTestCase(unittest.TestCase):
             self.assert_(text4['text'] == 'Test')
 
             self.text.write(text1_id, {
-                'text': False,
+                'text': None,
                 })
             text1 = self.text.read(text1_id, ['text'])
             self.assert_(text1['text'] == None)
@@ -1327,7 +1341,8 @@ class FieldsTestCase(unittest.TestCase):
             self.assert_(sha1_id)
 
             sha1 = self.sha.read(sha1_id, ['sha'])
-            self.assert_(sha1['sha'] == '640ab2bae07bedc4c163f679a746f7ab7fb5d1fa')
+            self.assertEqual(sha1['sha'],
+                '640ab2bae07bedc4c163f679a746f7ab7fb5d1fa')
 
             sha_ids = self.sha.search([
                 ('sha', '=', 'Test'),
@@ -1400,12 +1415,7 @@ class FieldsTestCase(unittest.TestCase):
             self.assert_(sha_ids == [sha1_id])
 
             sha_ids = self.sha.search([
-                ('sha', 'like', '640ab2bae07bedc4c163f679a746f7ab7fb5d1fa'),
-                ])
-            self.assert_(sha_ids == [sha1_id])
-
-            sha_ids = self.sha.search([
-                ('sha', 'like', '640a%'),
+                ('sha', 'like', 'Test'),
                 ])
             self.assert_(sha_ids == [sha1_id])
 
@@ -1415,17 +1425,7 @@ class FieldsTestCase(unittest.TestCase):
             self.assert_(sha_ids == [])
 
             sha_ids = self.sha.search([
-                ('sha', 'like', 'F%'),
-                ])
-            self.assert_(sha_ids == [])
-
-            sha_ids = self.sha.search([
-                ('sha', 'ilike', '640AB2BAE07BEDC4C163F679A746F7AB7FB5D1FA'),
-                ])
-            self.assert_(sha_ids == [sha1_id])
-
-            sha_ids = self.sha.search([
-                ('sha', 'ilike', '640A%'),
+                ('sha', 'ilike', 'Test'),
                 ])
             self.assert_(sha_ids == [sha1_id])
 
@@ -1435,17 +1435,7 @@ class FieldsTestCase(unittest.TestCase):
             self.assert_(sha_ids == [])
 
             sha_ids = self.sha.search([
-                ('sha', 'ilike', 'f%'),
-                ])
-            self.assert_(sha_ids == [])
-
-            sha_ids = self.sha.search([
-                ('sha', 'not like', '640ab2bae07bedc4c163f679a746f7ab7fb5d1fa'),
-                ])
-            self.assert_(sha_ids == [])
-
-            sha_ids = self.sha.search([
-                ('sha', 'not like', '640a%'),
+                ('sha', 'not like', 'Test'),
                 ])
             self.assert_(sha_ids == [])
 
@@ -1454,33 +1444,13 @@ class FieldsTestCase(unittest.TestCase):
                 ])
             self.assert_(sha_ids == [sha1_id])
 
-            sha_ids = self.sha.search([
-                ('sha', 'not like', 'F%'),
-                ])
-            self.assert_(sha_ids == [sha1_id])
-
-            sha_ids = self.sha.search([
-                ('sha', 'not ilike', '640AB2BAE07BEDC4C163F679A746F7AB7FB5D1FA'),
-                ])
-            self.assert_(sha_ids == [])
-
-            sha_ids = self.sha.search([
-                ('sha', 'not ilike', '640A%'),
-                ])
-            self.assert_(sha_ids == [])
-
             sha_ids = self.sha.search([
                 ('sha', 'not ilike', 'foo'),
                 ])
             self.assert_(sha_ids == [sha1_id])
 
-            sha_ids = self.sha.search([
-                ('sha', 'not ilike', 'f%'),
-                ])
-            self.assert_(sha_ids == [sha1_id])
-
             sha2_id = self.sha.create({
-                'sha': False,
+                'sha': None,
                 })
             self.assert_(sha2_id)
 
@@ -1512,10 +1482,11 @@ class FieldsTestCase(unittest.TestCase):
             self.assert_(sha4_id)
 
             sha4 = self.sha_default.read(sha4_id, ['sha'])
-            self.assert_(sha4['sha'] == 'ba79baeb9f10896a46ae74715271b7f586e74640')
+            self.assertEqual(sha4['sha'],
+                'ba79baeb9f10896a46ae74715271b7f586e74640')
 
             self.sha.write(sha1_id, {
-                'sha': False,
+                'sha': None,
                 })
             sha1 = self.sha.read(sha1_id, ['sha'])
             self.assert_(sha1['sha'] == None)
@@ -1524,7 +1495,8 @@ class FieldsTestCase(unittest.TestCase):
                 'sha': 'Test',
                 })
             sha2 = self.sha.read(sha2_id, ['sha'])
-            self.assert_(sha2['sha'] == '640ab2bae07bedc4c163f679a746f7ab7fb5d1fa')
+            self.assertEqual(sha2['sha'],
+                '640ab2bae07bedc4c163f679a746f7ab7fb5d1fa')
 
             self.failUnlessRaises(Exception, self.sha_required.create, {})
             transaction.cursor.rollback()
@@ -1811,7 +1783,7 @@ class FieldsTestCase(unittest.TestCase):
             self.assert_(date7_id)
 
             date8_id = self.date.create({
-                'date': False,
+                'date': None,
                 })
             self.assert_(date8_id)
 
@@ -2059,8 +2031,8 @@ class FieldsTestCase(unittest.TestCase):
                 })
             self.assert_(datetime5_id)
             datetime5 = self.datetime.read(datetime5_id, ['datetime'])
-            self.assert_(datetime5['datetime'] == datetime.datetime(2009, 1, 1, 12,
-                0, 0))
+            self.assertEqual(datetime5['datetime'],
+                datetime.datetime(2009, 1, 1, 12, 0, 0))
 
             self.failUnlessRaises(Exception, self.datetime_required.create, {})
             transaction.cursor.rollback()
@@ -2076,7 +2048,7 @@ class FieldsTestCase(unittest.TestCase):
             self.assert_(datetime7_id)
 
             datetime8_id = self.datetime.create({
-                'datetime': False,
+                'datetime': None,
                 })
             self.assert_(datetime8_id)
 
@@ -2089,7 +2061,260 @@ class FieldsTestCase(unittest.TestCase):
 
             transaction.cursor.rollback()
 
-    def test0100one2one(self):
+    def test0100time(self):
+        '''
+        Test Time.
+        '''
+        with Transaction().start(DB_NAME, USER,
+                context=CONTEXT) as transaction:
+            pre_evening = datetime.time(16, 30)
+            evening = datetime.time(18, 45, 3)
+            night = datetime.time(20, 00)
+            default_time = datetime.time(16, 30)
+
+            time1_id = self.time.create({
+                    'time': evening,
+                    })
+            self.assert_(time1_id)
+
+            time1 = self.time.read(time1_id, ['time'])
+            self.assertEqual(time1['time'], evening)
+
+            time_ids = self.time.search([
+                ('time', '=', evening),
+                ])
+            self.assert_(time_ids == [time1_id])
+
+            time_ids = self.time.search([
+                ('time', '=', night),
+                ])
+            self.assert_(time_ids == [])
+
+            time_ids = self.time.search([
+                ('time', '=', False),
+                ])
+            self.assert_(time_ids == [])
+
+            time_ids = self.time.search([
+                ('time', '!=', evening),
+                ])
+            self.assert_(time_ids == [])
+
+            time_ids = self.time.search([
+                ('time', '!=', night),
+                ])
+            self.assert_(time_ids == [time1_id])
+
+            time_ids = self.time.search([
+                ('time', '!=', False),
+                ])
+            self.assert_(time_ids == [time1_id])
+
+            time_ids = self.time.search([
+                ('time', 'in', [evening]),
+                ])
+            self.assert_(time_ids == [time1_id])
+
+            time_ids = self.time.search([
+                ('time', 'in', [night]),
+                ])
+            self.assert_(time_ids == [])
+
+            time_ids = self.time.search([
+                ('time', 'in', [False]),
+                ])
+            self.assert_(time_ids == [])
+
+            time_ids = self.time.search([
+                ('time', 'in', []),
+                ])
+            self.assert_(time_ids == [])
+
+            time_ids = self.time.search([
+                ('time', 'not in', [evening]),
+                ])
+            self.assert_(time_ids == [])
+
+            time_ids = self.time.search([
+                ('time', 'not in', [night]),
+                ])
+            self.assert_(time_ids == [time1_id])
+
+            time_ids = self.time.search([
+                ('time', 'not in', [False]),
+                ])
+            self.assert_(time_ids == [time1_id])
+
+            time_ids = self.time.search([
+                ('time', 'not in', []),
+                ])
+            self.assert_(time_ids == [time1_id])
+
+            time_ids = self.time.search([
+                ('time', '<', night),
+                ])
+            self.assert_(time_ids == [time1_id])
+
+            time_ids = self.time.search([
+                ('time', '<', pre_evening),
+                ])
+            self.assert_(time_ids == [])
+
+            time_ids = self.time.search([
+                ('time', '<', evening),
+                ])
+            self.assert_(time_ids == [])
+
+            time_ids = self.time.search([
+                ('time', '<=', evening),
+                ])
+            self.assert_(time_ids == [time1_id])
+
+            time_ids = self.time.search([
+                ('time', '<=', pre_evening),
+                ])
+            self.assert_(time_ids == [])
+
+            time_ids = self.time.search([
+                ('time', '<=', night),
+                ])
+            self.assert_(time_ids == [time1_id])
+
+            time_ids = self.time.search([
+                ('time', '>', night),
+                ])
+            self.assert_(time_ids == [])
+
+            time_ids = self.time.search([
+                ('time', '>', pre_evening),
+                ])
+            self.assert_(time_ids == [time1_id])
+
+            time_ids = self.time.search([
+                ('time', '>', evening),
+                ])
+            self.assert_(time_ids == [])
+
+            time_ids = self.time.search([
+                ('time', '>=', night),
+                ])
+            self.assert_(time_ids == [])
+
+            time_ids = self.time.search([
+                ('time', '>=', pre_evening),
+                ])
+            self.assert_(time_ids == [time1_id])
+
+            time_ids = self.time.search([
+                ('time', '>=', evening),
+                ])
+            self.assert_(time_ids == [time1_id])
+
+            time2_id = self.time.create({
+                'time': pre_evening,
+                })
+            self.assert_(time2_id)
+
+            time2 = self.time.read(time2_id, ['time'])
+            self.assert_(time2['time'] == pre_evening)
+
+            time_ids = self.time.search([
+                ('time', '=', pre_evening),
+                ])
+            self.assert_(time_ids == [time2_id])
+
+            time_ids = self.time.search([
+                ('time', 'in', [pre_evening, evening]),
+                ])
+            self.assert_(time_ids == [time1_id, time2_id])
+
+            time_ids = self.time.search([
+                ('time', 'not in', [pre_evening, evening]),
+                ])
+            self.assert_(time_ids == [])
+
+            time3_id = self.time.create({})
+            self.assert_(time3_id)
+
+            time3 = self.time.read(time3_id, ['time'])
+            self.assert_(time3['time'] == None)
+
+            time4_id = self.time_default.create({})
+            self.assert_(time4_id)
+
+            time4 = self.time_default.read(time4_id, ['time'])
+            self.assert_(time4['time'] == default_time)
+
+            self.time.write(time1_id, {
+                'time': pre_evening,
+                })
+            time1 = self.time.read(time1_id, ['time'])
+            self.assert_(time1['time'] == pre_evening)
+
+            self.time.write(time2_id, {
+                'time': evening,
+                })
+            time2 = self.time.read(time2_id, ['time'])
+            self.assert_(time2['time'] == evening)
+
+            self.failUnlessRaises(Exception, self.time.create, {
+                    'time': 'test',
+                    })
+
+            self.failUnlessRaises(Exception, self.time.write, time1_id,
+                {
+                    'time': 'test',
+                    })
+
+            self.failUnlessRaises(Exception, self.time.create, {
+                    'time': 1,
+                    })
+
+            self.failUnlessRaises(Exception, self.time.write, time1_id,
+                {
+                    'time': 1,
+                    })
+
+            self.failUnlessRaises(Exception, self.time.write, time1_id,
+                {
+                    'time': '25:00:00',
+                    })
+
+            time5_id = self.time.create({
+                'time': '12:00:00',
+                })
+            self.assert_(time5_id)
+            time5 = self.time.read(time5_id, ['time'])
+            self.assert_(time5['time'] == datetime.time(12, 0))
+
+            self.failUnlessRaises(Exception, self.time_required.create, {})
+            transaction.cursor.rollback()
+
+            time6_id = self.time_required.create({
+                'time': evening,
+                })
+            self.assert_(time6_id)
+
+            time7_id = self.time.create({
+                'time': None,
+                })
+            self.assert_(time7_id)
+
+            time8_id = self.time.create({
+                'time': False,
+                })
+            self.assert_(time8_id)
+
+            time9_id = self.time.create({
+                'time': evening.replace(microsecond=1),
+                })
+            self.assert_(time9_id)
+            time9 = self.time.read(time9_id, ['time'])
+            self.assert_(time9['time'] == evening)
+
+            transaction.cursor.rollback()
+
+    def test0110one2one(self):
         '''
         Test One2One.
         '''
@@ -2214,6 +2439,284 @@ class FieldsTestCase(unittest.TestCase):
 
             transaction.cursor.rollback()
 
+    def test0110property(self):
+        '''
+        Test Property with supported field types.
+        '''
+        with Transaction().start(DB_NAME, USER,
+                context=CONTEXT) as transaction:
+
+            # Test Char
+            prop_id_a = self.property_.create({'char': 'Test'})
+            self.assert_(prop_id_a)
+
+            prop_id_b = self.property_.create({})
+            self.assert_(prop_id_b)
+
+            prop_id_c = self.property_.create({'char': 'FooBar'})
+            self.assert_(prop_id_c)
+
+            prop_a = self.property_.read(prop_id_a, ['char'])
+            self.assert_(prop_a['char'] == 'Test')
+
+            prop_ids = self.property_.search([('char', '=', 'Test')])
+            self.assert_(prop_ids == [prop_id_a])
+
+            prop_ids = self.property_.search([('char', '=', False)])
+            self.assert_(prop_ids == [prop_id_b])
+
+            prop_ids = self.property_.search([('char', '!=', False)])
+            self.assert_(prop_ids == [prop_id_a, prop_id_c])
+
+            prop_ids = self.property_.search([('char', 'like', 'Tes%')])
+            self.assert_(prop_ids == [prop_id_a])
+
+            prop_ids = self.property_.search([('char', 'like', '%Bar')])
+            self.assert_(prop_ids == [prop_id_c])
+
+            prop_ids = self.property_.search([('char', 'not like', 'Tes%')])
+            self.assert_(prop_ids == [prop_id_b, prop_id_c])
+
+            prop_ids = self.property_.search([('char', 'ilike', 'tes%')])
+            self.assert_(prop_ids == [prop_id_a])
+
+            prop_ids = self.property_.search([('char', 'ilike', '%bar')])
+            self.assert_(prop_ids == [prop_id_c])
+
+            prop_ids = self.property_.search([('char', 'not ilike', 'tes%')])
+            self.assert_(prop_ids == [prop_id_b, prop_id_c])
+
+            prop_ids = self.property_.search([('char', 'in', ['Test'])])
+            self.assert_(prop_ids == [prop_id_a])
+
+            prop_ids = self.property_.search([
+                    ('char', 'in', ['Test', 'FooBar'])])
+            self.assert_(prop_ids == [prop_id_a, prop_id_c])
+
+            prop_ids = self.property_.search([
+                    ('char', 'not in', ['Test', 'FooBar'])])
+            self.assert_(prop_ids == [prop_id_b])
+
+            model_field_obj = POOL.get('ir.model.field')
+            property_obj = POOL.get('ir.property')
+
+            # Test default value
+            property_field_id, = model_field_obj.search([
+                        ('model.model', '=', 'test.property'),
+                        ('name', '=', 'char'),
+                    ], limit=1)
+            property_obj.create({
+                        'field': property_field_id,
+                        'value': ',DEFAULT_VALUE',
+                    })
+
+            prop_id_d = self.property_.create({})
+            self.assert_(prop_id_d)
+
+            prop = self.property_.read(prop_id_d, ['char'])
+            self.assert_(prop['char'] == 'DEFAULT_VALUE')
+
+            prop_ids = self.property_.search([('char', '!=', False)])
+            self.assert_(prop_ids == [prop_id_a, prop_id_c, prop_id_d])
+
+            self.property_.write(prop_id_a, {'char': None})
+            prop_a = self.property_.read(prop_id_a, ['char'])
+            self.assert_(prop_a['char'] == None)
+
+            self.property_.write(prop_id_b, {'char': 'Test'})
+            prop_b = self.property_.read(prop_id_b, ['char'])
+            self.assert_(prop_b['char'] == 'Test')
+
+            transaction.cursor.rollback()
+
+            # Test Many2One
+            char_id_a = self.char.create({'char': 'Test'})
+            self.assert_(char_id_a)
+
+            char_id_b = self.char.create({'char': 'FooBar'})
+            self.assert_(char_id_b)
+
+            prop_id_a = self.property_.create({'many2one': char_id_a})
+            self.assert_(prop_id_a)
+
+            prop_id_b = self.property_.create({'many2one': char_id_b})
+            self.assert_(prop_id_b)
+
+            prop_id_c = self.property_.create({})
+            self.assert_(prop_id_c)
+
+            prop_ids = self.property_.search([('many2one', '=', char_id_a)])
+            self.assert_(prop_ids == [prop_id_a])
+
+            prop_ids = self.property_.search([('many2one', '!=', False)])
+            self.assert_(prop_ids == [prop_id_a, prop_id_b])
+
+            prop_ids = self.property_.search([('many2one', '=', False)])
+            self.assert_(prop_ids == [prop_id_c])
+
+            prop_a = self.property_.read(prop_id_a, ['many2one'])
+            self.assert_(prop_a['many2one'] == char_id_a)
+
+            prop_ids = self.property_.search([
+                    ('many2one', 'in', [char_id_a, char_id_b])])
+            self.assert_(prop_ids == [prop_id_a, prop_id_b])
+
+            prop_ids = self.property_.search([
+                    ('many2one', 'not in', [char_id_a, char_id_b])])
+            self.assert_(prop_ids == [prop_id_c])
+
+            self.property_.write(prop_id_b, {'many2one': char_id_a})
+            prop_b = self.property_.read(prop_id_b, ['many2one'])
+            self.assert_(prop_b['many2one'] == char_id_a)
+
+            transaction.cursor.rollback()
+
+            # Test Numeric
+            prop_id_a = self.property_.create({'numeric': Decimal('1.1')})
+            self.assert_(prop_id_a)
+
+            prop_id_b = self.property_.create({'numeric': Decimal('2.6')})
+            self.assert_(prop_id_b)
+
+            prop_id_c = self.property_.create({})
+            self.assert_(prop_id_c)
+
+            prop_ids = self.property_.search([('numeric', '!=', False)])
+            self.assert_(prop_ids == [prop_id_a, prop_id_b])
+
+            prop_ids = self.property_.search([('numeric', '=', False)])
+            self.assert_(prop_ids == [prop_id_c])
+
+            prop_ids = self.property_.search([
+                    ('numeric', '=', Decimal('1.1')),
+                    ])
+            self.assert_(prop_ids == [prop_id_a])
+
+            prop_ids = self.property_.search([
+                    ('numeric', '!=', Decimal('1.1'))])
+            self.assert_(prop_ids == [prop_id_b, prop_id_c])
+
+            prop_ids = self.property_.search([
+                    ('numeric', '<', Decimal('2.6')),
+                    ])
+            self.assert_(prop_ids == [prop_id_a])
+
+            prop_ids = self.property_.search([
+                    ('numeric', '<=', Decimal('2.6'))])
+            self.assert_(prop_ids == [prop_id_a, prop_id_b])
+
+            prop_ids = self.property_.search([
+                    ('numeric', '>', Decimal('1.1')),
+                    ])
+            self.assert_(prop_ids == [prop_id_b])
+
+            prop_ids = self.property_.search([
+                    ('numeric', '>=', Decimal('1.1'))])
+            self.assert_(prop_ids == [prop_id_a, prop_id_b])
+
+            prop_ids = self.property_.search([
+                    ('numeric', 'in', [Decimal('1.1')])])
+            self.assert_(prop_ids == [prop_id_a])
+
+            prop_ids = self.property_.search([
+                    ('numeric', 'in', [Decimal('1.1'), Decimal('2.6')])])
+            self.assert_(prop_ids == [prop_id_a, prop_id_b])
+
+            prop_ids = self.property_.search([
+                    ('numeric', 'not in', [Decimal('1.1')])])
+            self.assert_(prop_ids == [prop_id_b, prop_id_c])
+
+            prop_ids = self.property_.search([
+                    ('numeric', 'not in', [Decimal('1.1'), Decimal('2.6')])])
+            self.assert_(prop_ids == [prop_id_c])
+
+            # Test default value
+            property_field_id, = model_field_obj.search([
+                        ('model.model', '=', 'test.property'),
+                        ('name', '=', 'numeric'),
+                    ], limit=1)
+            property_obj.create({
+                        'field': property_field_id,
+                        'value': ',3.7',
+                    })
+
+            prop_id_d = self.property_.create({})
+            self.assert_(prop_id_d)
+
+            prop_d = self.property_.read(prop_id_d, ['numeric'])
+            self.assert_(prop_d['numeric'] == Decimal('3.7'))
+
+            self.property_.write(prop_id_a, {'numeric': None})
+            prop_a = self.property_.read(prop_id_a, ['numeric'])
+            self.assert_(prop_a['numeric'] == None)
+
+            self.property_.write(prop_id_b, {'numeric': Decimal('3.11')})
+            prop_b = self.property_.read(prop_id_b, ['numeric'])
+            self.assert_(prop_b['numeric'] == Decimal('3.11'))
+
+            transaction.cursor.rollback()
+
+            # Test Selection
+            prop_id_a = self.property_.create({'selection': 'option_a'})
+            self.assert_(prop_id_a)
+
+            prop_id_b = self.property_.create({'selection': 'option_b'})
+            self.assert_(prop_id_b)
+
+            prop_id_c = self.property_.create({})
+            self.assert_(prop_id_c)
+
+            prop_ids = self.property_.search([('selection', '=', 'option_a')])
+            self.assert_(prop_ids == [prop_id_a])
+
+            prop_ids = self.property_.search([('selection', '!=', False)])
+            self.assert_(prop_ids == [prop_id_a, prop_id_b])
+
+            prop_ids = self.property_.search([('selection', '=', False)])
+            self.assert_(prop_ids == [prop_id_c])
+
+            prop_ids = self.property_.search([('selection', '!=', 'option_a')])
+            self.assert_(prop_ids == [prop_id_b, prop_id_c])
+
+            prop_ids = self.property_.search([
+                    ('selection', 'in', ['option_a'])])
+            self.assert_(prop_ids == [prop_id_a])
+
+            prop_ids = self.property_.search([
+                    ('selection', 'in', ['option_a', 'option_b'])])
+            self.assert_(prop_ids == [prop_id_a, prop_id_b])
+
+            prop_ids = self.property_.search([
+                    ('selection', 'not in', ['option_a'])])
+            self.assert_(prop_ids == [prop_id_b, prop_id_c])
+
+            # Test default value
+            property_field_id, = model_field_obj.search([
+                        ('model.model', '=', 'test.property'),
+                        ('name', '=', 'selection'),
+                    ], limit=1)
+            property_obj.create({
+                        'field': property_field_id,
+                        'value': ',option_a',
+                    })
+
+            prop_id_d = self.property_.create({})
+            self.assert_(prop_id_d)
+
+            prop_d = self.property_.read(prop_id_d, ['selection'])
+            self.assert_(prop_d['selection'] == 'option_a')
+
+            self.property_.write(prop_id_a, {'selection': None})
+            prop_a = self.property_.read(prop_id_a, ['selection'])
+            self.assert_(prop_a['selection'] == None)
+
+            self.property_.write(prop_id_c, {'selection': 'option_b'})
+            prop_c = self.property_.read(prop_id_c, ['selection'])
+            self.assert_(prop_c['selection'] == 'option_b')
+
+            transaction.cursor.rollback()
+
+
 def suite():
     return unittest.TestLoader().loadTestsFromTestCase(FieldsTestCase)
 
