@@ -3,7 +3,9 @@
 {
     'name': 'WebDAV',
     'name_bg_BG': 'WebDAV',
+    'name_ca_ES': 'WebDAV',
     'name_de_DE': 'WebDAV',
+    'name_es_AR': 'WebDAV',
     'name_es_ES': 'WebDAV',
     'name_fr_FR': 'WebDAV',
     'name_nl_NL': 'WebDAV',
@@ -11,10 +13,14 @@
     'description': '''Basic module providing concept and administration of integrated WebDAV features.
 ''',
     'description_bg_BG': '''Общ модул предоставящ представа и администрация на интегрирани WebDAV услуги.
+''',
+    'description_ca_ES': '''Mòdul bàsic que proporciona el concepte i l'administració de serveis WebDAV integrats.
 ''',
     'description_de_DE': '''Basismodul für Konzept und Administration der integrierten WebDAV-Komponente
 ''',
-    'description_es_ES': '''Módulo basico que provee el concepto y la administración de servicios WebDAV integrados.
+    'description_es_AR': '''Módulo básico que provee el concepto y la administración de servicios WebDAV integrados.
+''',
+    'description_es_ES': '''Módulo básico que proporciona el concepto y la administración de servicios WebDAV integrados.
 ''',
     'description_fr_FR': '''Module de base fournissant les concepts et l'administration des fonctionnalités integrées de WebDAV.
 ''',
@@ -33,8 +39,10 @@
     'translation': [
         'locale/cs_CZ.po',
         'locale/bg_BG.po',
+        'locale/ca_ES.po',
         'locale/fr_FR.po',
         'locale/de_DE.po',
+        'locale/es_AR.po',
         'locale/es_ES.po',
         'locale/es_CO.po',
         'locale/nl_NL.po',
